@@ -30,13 +30,13 @@
 #include <kfinddialog.h>
 #include <kglobalsettings.h>
 #include <kiconloader.h>
+#include <klistviewsearchline.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kprocess.h>
 #include <krfcdate.h>
 #include <krun.h>
 #include <kurl.h>
-#include <kdeversion.h>
 
 #include "cvsservice_stub.h"
 #include "annotatedlg.h"
@@ -55,7 +55,7 @@ LogDialog::LogDialog(KConfig& cfg, QWidget *parent, const char *name)
     : KDialogBase(parent, name, false, QString::null,
                   Ok | Apply | Close | Help | User1 | User2 | User3, Close, true,
                   KGuiItem(i18n("&Annotate")),
-                  KGuiItem(i18n("&Diff")),
+                  KGuiItem(i18n("&Diff"), "vcs_diff"),
                   KGuiItem(i18n("&Find..."), "find"))
     , cvsService(0)
     , partConfig(cfg)
@@ -68,7 +68,20 @@ LogDialog::LogDialog(KConfig& cfg, QWidget *parent, const char *name)
     connect( tree, SIGNAL(revisionClicked(QString,bool)),
              this, SLOT(revisionSelected(QString,bool)) );
 
-    list = new LogListView(partConfig, mainWidget);
+    QWidget* listWidget = new QWidget(mainWidget);
+    QVBoxLayout* listLayout = new QVBoxLayout(listWidget);
+    QHBoxLayout* searchLayout = new QHBoxLayout(listLayout);
+    searchLayout->setMargin(KDialog::spacingHint());
+    searchLayout->setSpacing(KDialog::spacingHint());
+
+    list = new LogListView(partConfig, listWidget);
+    listLayout->addWidget(list, 1);
+
+    KListViewSearchLine* searchLine = new KListViewSearchLine(listWidget, list);
+    QLabel* searchLabel = new QLabel(searchLine, i18n("S&earch:"), listWidget);
+    searchLayout->addWidget(searchLabel);
+    searchLayout->addWidget(searchLine, 1);
+
     connect( list, SIGNAL(revisionClicked(QString,bool)),
              this, SLOT(revisionSelected(QString,bool)) );
 
@@ -78,7 +91,7 @@ LogDialog::LogDialog(KConfig& cfg, QWidget *parent, const char *name)
 
     tabWidget = new QTabWidget(mainWidget);
     tabWidget->addTab(tree, i18n("&Tree"));
-    tabWidget->addTab(list, i18n("&List"));
+    tabWidget->addTab(listWidget, i18n("&List"));
     tabWidget->addTab(plain, i18n("CVS &Output"));
     layout->addWidget(tabWidget, 3);
 
@@ -172,16 +185,8 @@ LogDialog::LogDialog(KConfig& cfg, QWidget *parent, const char *name)
     connect( this, SIGNAL(user3Clicked()),
              this, SLOT(findClicked()) );
 
-#if KDE_IS_VERSION(3,2,90)
-    setButtonGuiItem(Ok, KGuiItem(i18n("to view something", "&View"),"fileopen","",""));
-#else
-    if( KGlobalSettings::showIconsOnPushButtons() )
-      actionButton(Ok)->setIconSet(SmallIcon(QString::fromLatin1("fileopen")));
-    setButtonText(Ok, i18n("to view something", "&View"));
-    setButtonTip(Ok, QString::null);
-    setButtonWhatsThis(Ok, QString::null);
-#endif
-    setButtonGuiItem(Apply, KGuiItem(i18n("Create Patch..."), "", "", ""));
+    setButtonGuiItem(Ok, KGuiItem(i18n("to view something", "&View"),"fileopen"));
+    setButtonGuiItem(Apply, KGuiItem(i18n("Create Patch...")));
     setHelp("browsinglogs");
 
     setWFlags(Qt::WDestructiveClose | getWFlags());
