@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1999-2002 Bernd Gehrmann <bernd@mail.berlios.de>
- * Copyright (c) 2003-2004 Andr Wbbeking <Woebbeking@web.de>
+ * Copyright (c) 2003-2005 Andr Wbbeking <Woebbeking@web.de>
  *
  * This program may be distributed under the terms of the Q Public
  * License as defined by Trolltech AS of Norway and appearing in the
@@ -20,6 +20,7 @@
 #include <qpainter.h>
 
 #include <kdebug.h>
+#include <kglobalsettings.h>
 #include <kiconloader.h>
 #include <klocale.h>
 
@@ -678,6 +679,8 @@ void UpdateFileItem::paintCell(QPainter *p,
         color = view->remoteChangeColor();
         break;
     case Cervisia::NotInCVS:
+        color = view->notInCvsColor();
+        break;
     case Cervisia::Unknown:
     case Cervisia::UpToDate:
         break;
@@ -685,7 +688,7 @@ void UpdateFileItem::paintCell(QPainter *p,
 
     const QFont oldFont(p->font());
     QColorGroup mycg(cg);
-    if (color.isValid())
+    if (color.isValid() && color != KGlobalSettings::textColor())
     {
         QFont myFont(oldFont);
         myFont.setBold(true);
