@@ -3,7 +3,7 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
-		2001-2003 by Stanislav Visnovsky <visnovsky@kde.org>
+		2001-2004 by Stanislav Visnovsky <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,6 +19,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+
 **************************************************************************** */
 
 #include "catmanresource.h"
@@ -28,7 +39,9 @@
 #include "findinfilesdialog.h"
 #include "kbabeldictbox.h"
 #include "resources.h"
-#include "catalogmanagerpref.h"
+#include "projectpref.h"
+#include "kbprojectmanager.h"
+#include "projectwizard.h"
 #include "msgfmt.h"
 #include "toolaction.h"
 
@@ -43,12 +56,12 @@
 #include <kcursor.h>
 #include <kdatatool.h>
 #include <kdialogbase.h>
-#include <kedittoolbar.h>
+//#include <kedittoolbar.h>
+#include <kfiledialog.h>
 #include <kglobal.h>
 #include <kglobalsettings.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
-#include <kkeydialog.h>
 #include <klocale.h>
 #include <kpopupmenu.h>
 #include <kprogress.h>
@@ -79,22 +92,11 @@ QStringList CatalogManager::_toBeSearched;
 CatalogManager::CatalogManager(QString configFile )
                  :KMainWindow(0,0)
 {
-   if( configFile.isEmpty() ) _configFile = "kbabelrc";
-   else _configFile = configFile;
+   _configFile = configFile;
 
    init();
-   restoreSettings(_configFile);
-   setSettings(_settings);
-}
-
-CatalogManager::CatalogManager(CatManSettings settings)
-                 :KMainWindow(0,0)
-{
-   _configFile = "kbabelrc";
-
-   init();
-   _settings = settings;
-   setSettings(settings);
+   restoreSettings();
+   updateSettings();
 }
 
 CatalogManager::~CatalogManager()
@@ -118,17 +120,30 @@ void CatalogManager::init()
    _findDialog=0;
    _replaceDialog=0;
 
+   _project = KBabel::ProjectManager::open(_configFile);
+   
+   if ( _project == NULL )
+   {
+	KMessageBox::error( this, i18n("Cannot open project file\n%1").arg(_configFile)
+	    , i18n("Project File Error"));
+	    
+	_project = KBabel::ProjectManager::open(KBABEL_DEFAULTPROJECT);
+   }
+   
+   connect( _project, SIGNAL (signalCatManSettingsChanged())
+	, this, SLOT (updateSettings()));
+
    QWidget *view = new QWidget(this);
    QVBoxLayout* layout= new QVBoxLayout(view);
    layout->setMargin(0);
    layout->setSpacing(KDialog::spacingHint());
 
-   _catalogManager=new CatalogManagerView(_configFile, view,"catalog manager");
+   _catalogManager=new CatalogManagerView(_project, view,"catalog manager");
    layout->addWidget(_catalogManager);
    layout->setStretchFactor(_catalogManager,1);
 
-   connect(_catalogManager,SIGNAL(settingsChanged(CatManSettings))
-            ,this,SIGNAL(settingsChanged(CatManSettings)));
+   connect(this,SIGNAL(settingsChanged(KBabel::CatManSettings))
+            ,_catalogManager,SLOT(setSettings(KBabel::CatManSettings)));
    connect(_catalogManager,SIGNAL(openFile(QString,QString))
            ,this,SLOT(openFile(QString,QString)));
    connect(_catalogManager,SIGNAL(openFileInNewWindow(QString,QString))
@@ -141,7 +156,7 @@ void CatalogManager::init()
            ,this,SLOT(openFile(QString,QString,int)));
    connect(_catalogManager, SIGNAL(selectedChanged(uint)),
            this, SLOT(selectedChanged(uint)));
-
+	   
    KWin::setIcons(winId(),BarIcon("catalogmanager",32)
            ,SmallIcon("catalogmanager"));
 
@@ -161,8 +176,10 @@ void CatalogManager::init()
            , this, SLOT(clearProgressBar()));
    connect(_catalogManager,SIGNAL(progress(int))
            , _progressBar, SLOT(setProgress(int)));
+//   connect(_catalogManager, SIGNAL(signalBuildTree(bool))
+//	   , this, SLOT(enableMenuForFiles(bool)));
    connect(_catalogManager, SIGNAL(signalBuildTree(bool))
-	   , this, SLOT(enableMenuForFiles(bool)));
+	   , this, SLOT(enableActions(bool)));
    connect(this, SIGNAL(searchStopped())
 	   , _catalogManager, SLOT(stopSearch()));
    connect(_catalogManager, SIGNAL(prepareFindProgressBar(int))
@@ -171,8 +188,9 @@ void CatalogManager::init()
    setCentralWidget(view);
    resize( 600,300);
 
-   setupActions();
    setupStatusBar();
+   setupActions();
+
 
    QPopupMenu* popup;
    popup = (QPopupMenu*)(factory()->container("rmb_file", this));
@@ -240,6 +258,10 @@ void CatalogManager::setupActions()
     action = new KAction( i18n("Remove All Markings"), 0, _catalogManager,
                           SLOT(clearAllMarks()), actionCollection(), "remove_all_marking");
     action->setEnabled(false);
+    action = new KAction( i18n("Mark Modified Files"), 0, _catalogManager,
+                          SLOT(markModifiedFiles()), actionCollection(), "mark_modified_files");
+    // fixme to enabling this when loading is done using updateFinished() signal
+    action->setEnabled(true);
     action = new KAction( i18n("&Load Markings..."), 0, _catalogManager,
                           SLOT(loadMarks()), actionCollection(), "load_marking");
     action->setEnabled(false);
@@ -302,6 +324,25 @@ void CatalogManager::setupActions()
                          _catalogManager, SLOT(gotoNextMarked()),actionCollection(), "go_next_marked");
     action->setEnabled(false);
 
+    // project menu
+    // the project menu
+   action = new KAction(i18n("&New..."), "filenew"
+           , this, SLOT(projectNew()),actionCollection()
+           ,"project_new");
+
+   action = new KAction(i18n("&Open..."), "fileopen"
+           , this, SLOT(projectOpen()),actionCollection()
+           ,"project_open");
+
+   action = new KAction(i18n("C&lose"), "fileclose"
+           , this, SLOT(projectClose()),actionCollection()
+           ,"project_close");
+   action->setEnabled (_project->filename() != KBABEL_DEFAULTPROJECT);
+
+   action = new KAction(i18n("&Configure..."), "configure"
+           , this, SLOT(projectConfigure()),actionCollection()
+           ,"project_settings");
+
     // tools menu
     action = new KAction( i18n("&Statistics"), "statistics",  CTRL+Key_S,
                           _catalogManager, SLOT(statistics()), actionCollection(), "statistics");
@@ -331,6 +372,11 @@ void CatalogManager::setupActions()
                           _catalogManager, SLOT(mailMarkedFiles()), actionCollection(),	"mail_file_marked");
     action->setEnabled(false);
 
+    action = new KAction( i18n("&Pack"), "tar", CTRL+Key_B,
+                         _catalogManager, SLOT(packageFiles()), actionCollection(), "package_file");
+    action = new KAction( i18n("Pack &Marked"), "tar", CTRL+ALT+Key_B, _catalogManager, SLOT(packageMarkedFiles()), actionCollection(), "package_file_marked");
+    action->setEnabled(false);
+
     actionMap["statistics_marked"]        = NEEDS_DIR | NEEDS_MARK;
     actionMap["syntax"]                   = NEEDS_PO;
     actionMap["spellcheck"]               = NEEDS_PO;
@@ -338,6 +384,7 @@ void CatalogManager::setupActions()
     actionMap["rough_translation_marked"] = NEEDS_MARK;
     actionMap["mail_file"]                = NEEDS_PO;
     actionMap["mail_file_marked"]         = NEEDS_PO | NEEDS_MARK;
+    actionMap["package_file_marked"]      = NEEDS_PO | NEEDS_MARK;
 
     // dynamic tools
     QValueList<KDataToolInfo> tools = ToolAction::validationTools();
@@ -372,11 +419,11 @@ void CatalogManager::setupActions()
 
     // CVS submenu
     // Actions for PO files
-    (void)new KAction( i18n( "Update" ), 0, _catalogManager,
+    (void)new KAction( i18n( "Update" ), "down", 0, _catalogManager,
       SLOT( cvsUpdate( ) ), actionCollection( ), "cvs_update" );
     (void)new KAction( i18n( "Update Marked" ), 0, _catalogManager,
       SLOT( cvsUpdateMarked( ) ), actionCollection( ), "cvs_update_marked" );
-    (void)new KAction( i18n( "Commit" ), 0, _catalogManager,
+    (void)new KAction( i18n( "Commit" ), "up", 0, _catalogManager,
       SLOT( cvsCommit( ) ), actionCollection( ), "cvs_commit" );
     (void)new KAction( i18n( "Commit Marked" ), 0, _catalogManager,
       SLOT( cvsCommitMarked( ) ), actionCollection( ), "cvs_commit_marked" );
@@ -384,6 +431,8 @@ void CatalogManager::setupActions()
       SLOT( cvsStatus( ) ), actionCollection( ), "cvs_status" );
     (void)new KAction( i18n( "Status for Marked" ), 0, _catalogManager,
       SLOT( cvsStatusMarked( ) ), actionCollection( ), "cvs_status_marked" );
+    (void)new KAction( i18n( "Show Diff" ), 0, _catalogManager,
+      SLOT( cvsDiff( ) ), actionCollection( ), "cvs_diff" );
 
     actionMap["cvs_update"]        = NEEDS_PO | NEEDS_PO_CVS;
     actionMap["cvs_update_marked"] = NEEDS_PO | NEEDS_PO_CVS | NEEDS_MARK;
@@ -391,6 +440,7 @@ void CatalogManager::setupActions()
     actionMap["cvs_commit_marked"] = NEEDS_PO | NEEDS_PO_CVS | NEEDS_MARK;
     actionMap["cvs_status"]        = NEEDS_PO | NEEDS_PO_CVS;
     actionMap["cvs_status_marked"] = NEEDS_PO | NEEDS_PO_CVS | NEEDS_MARK;
+    actionMap["cvs_diff"]          = NEEDS_PO | NEEDS_PO_CVS;
 
     // Actions for POT files
     (void)new KAction( i18n( "Update Templates" ), 0, _catalogManager,
@@ -408,16 +458,9 @@ void CatalogManager::setupActions()
     actionMap["cvs_commit_marked_template"] = NEEDS_POT | NEEDS_POT_CVS | NEEDS_MARK;
 
     // settings menu
-    KStdAction::preferences(this, SLOT( optionsPreferences()), actionCollection());
-    KStdAction::keyBindings(this, SLOT( optionsKeys() ), actionCollection());
-    KStdAction::configureToolbars(this,SLOT(optionsEditToolbars()), actionCollection() );
+    // FIXME: KStdAction::preferences(this, SLOT( optionsPreferences()), actionCollection());
 
-#if KDE_IS_VERSION(3,1,90)
     createStandardStatusBarAction();
-#else
-    KToggleAction* toggle = KStdAction::showStatusbar( this,SLOT( dummySlot() ), actionCollection() );
-    connect( toggle, SIGNAL( toggled(bool) ), this, SLOT( optionsShowStatusbar(bool) ) );
-#endif
 
     setStandardToolBarMenuEnabled ( true );
 
@@ -433,7 +476,11 @@ void CatalogManager::setupActions()
     action = new KAction(i18n("&Delete"),Key_Delete,_catalogManager,SLOT(slotDeleteFile()),actionCollection(), "delete");
     action->setEnabled(false);
 
+#if KDE_IS_VERSION( 3, 2, 90 )
+    setupGUI();
+#else
     createGUI();
+#endif
 }
 
 void CatalogManager::setupStatusBar()
@@ -478,118 +525,31 @@ CatManSettings CatalogManager::settings() const
     return _catalogManager->settings();
 }
 
-void CatalogManager::setSettings(CatManSettings newSettings)
+void CatalogManager::updateSettings()
 {
-    _catalogManager->setSettings(newSettings);
-
-   _openNewWindow=newSettings.openWindow;
+    _settings = _project->catManSettings();
+    _catalogManager->setSettings(_settings);
+   _openNewWindow=_settings.openWindow;
 }
 
 void CatalogManager::saveSettings( QString configFile )
 {
     _settings = _catalogManager->settings(); // restore settings from the view
 
+    _project->setSettings( _settings );
+
     config = new KConfig(configFile);
 
-    KConfigGroupSaver cs(config,"CatalogManager");
-
-    config->writeEntry("PoBaseDir",_settings.poBaseDir);
-    config->writeEntry("PotBaseDir",_settings.potBaseDir);
-    config->writeEntry("OpenWindow",_settings.openWindow);
-
-    config->writeEntry("KillCmdOnExit",_settings.killCmdOnExit);
-    config->writeEntry("IndexWords", _settings.indexWords);
-
-    config->writeEntry("DirCommands",_settings.dirCommands);
-    config->writeEntry("DirCommandNames",_settings.dirCommandNames);
-    config->writeEntry("FileCommands",_settings.fileCommands);
-    config->writeEntry("FileCommandNames",_settings.fileCommandNames);
-
-    config->writeEntry("ValidationIgnoreURL",_settings.ignoreURL);
-
-    config->writeEntry("ShowFlagColumn",_settings.flagColumn);
-    config->writeEntry("ShowFuzzyColumn",_settings.fuzzyColumn);
-    config->writeEntry("ShowUntranslatedColumn",_settings.untranslatedColumn);
-    config->writeEntry("ShowTotalColumn",_settings.totalColumn);
-    config->writeEntry("ShowCVSColumn",_settings.cvsColumn);
-    config->writeEntry("ShowRevisionColumn",_settings.revisionColumn);
-    config->writeEntry("ShowTranslatorColumn",_settings.translatorColumn);
-
     _catalogManager->saveView(config);
 
     config->sync();
 }
 
-void CatalogManager::restoreSettings( QString configFile )
+void CatalogManager::restoreSettings()
 {
-    config = new KConfig(configFile);
-
-    KConfigGroupSaver cs(config,"CatalogManager");
-
-    _settings.poBaseDir=config->readEntry("PoBaseDir"
-               ,::Defaults::CatalogManager::poBaseDir());
-    _settings.potBaseDir=config->readEntry("PotBaseDir"
-	       ,::Defaults::CatalogManager::potBaseDir());
-    _settings.openWindow=config->readBoolEntry("OpenWindow"
-               ,::Defaults::CatalogManager::openWindow);
-
-    _settings.killCmdOnExit=config->readBoolEntry("KillCmdOnExit"
-               ,::Defaults::CatalogManager::killCmdOnExit);
-    _settings.indexWords=config->readBoolEntry("IndexWords"
-               ,::Defaults::CatalogManager::indexWords);
-
-
-    if(config->hasKey("DirCommands"))
-    {
-          _settings.dirCommands = config->readListEntry("DirCommands");
-          _settings.dirCommandNames =
-              config->readListEntry("DirCommandNames");
-    }
-    else
-    {
-          _settings.dirCommands = ::Defaults::CatalogManager::dirCommands();
-          _settings.dirCommandNames =
-              ::Defaults::CatalogManager::dirCommandNames();
-    }
-    if(config->hasKey("FileCommands"))
-    {
-          _settings.fileCommands = config->readListEntry("FileCommands");
-          _settings.fileCommandNames =
-              config->readListEntry("FileCommandNames");
-    }
-    else
-    {
-          _settings.fileCommands=::Defaults::CatalogManager::fileCommands();
-          _settings.fileCommandNames =
-              ::Defaults::CatalogManager::fileCommandNames();
-    }
-
-    _settings.ignoreURL=config->readEntry("ValidationIgnoreURL"
-	, ::Defaults::CatalogManager::ignoreURL());
-
-    _settings.flagColumn=config->readBoolEntry("ShowFlagColumn"
-	, ::Defaults::CatalogManager::flagColumn);
-    _settings.fuzzyColumn=config->readBoolEntry("ShowFuzzyColumn"
-	, ::Defaults::CatalogManager::fuzzyColumn);
-    _settings.untranslatedColumn=config->readBoolEntry("ShowUntranslatedColumn"
-	, ::Defaults::CatalogManager::untranslatedColumn);
-    _settings.totalColumn=config->readBoolEntry("ShoTotalColumn"
-	, ::Defaults::CatalogManager::totalColumn);
-    _settings.cvsColumn=config->readBoolEntry("ShowCVSColumn"
-	, ::Defaults::CatalogManager::cvsColumn);
-    _settings.revisionColumn=config->readBoolEntry("ShowRevisionColumn"
-	, ::Defaults::CatalogManager::revisionColumn);
-    _settings.translatorColumn=config->readBoolEntry("ShowTranslatorColumn"
-	, ::Defaults::CatalogManager::translatorColumn);
-
-    if(!config->hasGroup("PathInfo"))
-    {
-           config->setGroup("PathInfo");
-           config->writeEntry("MessageRoot",_settings.poBaseDir);
-           config->writeEntry("TemplateRoot",_settings.potBaseDir);
-    }
-
-    _catalogManager->restoreView(config);
+    _settings = _project->catManSettings();
+    _openNewWindow=_settings.openWindow;
+    _catalogManager->restoreView(_project->config());
 }
 
 void CatalogManager::setPreferredWindow(WId window)
@@ -640,7 +600,7 @@ void CatalogManager::openFile(QString filename, QString package)
         kdDebug(KBABEL_CATMAN) << callfunc << endl;
 
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
-            KMessageBox::error(this, i18n("Can't send a message to KBabel.\n"
+            KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
     }
 }
@@ -670,7 +630,7 @@ void CatalogManager::openFile(QString filename, QString package, int msgid)
         kdDebug(KBABEL_CATMAN) << callfunc << endl;
 
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
-            KMessageBox::error(this, i18n("Can't send a message to KBabel.\n"
+            KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
     }
 }
@@ -698,7 +658,7 @@ void CatalogManager::openFileInNewWindow(QString filename, QString package)
         }
 
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
-            KMessageBox::error(this, i18n("Can't send a message to KBabel.\n"
+            KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
     }
 }
@@ -725,7 +685,7 @@ void CatalogManager::openTemplate(QString openFilename,QString saveFilename,QStr
         }
 
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
-            KMessageBox::error(this, i18n("Can't send a message to KBabel.\n"
+            KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
     }
 }
@@ -752,7 +712,7 @@ void CatalogManager::openTemplateInNewWindow(QString openFilename,QString saveFi
         }
 
         if( !client->send("kbabel","KBabelIFace", callfunc, data) )
-            KMessageBox::error(this, i18n("Can't send a message to KBabel.\n"
+            KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
     }
 }
@@ -768,7 +728,7 @@ void CatalogManager::spellcheck()
         QDataStream arg(data, IO_WriteOnly);
         arg << fileList;
         if( !client->send("kbabel","KBabelIFace", "spellcheck(QStringList)", data) )
-            KMessageBox::error(this, i18n("Can't send a message to KBabel.\n"
+            KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
     }
 }
@@ -784,7 +744,7 @@ void CatalogManager::markedSpellcheck()
         QDataStream arg(data, IO_WriteOnly);
         arg << fileList;
         if( !client->send("kbabel","KBabelIFace", "spellcheck(QStringList)", data) )
-            KMessageBox::error(this, i18n("Can't send a message to KBabel.\n"
+            KMessageBox::error(this, i18n("Can not send a message to KBabel.\n"
                                           "Please check your installation of KDE."));
     }
 }
@@ -798,7 +758,7 @@ bool CatalogManager::startKBabel()
 
     // find out, if there is a running kbabel
     QCStringList apps = client->registeredApplications();
-    for( QCStringList::Iterator it = apps.begin() ; it != apps.end() ; it++ )
+    for( QCStringList::Iterator it = apps.begin() ; it != apps.end() ; ++it )
     {
         QString clientID = *it;
         if( clientID=="kbabel" )
@@ -848,7 +808,7 @@ void CatalogManager::prepareStatusProgressBar(QString msg, int max)
 {
    _totalFound = 0;
    _foundToBeSent = 0;
-   _statusProgressBar->setRange(0,max);
+   _statusProgressBar->setTotalSteps(max);
    _statusProgressLabel->setText(msg);
    _foundLabel->setText( i18n("Found: 0/0") );
 
@@ -858,7 +818,7 @@ void CatalogManager::prepareStatusProgressBar(QString msg, int max)
 
 void CatalogManager::prepareStatusProgressBar(int max)
 {
-   _statusProgressBar->setRange(0,max);
+   _statusProgressBar->setTotalSteps(max);
 }
 
 void CatalogManager::clearStatusProgressBar()
@@ -906,11 +866,8 @@ void CatalogManager::find()
         _findOptions = _findDialog->findOpts();
 
         // get from options the information for ignoring text parts
-        config = new KConfig( "kbabelrc");
-        config->setGroup( "Misc" );
-        KBabel::Defaults::Misc defaultMisc;
-        _findOptions.contextInfo = config->readEntry( "contextInfo", defaultMisc.contextInfo().pattern() );
-        _findOptions.accelMarker = (config->readEntry( "accelMarker", QString(defaultMisc.accelMarker()) ))[0];
+        _findOptions.contextInfo = QRegExp( _project->miscSettings().contextInfo );
+        _findOptions.accelMarker = _project->miscSettings().accelMarker;
 
         _foundFilesList.clear();
         kdDebug(KBABEL_CATMAN) << "Calling catalogmanagerview::find" << endl;
@@ -940,7 +897,7 @@ void CatalogManager::find()
                 if( !client->send("kbabel","KBabelIFace",
                                   "findInFile(QCString,QCString,QString,int,int,int,int,int,int,int,int,int,int)", data)
                     ) {
-                    KMessageBox::error( this, i18n("DCOP communication with KBabel failed."), i18n("DCOP communication error"));
+                    KMessageBox::error( this, i18n("DCOP communication with KBabel failed."), i18n("DCOP Communication Error"));
                     stopSearching();
                     return;
                 }
@@ -989,11 +946,8 @@ void CatalogManager::replace()
         _findOptions = options;
 
         // get from options the information for ignoring text parts
-        config = new KConfig( "kbabelrc");
-        config->setGroup( "Misc" );
-        KBabel::Defaults::Misc defaultMisc;
-        options.contextInfo = config->readEntry( "contextInfo", defaultMisc.contextInfo().pattern() );
-        options.accelMarker = (config->readEntry( "accelMarker", QString(defaultMisc.accelMarker()) ))[0];
+        options.contextInfo = QRegExp( _project->miscSettings().contextInfo );
+        options.accelMarker = _project->miscSettings().accelMarker;
 
         _foundFilesList.clear();
         QString url = _catalogManager->find(options, _toBeSearched );
@@ -1093,31 +1047,10 @@ void CatalogManager::optionsPreferences()
 {
    if(!_prefDialog)
    {
-      _prefDialog = new CatalogManagerPreferences(_catalogManager->settings());
-      _prefDialog->setHelp("preferences_catalogmanager");
+      _prefDialog = new KBabel::ProjectDialog(_project);
    }
 
-   if(_prefDialog->exec())
-   {
-      CatManSettings settings;
-      _prefDialog->mergeSettings(settings);
-      setSettings(settings);
-      emit settingsChanged(settings);
-   }
-}
-
-void CatalogManager::optionsKeys()
-{
-    KKeyDialog::configure(actionCollection());
-}
-
-
-void CatalogManager::optionsEditToolbars()
-{
-    saveView();
-    KEditToolbar dlg(actionCollection());
-    connect(&dlg,SIGNAL(newToolbarConfig()),this,SLOT(slotNewToolbarConfig()));
-    dlg.exec();
+   _prefDialog->exec();
 }
 
 void CatalogManager::newToolbarConfig()
@@ -1157,4 +1090,186 @@ void CatalogManager::restoreView()
     toggle->setChecked(!statusBar()->isHidden() );
 }
 
+
+void CatalogManager::projectNew()
+{
+    KBabel::Project::Ptr p = KBabel::ProjectWizard::newProject();
+    if( p )
+    {
+	disconnect( _project, SIGNAL (signalCatManSettingsChanged())
+	    , this, SLOT (updateSettings()));
+        _project = p;
+	connect( _project, SIGNAL (signalCatManSettingsChanged())
+	    , this, SLOT (updateSettings()));
+
+	_configFile = _project->filename();
+        restoreSettings();
+	updateSettings();
+        changeProjectActions(p->filename());
+	emit settingsChanged(_settings);
+    }
+}
+
+void CatalogManager::projectOpen()
+{
+    QString oldproject = _project->filename();
+    if( oldproject == KBABEL_DEFAULTPROJECT )
+    {
+        oldproject = "";
+    }
+    QString file = KFileDialog::getOpenFileName(oldproject, i18n("*|All Files"), this);
+    if (file.isEmpty())
+    {
+        return;
+    }
+    KBabel::Project::Ptr p = KBabel::ProjectManager::open(file);
+    if( p )
+    {
+	disconnect( _project, SIGNAL (signalCatManSettingsChanged())
+	    , this, SLOT (updateSettings()));
+        _project = p;
+	connect( _project, SIGNAL (signalCatManSettingsChanged())
+	    , this, SLOT (updateSettings()));
+
+	_configFile = p->filename();
+        restoreSettings();
+	updateSettings();
+        changeProjectActions(file);
+	emit settingsChanged(_settings);
+    }
+    else
+    {
+	KMessageBox::error (this, i18n("Cannot open project file %1").arg(file));
+    }
+}
+
+void CatalogManager::projectClose()
+{
+    disconnect( _project, SIGNAL (signalCatManSettingsChanged())
+	    , this, SLOT (updateSettings()));
+    _project = KBabel::ProjectManager::open(KBABEL_DEFAULTPROJECT);
+    connect( _project, SIGNAL (signalCatManSettingsChanged())
+	    , this, SLOT (updateSettings()));
+    _configFile = _project->filename();
+    restoreSettings();
+    updateSettings();
+    changeProjectActions(KBABEL_DEFAULTPROJECT);
+    emit settingsChanged(_settings);
+}
+
+void CatalogManager::changeProjectActions(const QString& project)
+{
+    bool def = project == KBABEL_DEFAULTPROJECT;
+
+    KAction* saveAction=(KAction*)actionCollection()->action( "project_close" );
+    saveAction->setEnabled( ! def );
+}
+
+void CatalogManager::projectConfigure()
+{
+    KBabel::ProjectDialog* _projectDialog = new ProjectDialog(_project);
+
+    // settings are updated via signals
+    _projectDialog->exec();
+
+    delete _projectDialog;
+}
+
+void CatalogManager::enableActions()
+{
+    enableActions(true);
+}
+
+void CatalogManager::disableActions()
+{
+    enableActions(false);
+}
+
+void CatalogManager::enableActions(bool enable)
+{
+    KAction* action;
+    // the file menu
+    
+    action = (KAction*)actionCollection()->action( "open" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "open_new_window" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "find_in_files" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "replace_in_files" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "reload" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "toggle_marking" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "toggle_all_marking" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "mark_modified_files" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "load_marking" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "save_marking" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_next_untrans" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_prev_untrans" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_next_fuzzy" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_prev_fuzzy" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_next_fuzzyUntr" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_prev_fuzzyUntr" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_next_error" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_prev_error" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_next_template" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_prev_template" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_next_po" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_prev_po" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_next_marked" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "go_prev_marked" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "statistics" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "package_file" );
+    action->setEnabled(enable);
+
+    action = (KAction*)actionCollection()->action( "rough_translation" );
+    action->setEnabled(enable);
+}
+
 #include "catalogmanager.moc"
