@@ -20,6 +20,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+      
 **************************************************************************** */
 
 #include "catmanresource.h"
@@ -128,11 +139,13 @@ bool CatManListItem::marked() const
 
 void CatManListItem::setOpen(bool open)
 {
+  bool needWork = needsWork();
 	QListViewItem::setOpen(open);
 
     if(open && _type==Dir)
     {
-       QPixmap icon = ICON_FOLDER_OPEN;
+       QPixmap icon;
+       icon = needWork ? ICON_FOLDER_OPEN_WORK : ICON_FOLDER_OPEN_OK;
 
        if(!_template.exists())
        {
@@ -367,17 +380,18 @@ void CatManListItem::update(bool showPoInfo,bool includeChildren
              //setSelectable(false);
              _type=Dir;
 
+       bool needWork = needsWork();
 			 QPixmap icon;
 			 if(!isOpen())
 			 {
-			 	if(needsWork())
+			 	if( needWork )
 					icon = ICON_FOLDER_CLOSED_WORK;
 				 else
 				    icon = ICON_FOLDER_CLOSED_OK;
 			 }
 			 else
 			 {
-				icon = ICON_FOLDER_OPEN;
+         icon = needWork ? ICON_FOLDER_OPEN_WORK : ICON_FOLDER_OPEN_OK;
 			 }
 
              // check if the same directory exists also in the
@@ -484,11 +498,11 @@ void CatManListItem::update(bool showPoInfo,bool includeChildren
 	     QPixmap icon;
 	     if(!isOpen())
 	     {
-		icon = ICON_FOLDER_CLOSED_WORK;
+           icon = ICON_FOLDER_CLOSED_WORK;
 	     }
 	     else
 	     {
-		icon = ICON_FOLDER_OPEN;
+         icon = needsWork() ? ICON_FOLDER_OPEN_WORK : ICON_FOLDER_OPEN_OK;
 	     }
 
              setPixmap(COL_NAME, icon );
