@@ -3,7 +3,7 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
-		2001-2003 by Stanislav Visnovsky
+		2001-2005 by Stanislav Visnovsky
 			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+  
 **************************************************************************** */
 #include <qtextstream.h>
 #include <qfile.h>
@@ -43,6 +54,7 @@
 #include <ktrader.h>
 #include <kurl.h>
 
+#include "kbprojectmanager.h"
 #include "catalog.h"
 #include "catalog_private.h"
 #include "catalogitem.h"
@@ -53,35 +65,15 @@
 
 #include "resources.h"
 #include "version.h"
-
-// from libgettext
-extern "C"
-{
-#include "libgettext/fstrcmp.h"
-}
+#include "stringdistance.h"
 
 using namespace KBabel;
 
-Catalog::Catalog(QObject* parent, const char* name, QString configFile)
+Catalog::Catalog(QObject* parent, const char* name, QString projectFile)
         : QObject(parent,name)
 {
-   d = new CatalogPrivate();
-
-   d->_configFile=configFile;
-   KConfig *config;
-   bool bConfigMustBeDeleted = false;
-   if(d->_configFile.isEmpty() )
-   {
-       config = KGlobal::config();
-   }
-   else
-   {
-       config=new KConfig(d->_configFile);
-       bConfigMustBeDeleted = true;
-   }
-   readPreferences(config);
-   if (  bConfigMustBeDeleted )
-       delete config;
+   d = new CatalogPrivate(ProjectManager::open(projectFile));
+   readPreferences();
 }
 
 Catalog::Catalog(const Catalog& c): QObject(c.parent(),c.name()
@@ -135,7 +127,7 @@ QString Catalog::context(uint index) const
     QStringList lines = QStringList::split("\n",c);
 
     QString result;
-    for( QStringList::Iterator it=lines.begin(); it!=lines.end(); it++)
+    for( QStringList::Iterator it=lines.begin(); it!=lines.end(); ++it)
     {
 	if( (*it).startsWith( "#:") )
 	{
@@ -283,7 +275,16 @@ bool Catalog::setHeader(CatalogItem newHeader)
 {
    if(newHeader.isValid())
    {
+      // normalize the values - ensure every key:value pair is only on a single line
+      QString values = newHeader.msgstr().first();
+      values.replace ("\n", "");
+      values.replace ("\\n", "\\n\n");
+      
+      kdDebug () << "Normalized header: " << values << endl;
+      
       d->_header=newHeader;
+      d->_header.setMsgstr (values);
+      
       setModified(true);
 
       emit signalHeaderChanged();
@@ -313,14 +314,18 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
    QStringList::Iterator it,ait;
    QString temp;
    bool found;
-   if(!usePrefs || d->_saveSettings.updateLastTranslator)
+
+   IdentitySettings identityOptions = identitySettings();
+   SaveSettings saveOptions = saveSettings();
+   
+   if(!usePrefs || saveOptions.updateLastTranslator)
    {
       found=false;
 
-      temp="Last-Translator: "+d->_identitySettings.authorName;
-      if(!d->_identitySettings.authorEmail.isEmpty())
+      temp="Last-Translator: "+identityOptions.authorName;
+      if(!identityOptions.authorEmail.isEmpty())
       {
-         temp+=(" <"+d->_identitySettings.authorEmail+">");
+         temp+=(" <"+identityOptions.authorEmail+">");
       }
       temp+="\\n";
       for( it = headerList.begin(); it != headerList.end(); ++it )
@@ -336,7 +341,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateRevisionDate)
+   if(!usePrefs || saveOptions.updateRevisionDate)
    {
       found=false;
 
@@ -355,11 +360,11 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateProject)
+   if(!usePrefs || saveOptions.updateProject)
    {
       found=false;
 
-      temp="Project-Id-Version: "+d->_saveSettings.projectString+"\\n";
+      temp="Project-Id-Version: "+saveOptions.projectString+"\\n";
       temp.replace( "@PACKAGE@", packageName());
 
       for( it = headerList.begin(); it != headerList.end(); ++it )
@@ -375,14 +380,14 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateLanguageTeam)
+   if(!usePrefs || saveOptions.updateLanguageTeam)
    {
       found=false;
 
-      temp="Language-Team: "+d->_identitySettings.languageName;
-      if(!d->_identitySettings.mailingList.isEmpty())
+      temp="Language-Team: "+identityOptions.languageName;
+      if(!identityOptions.mailingList.isEmpty())
       {
-         temp+=(" <"+d->_identitySettings.mailingList+">");
+         temp+=(" <"+identityOptions.mailingList+">");
       }
       temp+="\\n";
       for( it = headerList.begin(); it != headerList.end(); ++it )
@@ -398,18 +403,18 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateCharset)
+   if(!usePrefs || saveOptions.updateCharset)
    {
       found=false;
 
 	  QString encodingStr;
-      if(d->_saveSettings.useOldEncoding && d->fileCodec)
+      if(saveOptions.useOldEncoding && d->fileCodec)
       {
 		  encodingStr = charsetString(d->fileCodec);
       }
       else
       {
-          encodingStr=charsetString(d->_saveSettings.encoding);
+          encodingStr=charsetString(saveOptions.encoding);
       }
       temp="Content-Type: text/plain; charset="+encodingStr+"\\n";
       QString charsettemp="; charset="+encodingStr+"\\n";
@@ -427,7 +432,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
           headerList.append(temp);
        }
    }
-   if(!usePrefs || d->_saveSettings.updateEncoding)
+   if(!usePrefs || saveOptions.updateEncoding)
    {
       found=false;
 
@@ -482,11 +487,11 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 
 
    temp="Plural-Forms: %1\\n";
-   temp=temp.arg(d->_identitySettings.gnuPluralFormHeader);
+   temp=temp.arg(identityOptions.gnuPluralFormHeader);
    found=false;
 
    // update plural form header
-   if( !d->_identitySettings.gnuPluralFormHeader.isEmpty() )
+   if( !identityOptions.gnuPluralFormHeader.isEmpty() )
    {
 	for( it = headerList.begin(); it != headerList.end(); ++it )
 	{
@@ -497,7 +502,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
     	    }
 	}
     }
-    if(!found && !d->_identitySettings.gnuPluralFormHeader.isEmpty() )
+    if(!found && !identityOptions.gnuPluralFormHeader.isEmpty() )
     {
        headerList.append(temp);
     }
@@ -513,7 +518,7 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
    oldHeader.setMsgstr(msgstr);
 
    //comment = description, copyrights
-   if(!usePrefs || (d->_saveSettings.FSFCopyright != NoChange))
+   if(!usePrefs || (saveOptions.FSFCopyright != ProjectSettingsBase::NoChange))
    {
       found=false;
 
@@ -530,10 +535,12 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 	    if((*it).contains(QRegExp("^# *Copyright \\(C\\) YEAR Free Software Foundation, Inc\\.")))
 	    {
 		//template string
-    		if(d->_saveSettings.FSFCopyright == Remove) (*it).replace(" YEAR Free Software Foundation, Inc","");
-		else (*it).replace("YEAR", QDate::currentDate().toString("yyyy"));
+    		if( saveOptions.FSFCopyright == ProjectSettingsBase::Remove) 
+		    (*it).replace(" YEAR Free Software Foundation, Inc","");
+		else 
+		    (*it).replace("YEAR", QDate::currentDate().toString("yyyy"));
 	    } else
-	    if( d->_saveSettings.FSFCopyright == Update )
+	    if( saveOptions.FSFCopyright == ProjectSettingsBase::Update )
 	    {
 		    //update years
 		    QString cy = QDate::currentDate().toString("yyyy");
@@ -552,19 +559,26 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 	}
    }
 
-   if(!usePrefs || d->_saveSettings.updateDescription)
+   if(!usePrefs || saveOptions.updateDescription)
    {
-      temp="# "+d->_saveSettings.descriptionString;
+      temp="# "+saveOptions.descriptionString;
       temp.replace( "@PACKAGE@", packageName());
-      temp.replace( "@LANGUAGE@", d->_identitySettings.languageName);
+      temp.replace( "@LANGUAGE@", identityOptions.languageName);
+      // strip the ending whitespace
+      temp.replace( QRegExp("\\s*$"), "" );
+      
+      kdDebug () << "Temp is '" << temp << "'" << endl;
 
       found=false;
       bool foundTemplate=false;
 
       for( it = commentList.begin(); it != commentList.end(); ++it )
       {
+         kdDebug () << "testing '" << (*it) << "'" << endl;
+
          if((*it).contains(QRegExp("^"+temp+"$")))
          {
+	    kdDebug () << "Match " << endl;
             found=true;
          }
 	 if((*it).contains(QRegExp("^# SOME DESCRIPTIVE TITLE.$")))
@@ -577,14 +591,14 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
        if(!found) commentList.prepend(temp);
    }
 
-   if(!usePrefs || d->_saveSettings.updateTranslatorCopyright)
+   if(!usePrefs || saveOptions.updateTranslatorCopyright)
    {
       QStringList foundAuthors;
 
-      temp="# "+d->_identitySettings.authorName;
-      if(!d->_identitySettings.authorEmail.isEmpty())
+      temp="# "+identityOptions.authorName;
+      if(!identityOptions.authorEmail.isEmpty())
       {
-         temp+=(" <"+d->_identitySettings.authorEmail+">");
+         temp+=(" <"+identityOptions.authorEmail+">");
       }
       temp+=", "+QDate::currentDate().toString("yyyy")+".";
 
@@ -611,8 +625,8 @@ CatalogItem Catalog::updatedHeader(CatalogItem oldHeader, bool usePrefs) const
 
 	  for( it = foundAuthors.begin() ; it!=foundAuthors.end(); ++it )
 	  {
-		if( (*it).contains(QRegExp(d->_identitySettings.authorName+".*"
-		    +d->_identitySettings.authorEmail)))
+		if( (*it).contains(QRegExp(identityOptions.authorName+".*"
+		    +identityOptions.authorEmail)))
 		{
 		    foundAuthor = true;
 		    if( (*it).contains(QRegExp(cy))) found = true;
@@ -720,7 +734,7 @@ QString Catalog::packageName() const
 
     QString package=d->_url.fileName();
 
-    int index=package.find(QRegExp("(\\."+d->_identitySettings.languageCode+")?\\.pot?$"));
+    int index=package.find(QRegExp("(\\."+identitySettings().languageCode+")?\\.pot?$"));
 
     if(index>0)
       package=package.left(index);
@@ -757,14 +771,16 @@ QString Catalog::packageDir() const
 
 QString Catalog::encoding() const
 {
+    SaveSettings options = saveSettings();
+    
     QString encodingStr;
-    if(d->_saveSettings.useOldEncoding && d->fileCodec)
+    if(options.useOldEncoding && d->fileCodec)
     {
 	    encodingStr = charsetString(d->fileCodec);
     }
     else
     {
-        encodingStr= charsetString(d->_saveSettings.encoding);
+        encodingStr= charsetString(options.encoding);
     }
 
     return encodingStr;
@@ -775,21 +791,28 @@ ConversionStatus Catalog::openURL(const KURL& url, const QString& package)
    QString target;
    ConversionStatus error = OK;
 
-   if(KIO::NetAccess::download(url, target))
+   if(KIO::NetAccess::download(url, target, NULL))
    {
        CatalogImportPlugin* filter=0;
 
        // gimme plugin for this MIME type
-       KMimeType::Ptr mime = KMimeType::findByURL( target );
+       KMimeType::Ptr mime = KMimeType::findByURL( url, 0, true );
+       kdDebug() << "Found mimetype: " << mime->name() <<  endl;
        KTrader::OfferList offers = KTrader::self()->query("KBabelFilter", "('"+mime->name()+"' in [X-KDE-Import])");
        KService::Ptr ptr = offers.first();
 
        // we have no offer for this MIME type
        if( !ptr )
        {
-    	    kdDebug(KBABEL) << "No plugin for this type" << endl;
-    	    KIO::NetAccess::removeTempFile(target);
-    	    return NO_PLUGIN;
+    	    kdDebug(KBABEL) << "No plugin for this type, will try PO" << endl;
+    	    offers = KTrader::self()->query("KBabelFilter", "('application/x-gettext' in [X-KDE-Import])");
+    	    ptr = offers.first();
+    	    if( !ptr )
+	    {
+    		kdDebug(KBABEL) << "No plugin for PO files, giving up" << endl;
+    		KIO::NetAccess::removeTempFile(target);
+    		return NO_PLUGIN;
+	    }
        }
 
        // try to load the library, if unsuccesfull, we have an installation problem
@@ -814,7 +837,7 @@ ConversionStatus Catalog::openURL(const KURL& url, const QString& package)
 
 	connect( this, SIGNAL( signalStopActivity() ),
 	    filter, SLOT( stop() ));
-
+	    
 	// load in the file (target is always local)
 	d->_active = true;
 	kdDebug(KBABEL) << "openURL active" << endl;
@@ -861,12 +884,12 @@ ConversionStatus Catalog::openURL(const KURL& openUrl, const KURL& saveURL, cons
    QString target;
    ConversionStatus error = OK;
 
-   if(KIO::NetAccess::download(openUrl, target))
+   if(KIO::NetAccess::download(openUrl, target, NULL))
    {
        CatalogImportPlugin* filter=0;
 
        // gimme plugin for this MIME type
-       KMimeType::Ptr mime = KMimeType::findByURL( target );
+       KMimeType::Ptr mime = KMimeType::findByURL( openUrl, 0, true );
        KTrader::OfferList offers = KTrader::self()->query("KBabelFilter", "('"+mime->name()+"' in [X-KDE-Import])");
        KService::Ptr ptr = offers.first();
 
@@ -1191,6 +1214,7 @@ void Catalog::modifyUsingTool(KDataTool* tool, const QString& command)
 
 void Catalog::clear()
 {
+    d->_errorIndex.clear();
     d->_entries.clear();
     d->_url=KURL();
     d->_obsoleteEntries.clear();
@@ -1434,182 +1458,60 @@ bool Catalog::isLastView() const
 }
 
 
-void Catalog::readPreferences(KConfig *config)
+void Catalog::useProject(Project::Ptr project)
 {
-   KConfigGroupSaver groupSaver(config,"Header");
-
-   d->_saveSettings.autoUpdate=config->readBoolEntry("AutoUpdate"
-				   ,Defaults::Save::autoUpdate);
-   d->_saveSettings.updateLastTranslator=config->readBoolEntry("Update-Last-Translator"
-                       ,Defaults::Save::updateLastTranslator);
-   d->_saveSettings.updateRevisionDate=config->readBoolEntry("Update-Revision-Date"
-                       ,Defaults::Save::updateRevisionDate);
-   d->_saveSettings.updateLanguageTeam=config->readBoolEntry("Update-Language-Team"
-                       ,Defaults::Save::updateLanguageTeam);
-   d->_saveSettings.updateCharset=config->readBoolEntry("Update-Charset"
-                       ,Defaults::Save::updateCharset);
-   d->_saveSettings.updateEncoding=config->readBoolEntry("Update-Encoding"
-                       ,Defaults::Save::updateEncoding);
-   d->_saveSettings.encoding=(FileEncoding)(config->readNumEntry("Encoding"
-						   ,(int)Defaults::Save::encoding));
-   d->_saveSettings.useOldEncoding=config->readBoolEntry("UseOldEncoding"
-                           ,Defaults::Save::useOldEncoding);
-
-   d->_saveSettings.updateProject=config->readBoolEntry("Update-Project"
-                       ,Defaults::Save::updateProject);
-   d->_saveSettings.projectString=config->readEntry("ProjectString"
-                       ,Defaults::Save::projectString());
-
-   d->_saveSettings.autoSyntaxCheck = config->readBoolEntry("AutoSyntaxCheck"
-                 ,Defaults::Save::autoSyntaxCheck);
-   d->_saveSettings.saveObsolete = config->readBoolEntry("SaveObsolete"
-                 ,Defaults::Save::saveObsolete);
-   d->_saveSettings.customDateFormat = config->readEntry("CustomDateFormat"
-                 ,Defaults::Save::customDateFormat());
-   d->_saveSettings.dateFormat = (Qt::DateFormat)( config->readNumEntry("DateFormat"
-                 ,(int)Defaults::Save::dateFormat) );
-   d->_saveSettings.updateDescription = config->readBoolEntry("UpdateDescription"
-		 ,Defaults::Save::updateDescription);
-   d->_saveSettings.descriptionString = config->readEntry("DescriptionString"
-		 ,Defaults::Save::descriptionString());
-   d->_saveSettings.updateTranslatorCopyright = config->readBoolEntry("UpdateTranslatorCopyright"
-		 ,Defaults::Save::updateTranslatorCopyright);
-   d->_saveSettings.FSFCopyright=(CopyrightUpdate)(config->readNumEntry("FSFCopyright"
-						   ,(int)Defaults::Save::FSFCopyright));
-
-   d->_saveSettings.autoSaveDelay=config->readNumEntry("AutoSaveDelay", Defaults::Save::autoSaveDelay);
-
-    Defaults::Identity defaultIdentity;
-   d->_identitySettings.authorName=config->readEntry("Author-Name"
-				   ,defaultIdentity.authorName());
-   d->_identitySettings.authorLocalizedName=config->readEntry("Local-Author-Name"
-				   ,defaultIdentity.authorName());
-   d->_identitySettings.authorEmail=config->readEntry("Author-Email"
-				   ,defaultIdentity.authorEmail());
-   d->_identitySettings.languageName=config->readEntry("Language"
-				   ,defaultIdentity.languageName());
-   d->_identitySettings.languageCode=config->readEntry("LanguageCode"
-				   ,defaultIdentity.languageCode());
-   d->_identitySettings.mailingList=config->readEntry("Mailinglist"
-				   ,defaultIdentity.mailingList());
-   d->_identitySettings.timeZone=config->readEntry("Timezone"
-				   ,defaultIdentity.timezone());
-
-   config->setGroup("Misc");
-
-   Defaults::Misc defaultMisc;
-   QString temp=config->readEntry("AccelMarker"
-				   ,defaultMisc.accelMarker());
-   if(temp.length() > 0)
-		   d->_miscSettings.accelMarker=temp[0];
-
-   temp = config->readEntry("ContextInfo"
-				   ,defaultMisc.contextInfo().pattern());
-   d->_miscSettings.contextInfo.setPattern(temp);
-
-   temp = config->readEntry("SingularPlural"
-				   ,defaultMisc.singularPlural().pattern());
-   d->_miscSettings.singularPlural.setPattern(temp);
-
-   d->_identitySettings.numberOfPluralForms=config->readNumEntry("PluralForms"
-           , defaultIdentity.numberOfPluralForms);
-   if (d->_identitySettings.numberOfPluralForms < 1)
-   {
-	kdWarning() << "Invalid number of plural forms, replacing by 2: " << d->_identitySettings.numberOfPluralForms << endl;
-	d->_identitySettings.numberOfPluralForms = 2;
-   }
-   d->_identitySettings.checkPluralArgument=config->readBoolEntry("CheckPluralArgument",true);
-   d->_identitySettings.gnuPluralFormHeader=config->readEntry("PluralFormsHeader"
-           , defaultIdentity.gnuPluralFormHeader);
-
-   getNumberOfPluralForms();
-
-   d->_miscSettings.useBzip = config->readBoolEntry("BZipCompression", defaultMisc.useBzip);
-   d->_miscSettings.compressSingleFile = config->readBoolEntry("CompressSingleFile", defaultMisc.compressSingleFile);
-
-   config->setGroup("Tags");
-
-   d->_tagSettings.tagExpressions=config->readListEntry("TagExpressions");
-   if( d->_tagSettings.tagExpressions.empty() ) d->_tagSettings.tagExpressions = Defaults::Tag::tagExpressions();
-   d->_tagExtractor->setRegExpList(d->_tagSettings.tagExpressions) ;
-
-   d->_tagSettings.argExpressions=config->readListEntry("ArgExpressions");
-   if( d->_tagSettings.argExpressions.empty() ) d->_tagSettings.argExpressions = Defaults::Tag::argExpressions();
-   d->_argExtractor->setRegExpList(d->_tagSettings.argExpressions) ;
-
+    d->_project->config()->sync();
+    d->_project = project;
+    readPreferences();
+    emit signalSettingsChanged(saveSettings());
+    emit signalSettingsChanged(identitySettings());
+    emit signalSettingsChanged(miscSettings());
+    emit signalSettingsChanged(tagSettings());
 }
 
-void Catalog::savePreferences(KConfig *config)
+Project::Ptr Catalog::project() const
 {
-   KConfigGroupSaver groupSaver(config,"Header");
+    return d->_project;
+}
 
-   config->writeEntry("AutoUpdate",d->_saveSettings.autoUpdate);
-   config->writeEntry("Update-Last-Translator",d->_saveSettings.updateLastTranslator);
-   config->writeEntry("Update-Revision-Date",d->_saveSettings.updateRevisionDate);
-   config->writeEntry("Update-Language-Team",d->_saveSettings.updateLanguageTeam);
-   config->writeEntry("Update-Charset",d->_saveSettings.updateCharset);
-   config->writeEntry("Update-Encoding",d->_saveSettings.updateEncoding);
-   config->writeEntry("Encoding",(int)d->_saveSettings.encoding);
-   config->writeEntry("UseOldEncoding",d->_saveSettings.useOldEncoding);
+void Catalog::readPreferences()
+{
+    getNumberOfPluralForms();
+    
+    d->_project->config()->setGroup("Tags");
+    d->_tagSettings.tagExpressions=d->_project->config()->readListEntry("TagExpressions");
+    if( d->_tagSettings.tagExpressions.empty() ) d->_tagSettings.tagExpressions = Defaults::Tag::tagExpressions();
+    d->_tagExtractor->setRegExpList(d->_tagSettings.tagExpressions) ;
 
-   config->writeEntry("Update-Project", d->_saveSettings.updateProject);
-   config->writeEntry("ProjectString", d->_saveSettings.projectString);
+    d->_tagSettings.argExpressions=d->_project->config()->readListEntry("ArgExpressions");
+    if( d->_tagSettings.argExpressions.empty() ) d->_tagSettings.argExpressions = Defaults::Tag::argExpressions();
+    d->_argExtractor->setRegExpList(d->_tagSettings.argExpressions) ;
+}
 
-   config->writeEntry("AutoSyntaxCheck",d->_saveSettings.autoSyntaxCheck);
-   config->writeEntry("SaveObsolete",d->_saveSettings.saveObsolete);
-   config->writeEntry("CustomDateFormat",d->_saveSettings.customDateFormat);
-   config->writeEntry("DateFormat",(int)d->_saveSettings.dateFormat);
+void Catalog::savePreferences()
+{
+   d->_project->config()->setGroup("Tags");
 
-   config->writeEntry("UpdateDescription", d->_saveSettings.updateDescription);
-   config->writeEntry("DescriptionString", d->_saveSettings.descriptionString);
-   config->writeEntry("UpdateTranslatorCopyright",d->_saveSettings.updateTranslatorCopyright);
-   config->writeEntry("FSFCopyright", (int)d->_saveSettings.FSFCopyright);
+   d->_project->config()->writeEntry( "TagExpressions", d->_tagSettings.tagExpressions );
+   d->_project->config()->writeEntry( "ArgExpressions", d->_tagSettings.argExpressions );
 
-   config->writeEntry("AutoSaveDelay", d->_saveSettings.autoSaveDelay);
-
-   config->writeEntry("Author-Name",d->_identitySettings.authorName);
-   config->writeEntry("Local-Author-Name",d->_identitySettings.authorLocalizedName);
-   config->writeEntry("Author-Email",d->_identitySettings.authorEmail);
-   config->writeEntry("Language",d->_identitySettings.languageName);
-   config->writeEntry("LanguageCode",d->_identitySettings.languageCode);
-   config->writeEntry("Mailinglist",d->_identitySettings.mailingList);
-   config->writeEntry("Timezone",d->_identitySettings.timeZone);
-
-
-   config->setGroup("Misc");
-   QString temp(d->_miscSettings.accelMarker);
-   config->writeEntry("AccelMarker",temp);
-   config->writeEntry("ContextInfo",d->_miscSettings.contextInfo.pattern());
-   config->writeEntry("SingularPlural",d->_miscSettings.singularPlural.pattern());
-   config->writeEntry("PluralForms",d->_identitySettings.numberOfPluralForms);
-   config->writeEntry("PluralFormsHeader",d->_identitySettings.gnuPluralFormHeader);
-   config->writeEntry("BZipCompression", d->_miscSettings.useBzip);
-   config->writeEntry("CompressSingleFile", d->_miscSettings.compressSingleFile);
-   config->writeEntry("CheckPluralArgument", d->_identitySettings.checkPluralArgument);
-
-   config->setGroup("Tags");
-
-   config->writeEntry( "TagExpressions", d->_tagSettings.tagExpressions );
-   config->writeEntry( "ArgExpressions", d->_tagSettings.argExpressions );
-
-   config->sync();
+   d->_project->config()->sync();
 }
 
 IdentitySettings Catalog::identitySettings() const
 {
-    return d->_identitySettings;
-
+    return d->_project->identitySettings();
 }
 
 SaveSettings Catalog::saveSettings() const
 {
-    return d->_saveSettings;
+    return d->_project->saveSettings();
 
 }
 
 MiscSettings Catalog::miscSettings() const
 {
-    return d->_miscSettings;
+    return d->_project->miscSettings();
 
 }
 
@@ -1636,25 +1538,25 @@ bool Catalog::isReadOnly() const
 
 void Catalog::setSettings(SaveSettings settings)
 {
-   d->_saveSettings=settings;
-
-   emit signalSettingsChanged(settings);
+   d->_project->setSettings(settings);
 }
 
 void Catalog::setSettings(IdentitySettings settings)
 {
-   QString oldLanguageCode = d->_identitySettings.languageCode;
-   int oldForms =d->_identitySettings.numberOfPluralForms;
+   IdentitySettings oldsettings = d->_project->identitySettings();
+   
+   QString oldLanguageCode = oldsettings.languageCode;
+   int oldForms = oldsettings.numberOfPluralForms;
 
 
-   d->_identitySettings=settings;
+   d->_project->setSettings(settings);
 
-   if(oldLanguageCode != d->_identitySettings.languageCode)
+   if(oldLanguageCode != settings.languageCode)
    {
        getNumberOfPluralForms();
    }
 
-    if(oldForms != d->_identitySettings.numberOfPluralForms)
+    if(oldForms != settings.numberOfPluralForms)
     {
         getNumberOfPluralForms();
     }
@@ -1664,9 +1566,8 @@ void Catalog::setSettings(IdentitySettings settings)
 
 void Catalog::setSettings(MiscSettings settings)
 {
-	d->_miscSettings=settings;
-
-	emit signalSettingsChanged(settings);
+    d->_project->setSettings(settings);
+    emit signalSettingsChanged(settings);
 }
 
 void Catalog::setSettings(TagSettings settings)
@@ -1769,18 +1670,22 @@ QString Catalog::dateTime() const
     QString dateTimeString;
     QDate date=QDate::currentDate();
     QTime time=QTime::currentTime();
+    
+    SaveSettings options = d->_project->saveSettings();
 
-    switch(d->_saveSettings.dateFormat)
+    switch(options.dateFormat)
     {
        case Qt::LocalDate:
        {
           return KGlobal::locale()->formatDateTime(QDateTime::currentDateTime());
        }
        case Qt::ISODate:
-          dateTimeString = Defaults::Save::customDateFormat();
+	  d->_project->settings()->useDefaults( true );
+          dateTimeString = d->_project->settings()->customDateFormat();
+	  d->_project->settings()->useDefaults( false );
           break;
        case Qt::TextDate:
-          dateTimeString = d->_saveSettings.customDateFormat;
+          dateTimeString = options.customDateFormat;
           break;
     }
 
@@ -1877,12 +1782,13 @@ QString Catalog::dateTime() const
     }
 
     // timezone
-    dateTimeString.replace( "%Z", d->_identitySettings.timeZone );
+    dateTimeString.replace( "%Z", d->_project->identitySettings().timeZone );
     QTime t;
     int sgn = KRFCDate::localUTCOffset() < 0 ? -1 : 1 ;
     t = t.addSecs( sgn*KRFCDate::localUTCOffset()*60 );
     dateTimeString.replace( "%z", (sgn<0 ? "-" : "+") +t.toString("hhmm"));
 
+    kdDebug () << "New date: " << dateTimeString;
     return dateTimeString;
 }
 
@@ -1914,7 +1820,7 @@ ConversionStatus Catalog::saveFileAs(const KURL &url, bool overwrite)
    }
 
 
-   if(d->_saveSettings.autoUpdate)
+   if(d->_project->saveSettings().autoUpdate)
    {
       d->_header=updatedHeader(d->_header);
       emit signalHeaderChanged();
@@ -1956,7 +1862,7 @@ ConversionStatus Catalog::saveFileAs(const KURL &url, bool overwrite)
       if(status == OK)
       {
          KURL temp(tempFile);
-         if( !KIO::NetAccess::upload( temp.url(), targetURL.url() ) )
+         if( !KIO::NetAccess::upload( temp.url(), targetURL, NULL ) )
          {
             status = OS_ERROR;
          }
@@ -2022,7 +1928,7 @@ ConversionStatus Catalog::writeFile(QString localFile , bool overwrite)
     CatalogExportPlugin* filter=0;
 
     // gimme plugin for this MIME type
-    KMimeType::Ptr mime = KMimeType::findByURL( localFile );
+    KMimeType::Ptr mime = KMimeType::findByURL( KURL::fromPathOrURL( localFile ) );
     KTrader::OfferList offers = KTrader::self()->query("KBabelFilter", "('"+mime->name()+"' in [X-KDE-Export])");
     KService::Ptr ptr = offers.first();
 
@@ -2456,6 +2362,8 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 	d->_active=true;
 	d->_stop=false;
 	connect( this, SIGNAL( signalStopActivity() ), this, SLOT( stopInternal() ));
+	
+	MiscSettings miscOptions = miscSettings();
 
 	len=0;
 	int pos=0;
@@ -2527,8 +2435,8 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
 
         if(findOpts->ignoreContextInfo)
         {
-            contextInfoPos = d->_miscSettings.contextInfo.search(targetStr);
-	    contextInfoLength = d->_miscSettings.contextInfo.matchedLength();
+            contextInfoPos = miscOptions.contextInfo.search(targetStr);
+	    contextInfoLength = miscOptions.contextInfo.matchedLength();
             if(contextInfoPos >= 0)
             {
                 targetStr.remove(contextInfoPos,contextInfoLength);
@@ -2539,9 +2447,9 @@ bool Catalog::findNext(const FindOptions* findOpts, DocPosition& docPos, int& le
         }
 
         if(findOpts->ignoreAccelMarker
-                && targetStr.contains(d->_miscSettings.accelMarker))
+                && targetStr.contains(miscOptions.accelMarker))
         {
-            accelMarkerPos = targetStr.find(d->_miscSettings.accelMarker);
+            accelMarkerPos = targetStr.find(miscOptions.accelMarker);
             targetStr.remove(accelMarkerPos,1);
 
             if(docPos.offset > (uint)accelMarkerPos)
@@ -2725,6 +2633,8 @@ bool Catalog::findPrev(const FindOptions* findOpts, DocPosition& docPos, int& le
 	d->_active=true;
 	d->_stop=false;
 	connect( this, SIGNAL( signalStopActivity() ), this, SLOT( stopInternal() ));
+	
+	MiscSettings miscOptions = miscSettings();
 
 	len=0;
 	int pos=0;
@@ -2770,8 +2680,8 @@ bool Catalog::findPrev(const FindOptions* findOpts, DocPosition& docPos, int& le
 
         if(findOpts->ignoreContextInfo)
         {
-            contextInfoPos = d->_miscSettings.contextInfo.search(targetStr);
-	    contextInfoLength = d->_miscSettings.contextInfo.matchedLength();
+            contextInfoPos = miscOptions.contextInfo.search(targetStr);
+	    contextInfoLength = miscOptions.contextInfo.matchedLength();
             if(contextInfoPos >= 0)
             {
                 targetStr.remove(contextInfoPos,contextInfoLength);
@@ -2782,9 +2692,9 @@ bool Catalog::findPrev(const FindOptions* findOpts, DocPosition& docPos, int& le
         }
 
         if(findOpts->ignoreAccelMarker
-                && targetStr.contains(d->_miscSettings.accelMarker))
+                && targetStr.contains(miscOptions.accelMarker))
         {
-            accelMarkerPos = targetStr.find(d->_miscSettings.accelMarker);
+            accelMarkerPos = targetStr.find(miscOptions.accelMarker);
             targetStr.remove(accelMarkerPos,1);
 
             if(docPos.offset > (uint)accelMarkerPos)
@@ -3051,7 +2961,7 @@ Catalog::DiffResult Catalog::diff(uint entry, QString *result)
             QStringList::ConstIterator it;
             for(it = list.begin(); it != list.end(); ++it)
             {
-                double weight = fstrcmp( id.utf8(), (*it).utf8() );
+                double weight = LevenshteinDistance()(id, (*it));
                 if(weight > bestWeight)
                 {
                     bestWeight = weight;
@@ -3090,7 +3000,7 @@ Catalog::DiffResult Catalog::diff(uint entry, QString *result)
                 emit signalProgress(percent);
             }
 
-            double weight = fstrcmp( id.utf8(), (*it).utf8() );
+            double weight = LevenshteinDistance()( id, (*it) );
             if(weight > bestWeight)
             {
                 bestWeight = weight;
@@ -3464,13 +3374,15 @@ int Catalog::defaultNumberOfPluralForms() const
 
 void Catalog::getNumberOfPluralForms()
 {
-    if(d->_identitySettings.numberOfPluralForms > 0)
+    IdentitySettings options = identitySettings();
+    
+    if(options.numberOfPluralForms > 0)
     {
-        d->numberOfPluralForms = d->_identitySettings.numberOfPluralForms;
+        d->numberOfPluralForms = options.numberOfPluralForms;
         return;
     }
 
-    QString lang=d->_identitySettings.languageCode;
+    QString lang=options.languageCode;
     if(lang.isEmpty())
     {
         d->numberOfPluralForms=-1;
@@ -3559,6 +3471,13 @@ bool Catalog::isModified() const
 void Catalog::setEntries(QValueVector<CatalogItem> entries)
 {
     d->_entries=entries;
+    
+    // update the project for entries
+    for ( QValueVector<CatalogItem>::Iterator it = d->_entries.begin();
+					it != d->_entries.end(); ++it)
+    {
+	it->setProject( d->_project );
+    }
 }
 
 void Catalog::setObsoleteEntries(QValueList<CatalogItem> entries)
@@ -3637,4 +3556,35 @@ QString Catalog::mimeTypes() const
     return d->_mimeTypes;
 }
 
+void Catalog::wordCount (uint &total, uint &fuzzy, uint &untranslated) const
+{
+    total = 0;
+    fuzzy = 0;
+    untranslated = 0;
+    
+    QRegExp separator( "[ \n\t]+" );
+    
+    for ( QValueVector<CatalogItem>::Iterator it = d->_entries.begin();
+					it != d->_entries.end(); ++it)
+    {
+	// find out the number of words for this message
+	
+	// join all forms together
+	QString message = (*it).msgid ().join (" ");
+	
+	// remove tags first
+	d->_tagExtractor->setString( message );
+	message = d->_tagExtractor->plainString(false);
+
+	QStringList words = QStringList::split ( separator, message );
+	
+	total += words.count();
+
+	if ( (*it).isFuzzy() )
+	    fuzzy += words.count();
+	else if ( (*it).isUntranslated() )
+	    untranslated += words.count();
+    }    
+}
+
 #include "catalog.moc"
