@@ -239,7 +239,7 @@ TabView::TabView(TraceItemView* parentView,
   QVBoxLayout* vbox = new QVBoxLayout( this, 6, 6);
 
   _nameLabel = new KSqueezedTextLabel( this, "nameLabel" );
-  _nameLabel->setText(i18n("(No trace file loaded)"));
+  _nameLabel->setText(i18n("(No profile data file loaded)"));
   vbox->addWidget( _nameLabel );
 
   _mainSplitter   = new QSplitter(Qt::Horizontal, this);
@@ -277,28 +277,42 @@ TabView::TabView(TraceItemView* parentView,
 
 
   // default positions...
-  addBottom( addTab( i18n("Parts"),
-		     new PartView(this, _bottomTW, "PartView")));
+
   addTop( addTab( i18n("Types"),
-		  new CostTypeView(this, _topTW, "CostTypeView")));
+		  new CostTypeView(this, _topTW,
+				   "CostTypeView")));
   addTop( addTab( i18n("Callers"),
-	     new CallView(true, this, _topTW, "CallerView")));
-  addBottom( addTab( i18n("Calls"),
-		new CallView(false, this, _bottomTW, "CallView")));
-  addBottom( addTab( i18n("Call Graph"),
-		new CallGraphView(this, _bottomTW, "CallGraphView")));
+		  new CallView(true, this, _topTW,
+			       "CallerView")));
   addTop( addTab( i18n("All Callers"),
-	     new CoverageView(true, this, _topTW, "AllCallerView")));
-  addBottom( addTab( i18n("All Calls"),
-		new CoverageView(false, this, _bottomTW, "AllCallView")));
-  addBottom( addTab( i18n("Caller Map"),
-		new CallMapView(true, this, _bottomTW, "CallerMapView")));
-  addTop( addTab( i18n("Call Map"),
-	     new CallMapView(false, this, _topTW, "CallMapView")));
+		  new CoverageView(true, this, _topTW,
+				   "AllCallerView")));
+  addTop( addTab( i18n("Caller Map"),
+		  new CallMapView(true, this, _bottomTW,
+				  "CallerMapView")));
   addTop( addTab( i18n("Source"),
-	     new SourceView(this, _topTW, "SourceView")));
+		  new SourceView(this, _topTW,
+				 "SourceView")));
+
+  addBottom( addTab( i18n("Parts"),
+		     new PartView(this, _bottomTW,
+				  "PartView")));
+  addBottom( addTab( i18n("Call Graph"),
+		     new CallGraphView(this, _bottomTW,
+				       "CallGraphView")));
+  addBottom( addTab( i18n("Callees"),
+		     new CallView(false, this, _bottomTW,
+				  "CalleeView")));
+  addBottom( addTab( i18n("All Callees"),
+		     new CoverageView(false, this, _bottomTW,
+				      "AllCallView")));
+
+  addBottom( addTab( i18n("Callee Map"),
+		     new CallMapView(false, this, _topTW,
+				     "CallMapView")));
   addBottom( addTab( i18n("Assembler"),
-		new InstrView(this, _bottomTW, "InstrView")));
+		     new InstrView(this, _bottomTW,
+				   "InstrView")));
 
   // after all child widgets are created...
   _lastFocus = 0;
@@ -310,6 +324,15 @@ TabView::TabView(TraceItemView* parentView,
   QWhatsThis::add( this, whatsThis() );
 }
 
+void TabView::setData(TraceData* d)
+{
+  TraceItemView::setData(d);
+
+  TraceItemView* v;
+  for (v=_tabs.first();v;v=_tabs.next())
+    v->setData(d);  
+}
+
 TraceItemView* TabView::addTab(QString label, TraceItemView* view)
 {
     view->setTitle(label);
@@ -528,11 +551,11 @@ void TabView::moveTab(QWidget* w, Position p, bool wholeArea)
 QString TabView::whatsThis() const
 {
     return i18n( "<b>Information Tabs</b>"
-		 "<p>This widgets shows information for the "
+		 "<p>This widget shows information for the "
 		 "current selected function in different tabs: "
 		 "<ul>"
-		 "<li>The Costs tab shows a list of available cost types "
-		 "and the cumulative and self costs regarding to types.</li>"
+		 "<li>The Costs tab shows a list of available event types "
+		 "and the inclusive and self costs regarding to these types.</li>"
 		 "<li>The Parts tab shows a list of trace parts "
 		 "if the trace consists of more than one part (otherwise, "
 		 "this tab is hided). "
@@ -630,7 +653,8 @@ void TabView::doUpdate(int changeType)
       if (tw) {
         if (!tw->hasVisibleRect()) continue;
       }
-      canShow = v->set(changeType, _data, _costType, _groupType, _partList,
+      canShow = v->set(changeType, _data, _costType, _costType2,
+		       _groupType, _partList,
                        _activeItem, _selectedItem);
       v->notifyChange(changeType);
 
@@ -685,7 +709,9 @@ void TabView::selected(TraceItemView*, TraceItem* s)
 }
 
 
-void TabView::readViewConfig(KConfig* c, QString prefix, QString postfix)
+void TabView::readViewConfig(KConfig* c,
+			     QString prefix, QString postfix,
+			     bool withOptions)
 {
     if (0) qDebug("%s::readConfig(%s%s)", name(),
 		  prefix.ascii(), postfix.ascii());
@@ -743,9 +769,10 @@ void TabView::readViewConfig(KConfig* c, QString prefix, QString postfix)
       }
       else moveTab(v->widget(), Hidden);
 
-      v->readViewConfig(c, QString("%1-%2")
-                        .arg(prefix).arg(v->widget()->name()),
-                        postfix);
+      if (withOptions)
+	v->readViewConfig(c, QString("%1-%2")
+			  .arg(prefix).arg(v->widget()->name()),
+			  postfix, true);
     }
     if (activeTop)   _topTW->showPage(activeTop->widget());
     if (activeBottom)_bottomTW->showPage(activeBottom->widget());
@@ -757,27 +784,31 @@ void TabView::readViewConfig(KConfig* c, QString prefix, QString postfix)
     QString selectedType = g->readEntry("SelectedItemType", "");
     QString selectedName = g->readEntry("SelectedItemName", "");
     delete g;
-
+    
     if (!_data) return;
-
-    // restore active item
-    TraceItem::CostType t = TraceItem::costType(activeType);
-    if (t==TraceItem::NoCostType) t = TraceItem::Function;
-    TraceCost* activeItem = _data->search(t, activeName, _costType);
-    if (!activeItem) return;
-    activate(activeItem);
-
-    // restore selected item
-    t = TraceItem::costType(selectedType);
-    if (t==TraceItem::NoCostType) t = TraceItem::Function;
-    TraceCost* selectedItem = _data->search(t, selectedName,
-                                            _costType, activeItem);
-    if (selectedItem) select(selectedItem);
+    
+    if (withOptions) {  
+      // restore active item
+      TraceItem::CostType t = TraceItem::costType(activeType);
+      if (t==TraceItem::NoCostType) t = TraceItem::Function;
+      TraceCost* activeItem = _data->search(t, activeName, _costType);
+      if (!activeItem) return;
+      activate(activeItem);
+      
+      // restore selected item
+      t = TraceItem::costType(selectedType);
+      if (t==TraceItem::NoCostType) t = TraceItem::Function;
+      TraceCost* selectedItem = _data->search(t, selectedName,
+					      _costType, activeItem);
+      if (selectedItem) select(selectedItem);
+    }
 
     updateView();
 }
 
-void TabView::saveViewConfig(KConfig* c, QString prefix, QString postfix)
+void TabView::saveViewConfig(KConfig* c,
+			     QString prefix, QString postfix,
+			     bool withOptions)
 {
     KConfigGroup g(c, (prefix+postfix).ascii());
 
@@ -809,17 +840,17 @@ void TabView::saveViewConfig(KConfig* c, QString prefix, QString postfix)
       a = QString(_rightTW->currentPage()->name());
     g.writeEntry("ActiveRight", a);
 
-
-    if (_activeItem) {
+    if (withOptions)
+      if (_activeItem) {
 	g.writeEntry("ActiveItemType",
                      TraceItem::typeName(_activeItem->type()));
 	g.writeEntry("ActiveItemName", _activeItem->name());
 	if (_selectedItem) {
-	    g.writeEntry("SelectedItemType",
-                         TraceItem::typeName(_selectedItem->type()));
-	    g.writeEntry("SelectedItemName", _selectedItem->name());
+	  g.writeEntry("SelectedItemType",
+		       TraceItem::typeName(_selectedItem->type()));
+	  g.writeEntry("SelectedItemName", _selectedItem->name());
 	}
-    }
+      }
 
     QStringList topList, bottomList, leftList, rightList;
     TraceItemView *v;
@@ -850,9 +881,10 @@ void TabView::saveViewConfig(KConfig* c, QString prefix, QString postfix)
     g.writeEntry("LeftTabs", leftList);
     g.writeEntry("RightTabs", rightList);
 
-    for (v=_tabs.first();v;v=_tabs.next())
-      v->saveViewConfig(c, QString("%1-%2").arg(prefix)
-                        .arg(v->widget()->name()), postfix);
+    if (withOptions)
+      for (v=_tabs.first();v;v=_tabs.next())
+	v->saveViewConfig(c, QString("%1-%2").arg(prefix)
+			  .arg(v->widget()->name()), postfix, true);
 }
 
 #include "tabview.moc"
