@@ -20,6 +20,9 @@
  * KCachegrind top level window
  */
 
+#define TRACE_UPDATES 0
+#define ENABLE_DUMPDOCK 0
+
 #include <stdlib.h> // for system()
 
 #include <qvbox.h>
@@ -50,11 +53,15 @@
 #include <kkeydialog.h>
 #include <ktip.h>
 #include <kpopupmenu.h>
+#include <kdebug.h>
+
+#if ENABLE_DUMPDOCK
+#include "dumpselection.h"
+#endif
 
 #include "toplevel.h"
 #include "partselection.h"
 #include "functionselection.h"
-#include "dumpselection.h"
 #include "stackselection.h"
 #include "stackbrowser.h"
 #include "tracedata.h"
@@ -64,9 +71,6 @@
 #include "multiview.h"
 #include "callgraphview.h"
 
-#define TRACE_UPDATES 0
-#define ENABLE_DUMPDOCK 0
-
 
 TopLevel::TopLevel(const char *name)
   : KMainWindow(0, name), DCOPObject("KCachegrindIface")
@@ -149,12 +153,17 @@ void TopLevel::init()
   _data = 0;
   _function = 0;
   _costType = 0;
+  _costType2 = 0;
   _groupType = TraceCost::NoCostType;
   _group = 0;
 
+  _layoutCurrent = 0;
+  _layoutCount = 1;
+
   // for delayed slots
   _traceItemDelayed = 0;
   _costTypeDelayed = 0;
+  _costType2Delayed = 0;
   _groupTypeDelayed = TraceCost::NoCostType;
   _groupDelayed = 0;
   _directionDelayed = TraceItemView::None;
@@ -207,9 +216,11 @@ void TopLevel::saveCurrentState(QString postfix)
   KConfigGroup stateConfig(kconfig, QCString("CurrentState")+pf);
   stateConfig.writeEntry("CostType",
 			 _costType ? _costType->name() : QString("?"));
+  stateConfig.writeEntry("CostType2",
+			 _costType2 ? _costType2->name() : QString("?"));
   stateConfig.writeEntry("GroupType", TraceItem::typeName(_groupType));
 
-  _multiView->saveViewConfig(kconfig, QString("MainView"), postfix);
+  _multiView->saveViewConfig(kconfig, QString("MainView"), postfix, true);
 }
 
 /**
@@ -223,11 +234,17 @@ void TopLevel::saveTraceSettings()
   KConfigGroup pConfig(KGlobal::config(), QCString("TracePositions"));
   pConfig.writeEntry(QString("CostType%1").arg(key),
                      _costType ? _costType->name() : QString("?"));
+  pConfig.writeEntry(QString("CostType2%1").arg(key),
+                     _costType2 ? _costType2->name() : QString("?"));
   pConfig.writeEntry(QString("GroupType%1").arg(key),
                      TraceItem::typeName(_groupType));
 
   if (!_data) return;
 
+  KConfigGroup aConfig(KGlobal::config(), QCString("Layouts"));
+  aConfig.writeEntry(QString("Count%1").arg(key), _layoutCount);
+  aConfig.writeEntry(QString("Current%1").arg(key), _layoutCurrent);
+
   saveCurrentState(key);
   pConfig.writeEntry(QString("Group%1").arg(key),
                      _group ? _group->name() : QString::null);
@@ -252,7 +269,7 @@ void TopLevel::restoreCurrentState(QString postfix)
   KConfigGroup psConfig(kconfig, group);
   _partSelection->readVisualisationConfig(&psConfig);
 
-  _multiView->readViewConfig(kconfig, QString("MainView"), postfix);
+  _multiView->readViewConfig(kconfig, QString("MainView"), postfix, true);
   _taSplit->setChecked(_multiView->childCount()>1);
   _taSplitDir->setEnabled(_multiView->childCount()>1);
   _taSplitDir->setChecked(_multiView->orientation() == Qt::Horizontal);
@@ -262,16 +279,16 @@ void TopLevel::restoreCurrentState(QString postfix)
 void TopLevel::createDocks()
 {
   _partDock = new QDockWindow(QDockWindow::InDock, this);
-  _partDock->setCaption(i18n("Trace Part Overview"));
+  _partDock->setCaption(i18n("Parts Overview"));
   _partDock->setCloseMode( QDockWindow::Always );
   _partSelection = new PartSelection(_partDock, "partSelection");
   _partDock->setWidget(_partSelection);
   _partDock->setResizeEnabled(true);
   _partDock->setFixedExtentWidth(200);
   QWhatsThis::add( _partSelection, i18n(
-                   "<b>The Trace Part Overview</b>"
+                   "<b>The Parts Overview</b>"
                    "<p>A trace consists of multiple trace parts when "
-                   "there are several trace files from one profile run. "
+                   "there are several profile data files from one profile run. "
                    "The Trace Part Overview dockable shows these, "
                    "horizontally ordered in execution time; "
                    "the rectangle sizes are proportional to the total "
@@ -279,17 +296,17 @@ void TopLevel::createDocks()
                    "parts to constrain all costs shown to these parts only."
                    "</p>"
                    "<p>The parts are further subdivided: there is a "
-                   "partitioning and a cumulative split mode: "
+                   "partitioning and an callee split mode: "
                    "<ul><li>Partitioning: You see the "
                    "partitioning into groups for a trace part, according to "
                    "the group type selected. E.g. if ELF object groups are "
                    "selected, you see colored rectangles for each "
                    "used ELF object (shared library or executable), sized "
                    "according to the cost spent therein.</li>"
-                   "<li>Cumulative Split: A rectangle showing the cumulative "
+                   "<li>Callee: A rectangle showing the inclusive "
                    "cost of the current selected function in the trace part "
                    "is shown. "
-                   "This again is split up to show cumulative costs of its "
+                   "This is split up into smaller rectangles to show the costs of its "
                    "callees.</li></ul></p>"));
 
   _stackDock = new QDockWindow(QDockWindow::InDock, this);
@@ -303,7 +320,7 @@ void TopLevel::createDocks()
   QWhatsThis::add( _stackSelection, i18n(
                    "<b>The Top Cost Call Stack</b>"
                    "<p>This is a purely fictional 'most probable' call stack. "
-                   "It is build up by starting with the current selected "
+                   "It is built up by starting with the current selected "
                    "function and adds the callers/callees with highest cost "
                    "at the top and to bottom.</p>"
                    "<p>The <b>Cost</b> and <b>Calls</b> columns show the "
@@ -431,7 +448,55 @@ void TopLevel::readProperties(KConfig* c)
   }
 }
 
-void TopLevel::createActions()
+void TopLevel::createLayoutActions()
+{
+  QString hint;
+  KAction* action;
+
+  action = new KAction( i18n( "&Duplicate" ),
+			KShortcut(KKey("Ctrl+Plus")),
+                        this, SLOT(layoutDuplicate()),
+                        actionCollection(), "layout_duplicate" );
+  hint = i18n("<b>Duplicate Current Layout</b>"
+              "<p>Make a copy of the current layout.</p>");
+  action->setWhatsThis( hint );
+
+  action = new KAction( i18n( "&Remove" ), KShortcut(),
+                        this, SLOT(layoutRemove()),
+                        actionCollection(), "layout_remove" );
+  hint = i18n("<b>Remove Current Layout</b>"
+              "<p>Delete current layout and make the previous active.</p>");
+  action->setWhatsThis( hint );
+
+  action = new KAction( i18n( "&Go to Next" ),
+			KShortcut(KKey("Ctrl+Right")),
+                        this, SLOT(layoutNext()),
+                        actionCollection(), "layout_next" );
+  hint = i18n("Go to Next Layout");
+  action->setWhatsThis( hint );
+
+  action = new KAction( i18n( "&Go to Previous" ),
+			KShortcut(KKey("Ctrl+Left")),
+                        this, SLOT(layoutPrevious()),
+                        actionCollection(), "layout_previous" );
+  hint = i18n("Go to Previous Layout");
+  action->setWhatsThis( hint );
+
+  action = new KAction( i18n( "&Restore to Default" ), KShortcut(),
+                        this, SLOT(layoutRestore()),
+                        actionCollection(), "layout_restore" );
+  hint = i18n("Restore Layouts to Default");
+  action->setWhatsThis( hint );
+
+  action = new KAction( i18n( "&Save as Default" ), KShortcut(),
+                        this, SLOT(layoutSave()),
+                        actionCollection(), "layout_save" );
+  hint = i18n("Save Layouts as Default");
+  action->setWhatsThis( hint );
+}
+
+// TODO: split this up...
+void TopLevel::createMiscActions()
 {
   QString hint;
   KAction* action;
@@ -440,10 +505,22 @@ void TopLevel::createActions()
   hint = i18n("<b>New</b><p>Open new empty KCachegrind window.</p>");
   action->setWhatsThis( hint );
 
+  action = new KAction( i18n( "&Add..." ), KShortcut(),
+                        this, SLOT(addTrace()),
+                        actionCollection(), "file_add" );
+  hint = i18n("<b>Add Profile Data</b>"
+              "<p>This opens an additional profile data file in the current window.</p>");
+  action->setWhatsThis( hint );
+
   action = new KAction( i18n( "&Reload" ), "reload",
-                        KStdAccel::shortcut(KStdAccel::Reload),
+#if KDE_VERSION > 0x030190
+  // for KDE 3.2: KStdAccel::key is deprecated
+			KStdAccel::shortcut(KStdAccel::Reload),
+#else
+                        KStdAccel::key(KStdAccel::Reload),
+#endif
                         this, SLOT( reload() ), actionCollection(), "reload" );
-  hint = i18n("<b>Reload trace</b>"
+  hint = i18n("<b>Reload Profile Data</b>"
               "<p>This loads any new created parts, too.</p>");
   action->setWhatsThis( hint );
 
@@ -458,37 +535,41 @@ void TopLevel::createActions()
 
 
   _taDump = new KToggleAction( i18n( "&Force Dump" ), "redo",
-                               KStdAccel::shortcut(KStdAccel::Redo),
+#if KDE_VERSION > 0x030190
+  // for KDE 3.2: KStdAccel::key is deprecated
+			       KStdAccel::shortcut(KStdAccel::Redo),
+#else
+                               KStdAccel::key(KStdAccel::Redo),
+#endif
                                this, SLOT( forceTrace() ),
                                actionCollection(), "dump" );
   hint = i18n("<b>Force Dump</b>"
-              "<p>This forces a dump for a Cachegrind profile run "
+              "<p>This forces a dump for a Callgrind profile run "
               "in the current directory. This action is checked while "
               "KCachegrind looks for the dump. If the dump is "
               "finished, it automatically reloads the current trace. "
-              "If this is the one from the running Cachegrind, the new "
+              "If this is the one from the running Callgrind, the new "
               "created trace part will be loaded, too.</p>"
-              "<p>Force dump creates a file 'cachegrind.cmd', and "
+              "<p>Force dump creates a file 'callgrind.cmd', and "
               "checks every second for its existence. A running "
-              "Cachegrind will detect this file, dump a trace part, "
-              "and delete 'cachegrind.cmd'. "
+              "Callgrind will detect this file, dump a trace part, "
+              "and delete 'callgrind.cmd'. "
               "The deletion is detected by KCachegrind, "
-              "and it does a Reload. If there's <em>no</em> Cachegrind "
+              "and it does a Reload. If there's <em>no</em> Callgrind "
               "running, press 'Force Dump' again to cancel the dump "
-              "request. This deletes 'cachegrind.cmd' itself and "
+              "request. This deletes 'callgrind.cmd' itself and "
               "stops polling for a new dump.</p>"
-              "<p>Note: A Cachegrind run <em>only</em> detects "
-              "existence of 'cachegrind.cmd' when actively running "
+              "<p>Note: A Callgrind run <em>only</em> detects "
+              "existence of 'callgrind.cmd' when actively running "
               "a few milliseconds, i.e. "
               "<em>not</em> sleeping. Tip: For a profiled GUI program, "
-              "you can awake Cachegrind e.g. by resizing a window "
-              "of the program.</p>"
-              "<p>Note: You need the call-tree version for Cachegrind!</p>");
+              "you can awake Callgrind e.g. by resizing a window "
+              "of the program.</p>");
   _taDump->setWhatsThis( hint );
 
   action = KStdAction::open(this, SLOT(loadTrace()), actionCollection());
-  hint = i18n("<b>Open Trace</b>"
-              "<p>This opens a trace, with possible multiple parts</p>");
+  hint = i18n("<b>Open Profile Data</b>"
+              "<p>This opens a profile data file, with possible multiple parts</p>");
   action->setToolTip( hint );
   action->setWhatsThis( hint );
 
@@ -498,30 +579,30 @@ void TopLevel::createActions()
   KStdAction::showStatusbar(this,
                             SLOT(toggleStatusBar()), actionCollection());
 
-  _partDockShown = new KToggleAction(i18n("Trace Part Overview"), KShortcut(),
+  _partDockShown = new KToggleAction(i18n("Parts Overview"), KShortcut(),
                                      this, SLOT(togglePartDock()),
                                      actionCollection(),
                                      "settings_show_partdock");
 
-  hint = i18n("Show/Hide the Trace Part Overview Dockable");
+  hint = i18n("Show/Hide the Parts Overview Dockable");
   _partDockShown->setToolTip( hint );
   _partDockShown->setWhatsThis( hint );
 
-  _stackDockShown = new KToggleAction(i18n("Top Cost Call Stack"), KShortcut(),
+  _stackDockShown = new KToggleAction(i18n("Call Stack"), KShortcut(),
                                      this, SLOT(toggleStackDock()),
                                      actionCollection(),
                                      "settings_show_stackdock");
 
-  hint = i18n("Show/Hide the Top Call Stack Dockable");
+  hint = i18n("Show/Hide the Call Stack Dockable");
   _stackDockShown->setToolTip( hint );
   _stackDockShown->setWhatsThis( hint );
 
-  _functionDockShown = new KToggleAction(i18n("Flat Profile"), KShortcut(),
+  _functionDockShown = new KToggleAction(i18n("Function Profile"), KShortcut(),
                                          this, SLOT(toggleFunctionDock()),
                                          actionCollection(),
                                          "settings_show_profiledock");
 
-  hint = i18n("Show/Hide the Flat Profile Dockable");
+  hint = i18n("Show/Hide the Function Profile Dockable");
   _functionDockShown->setToolTip( hint );
   _functionDockShown->setWhatsThis( hint );
 
@@ -541,6 +622,10 @@ void TopLevel::createActions()
 				    this, SLOT(togglePercentage()),
 				    actionCollection(),
 				    "view_percentage");
+#if KDE_VERSION >= 0x030290
+  // for KDE 3.3: show another text instead of a checkmark
+  _taPercentage->setCheckedState(i18n("Show Absolute Costs"));
+#endif
 
   hint = i18n("Show relative instead of absolute costs");
   _taPercentage->setToolTip( hint );
@@ -571,23 +656,27 @@ void TopLevel::createActions()
 	      "<p>(*) Only if function grouping is switched on (e.g. ELF object grouping).");
   _taExpanded->setWhatsThis( hint );
 
-  _taCycles = new KToggleAction( i18n( "Show Cycles" ), "undo",
+  _taCycles = new KToggleAction( i18n( "Do Cycle Detection" ), "undo",
 				 KShortcut(),
 				 this, SLOT( toggleCycles() ), actionCollection(),
 				 "view_cycles" );
+#if KDE_VERSION >= 0x030290
+  // for KDE 3.3: show another text instead of a checkmark
+  _taCycles->setCheckedState(i18n("Skip Cycle Detection"));
+#endif
 
   hint = i18n("<b>Detect recursive cycles</b>"
               "<p>If this is switched off, the treemap drawing will show "
 	      "black areas when a recursive call is made instead of drawing the "
 	      "recursion ad infinitum. Note that "
-	      "the size of black areas often will be wrong, as inside of recursive "
-	      "cycles, you can't determine the cost of calls. However, the error is "
-	      "small for false cycles (see documentation)."
+	      "the size of black areas often will be wrong, as inside recursive "
+              "cycles the cost of calls cannot be determined; the error is small, "
+              "however, for false cycles (see documentation)."
 	      "<p>The correct handling for cycles is to detect them and collapse all "
-	      "functions of a cycle into a virtual function. This is done when this "
-	      "option is selected. Unfortunately with GUI applications, this often will "
-	      "lead to huge false cycles, making the analysis impossible. Therefore the "
-	      "possibility to switch this off.");
+              "functions of a cycle into a virtual function, which is done when this "
+              "option is selected. Unfortunately, with GUI applications, this often will "
+              "lead to huge false cycles, making the analysis impossible; therefore, there "
+              "is the option to switch this off.");
   _taCycles->setWhatsThis( hint );
 
   KStdAction::quit(this, SLOT(close()), actionCollection());
@@ -655,18 +744,28 @@ void TopLevel::createActions()
   _paForward->setWhatsThis( hint );
 #endif
 
-  saCost = new KSelectAction( i18n("Cost Type"), KShortcut(),
-                              actionCollection(), "view_cost_type");
-  hint = i18n("Select cost type of costs shown allover KCachegrind");
-  saCost->setComboWidth(300);
-  saCost->setToolTip( hint );
-  saCost->setWhatsThis( hint );
+  _saCost = new KSelectAction( i18n("Primary Event Type"), KShortcut(),
+			       actionCollection(), "view_cost_type");
+  hint = i18n("Select primary event type of costs");
+  _saCost->setComboWidth(300);
+  _saCost->setToolTip( hint );
+  _saCost->setWhatsThis( hint );
 
   // cost types are dependent on loaded data, thus KSelectAction
   // is filled in setData()
-  connect( saCost, SIGNAL(activated(const QString&)),
+  connect( _saCost, SIGNAL(activated(const QString&)),
            this, SLOT(costTypeSelected(const QString&)));
 
+  _saCost2 = new KSelectAction( i18n("Secondary Event Type"), KShortcut(),
+				actionCollection(), "view_cost_type2");
+  hint = i18n("Select secondary event type for cost e.g. shown in annotations");
+  _saCost2->setComboWidth(300);
+  _saCost2->setToolTip( hint );
+  _saCost2->setWhatsThis( hint );
+
+  connect( _saCost2, SIGNAL(activated(const QString&)),
+           this, SLOT(costType2Selected(const QString&)));
+
   saGroup = new KSelectAction( i18n("Grouping"), KShortcut(),
                                actionCollection(), "view_group_type");
 
@@ -699,7 +798,7 @@ void TopLevel::createActions()
                                   this, SLOT(splitDirSlot()),
                                   actionCollection(), "view_split_dir");
 
-  hint = i18n("Change Split Orientation when main window is splitted");
+  hint = i18n("Change Split Orientation when main window is split.");
   _taSplitDir->setToolTip( hint );
   _taSplitDir->setWhatsThis( hint );
 
@@ -735,6 +834,12 @@ void TopLevel::createActions()
 #endif
 }
 
+void TopLevel::createActions()
+{
+  createMiscActions();
+  createLayoutActions();
+}
+
 void TopLevel::toggleStatusBar()
 {
   if (statusBar()->isVisible())
@@ -779,9 +884,25 @@ void TopLevel::toggleFunctionDock()
 
 void TopLevel::togglePercentage()
 {
-  bool show = _taPercentage->isChecked();
+  setPercentage(_taPercentage->isChecked());
+}
+
+void TopLevel::setAbsoluteCost()
+{
+  setPercentage(false);
+}
+
+void TopLevel::setRelativeCost()
+{
+  setPercentage(true);
+}
+
+void TopLevel::setPercentage(bool show)
+{
   if (_showPercentage == show) return;
   _showPercentage = show;
+  if (_taPercentage->isChecked() != show)
+    _taPercentage->setChecked(show);
 
   // FIXME: Delete when no view gets this config from Configuration
   Configuration::setShowPercentage(_showPercentage);
@@ -817,7 +938,6 @@ void TopLevel::toggleExpanded()
 
 void TopLevel::toggleCycles()
 {
-    if (!_data) return;
   bool show = _taCycles->isChecked();
   if (_showCycles == show) return;
   _showCycles = show;
@@ -825,6 +945,8 @@ void TopLevel::toggleCycles()
   // FIXME: Delete when no view gets this config from Configuration
   Configuration::setShowCycles(_showCycles);
 
+  if (!_data) return;
+
   _data->invalidateDynamicCost();
   _data->updateFunctionCycles();
 
@@ -848,7 +970,11 @@ void TopLevel::stackVisibilityChanged(bool v)
   _stackDockShown->setChecked(v);
 }
 
+#if ENABLE_DUMPDOCK
 void TopLevel::dumpVisibilityChanged(bool v)
+#else
+void TopLevel::dumpVisibilityChanged(bool)
+#endif
 {
 #if ENABLE_DUMPDOCK
   _dumpDockShown->setChecked(v);
@@ -870,7 +996,7 @@ void TopLevel::querySlot()
 
 void TopLevel::configureKeys()
 {
-#if KDE_VERSION > 319
+#if KDE_VERSION > 0x030190
   // for KDE 3.2: KKeyDialog::configureKeys is deprecated
   KKeyDialog::configure(actionCollection(), this, true);
 #else
@@ -904,10 +1030,10 @@ void TopLevel::newWindow()
 
 void TopLevel::loadTrace()
 {
-    KURL url = KFileDialog::getOpenURL(QString::null,
-                                       i18n("cachegrind.out*|Cachegrind Trace"),
+    KURL url = KFileDialog::getOpenURL(":",
+                                       i18n("cachegrind.out* callgrind.out*|Callgrind Profile Data\n*|All Files"),
                                        this,
-                                       i18n("Select Cachegrind Trace"));
+                                       i18n("Select Callgrind Profile Data"));
     loadTrace(url);
 }
 
@@ -917,7 +1043,12 @@ void TopLevel::loadTrace(const KURL& url)
 
   // network transparancy
   QString tmpFile;
+#if KDE_VERSION > 0x030190
+  // for KDE 3.2: KIO::NetAccess::download with 2 args is deprecated
+  if(KIO::NetAccess::download( url, tmpFile, this )) {
+#else
   if(KIO::NetAccess::download( url, tmpFile )) {
+#endif
     _openRecent->addURL(url);
     _openRecent->saveEntries( KGlobal::config() );
 
@@ -946,6 +1077,55 @@ void TopLevel::loadTrace(QString file)
 }
 
 
+void TopLevel::addTrace()
+{
+    KURL url = KFileDialog::getOpenURL(QString::null,
+                                       i18n("cachegrind.out* callgrind.out*|Callgrind Profile Data\n*|All Files"),
+                                       this,
+                                       i18n("Add Callgrind Profile Data"));
+    addTrace(url);
+}
+
+void TopLevel::addTrace(const KURL& url)
+{
+  if (url.isEmpty()) return;
+
+  // network transparancy
+  QString tmpFile;
+#if KDE_VERSION > 0x030190
+  // for KDE 3.2: KIO::NetAccess::download with 2 args is deprecated
+  if(KIO::NetAccess::download( url, tmpFile, this )) {
+#else
+  if(KIO::NetAccess::download( url, tmpFile )) {
+#endif
+    _openRecent->addURL(url);
+    _openRecent->saveEntries( KGlobal::config() );
+
+    addTrace(tmpFile);
+    KIO::NetAccess::removeTempFile( tmpFile );
+  }
+}
+
+void TopLevel::addTrace(QString file)
+{
+  if (file.isEmpty()) return;
+
+  if (_data) {
+    _data->load(file);
+
+    // GUI update for added data
+    configChanged();
+    return;
+  }
+
+  // this constructor enables progress bar callbacks
+  TraceData* d = new TraceData(this);
+  d->load(file);
+  setData(d);
+}
+
+
+
 void TopLevel::loadDelayed(QString file)
 {
   _loadTraceDelayed = file;
@@ -980,7 +1160,7 @@ void TopLevel::exportGraph()
   if (!_data || !_function) return;
 
   QString n = QString("callgraph.dot");
-  GraphExporter ge(_data, _function, _costType, n);
+  GraphExporter ge(_data, _function, _costType, _groupType, n);
   ge.writeDot();
 
   QString cmd = QString("(dot %1 -Tps > %2.ps; kghostview %3.ps)&")
@@ -1001,6 +1181,16 @@ bool TopLevel::setCostType(QString s)
   return setCostType(ct);
 }
 
+bool TopLevel::setCostType2(QString s)
+{
+  TraceCostType* ct;
+
+  // Special type i18n("(Hidden)") gives 0
+  ct = (_data) ? _data->mapping()->type(s) : 0;
+
+  return setCostType2(ct);
+}
+
 void TopLevel::costTypeSelected(const QString& s)
 {
   TraceCostType* ct;
@@ -1009,6 +1199,14 @@ void TopLevel::costTypeSelected(const QString& s)
   setCostType(ct);
 }
 
+void TopLevel::costType2Selected(const QString& s)
+{
+  TraceCostType* ct;
+
+  ct = (_data) ? _data->mapping()->typeForLong(s) : 0;
+  setCostType2(ct);
+}
+
 bool TopLevel::setCostType(TraceCostType* ct)
 {
   if (_costType == ct) return false;
@@ -1016,20 +1214,48 @@ bool TopLevel::setCostType(TraceCostType* ct)
 
   if (ct) {
       int idx=0;
-      QStringList l = saCost->items();
+      QStringList l = _saCost->items();
       for (QStringList::Iterator it = l.begin(); it != l.end(); ++it, ++idx ) {
 	  if (*it == ct->longName())
-	      saCost->setCurrentItem(idx);
+	      _saCost->setCurrentItem(idx);
       }
   }
 
   _partSelection->setCostType(_costType);
   _stackSelection->setCostType(_costType);
 
-  _functionSelection->set(_costType);
+  _functionSelection->setCostType(_costType);
   _functionSelection->updateView();
 
-  _multiView->set(_costType);
+  _multiView->setCostType(_costType);
+  _multiView->updateView();
+
+  updateStatusBar();
+
+  return true;
+}
+
+bool TopLevel::setCostType2(TraceCostType* ct)
+{
+  if (_costType2 == ct) return false;
+  _costType2 = ct;
+
+  QString longName = ct ? ct->longName() : i18n("(Hidden)");
+
+  int idx=0;
+  QStringList l = _saCost2->items();
+  for (QStringList::Iterator it = l.begin(); it != l.end(); ++it, ++idx ) {
+    if (*it == longName)
+      _saCost2->setCurrentItem(idx);
+  }
+
+  //_partSelection->setCostType2(_costType2);
+  _stackSelection->setCostType2(_costType2);
+
+  _functionSelection->setCostType2(_costType2);
+  _functionSelection->updateView();
+
+  _multiView->setCostType2(_costType2);
   _multiView->updateView();
 
   updateStatusBar();
@@ -1185,11 +1411,22 @@ void TopLevel::setCostTypeDelayed(TraceCostType* ct)
   QTimer::singleShot (0, this, SLOT(setCostTypeDelayed()));
 }
 
+void TopLevel::setCostType2Delayed(TraceCostType* ct)
+{
+  _costType2Delayed = ct;
+  QTimer::singleShot (0, this, SLOT(setCostType2Delayed()));
+}
+
 void TopLevel::setCostTypeDelayed()
 {
   setCostType(_costTypeDelayed);
 }
 
+void TopLevel::setCostType2Delayed()
+{
+  setCostType2(_costType2Delayed);
+}
+
 void TopLevel::setGroupTypeDelayed(TraceItem::CostType gt)
 {
   _groupTypeDelayed = gt;
@@ -1304,7 +1541,7 @@ void TopLevel::setTraceItemDelayed()
 }
 
 /**
- * A TraceData object can't be viewed many times in different
+ * A TraceData object cannot be viewed many times in different
  * toplevel windows. Thus, this toplevel window takes ownership
  * of the TraceData object: on closing the window or opening
  * another trace, the object is destroyed.
@@ -1349,20 +1586,31 @@ void TopLevel::setData(TraceData* data)
       for (int i=0;i<m->virtualCount();i++)
 	  types << m->virtualType(i)->longName();
   }
-  saCost->setItems(types);
-  saCost->setComboWidth(300);
+  _saCost->setItems(types);
+  _saCost->setComboWidth(300);
 
-  /* this is needed to let the other widgets know the types */
-  restoreTraceTypes();
+  if (types.count()>0) {
+    // second type list gets an additional "(Hidden)"
+    types.prepend(i18n("(Hidden)"));
+  }
+  _saCost2->setItems(types);
+  _saCost2->setComboWidth(300);
+  // default is hidden
+  if (types.count()>0)
+    _saCost2->setCurrentItem(0);
 
   _partSelection->setData(_data);
   _stackSelection->setData(_data);
-
   _functionSelection->setData(_data);
   _functionSelection->updateView();
   _multiView->setData(_data);
   _multiView->updateView();
 
+  /* this is needed to let the other widgets know the types */
+  restoreTraceTypes();
+
+  restoreTraceSettings();
+
   QString caption;
   if (_data) {
     caption = _data->traceName();
@@ -1371,8 +1619,6 @@ void TopLevel::setData(TraceData* data)
   }
   setCaption(caption);
 
-  restoreTraceSettings();
-
   if (!_data || (!_forcePartDock && _data->parts().count()<2)) {
     _partDock->hide();
     _partDockShown->setChecked(false);
@@ -1385,6 +1631,104 @@ void TopLevel::setData(TraceData* data)
   updateStatusBar();
 }
 
+void TopLevel::addCostMenu(QPopupMenu* popup, bool withCost2)
+{
+  if (_data) {
+    QPopupMenu *popup1 = new QPopupMenu(popup);
+    QPopupMenu *popup2 = 0;
+    popup1->setCheckable(true);
+
+    if (withCost2) {
+      popup2 = new QPopupMenu(popup);
+      popup2->setCheckable(true);
+
+      if (_costType2) {
+	popup2->insertItem(i18n("Hide"),199);
+	popup2->insertSeparator();
+      }
+    }
+
+    TraceCostMapping* m = _data->mapping();
+    TraceCostType* ct;
+    for (int i=0;i<m->realCount();i++) {
+      ct = m->realType(i);
+      popup1->insertItem(ct->longName(), 100+i);
+      if (_costType == ct) popup1->setItemChecked(100+i,true);
+      if (popup2) {
+	popup2->insertItem(ct->longName(), 100+i);
+	if (_costType2 == ct) popup2->setItemChecked(100+i,true);
+      }
+    }
+    for (int i=0;i<m->virtualCount();i++) {
+      ct = m->virtualType(i);
+      popup1->insertItem(ct->longName(), 200+i);
+      if (_costType == ct) popup1->setItemChecked(200+i,true);
+      if (popup2) {
+	popup2->insertItem(ct->longName(), 200+i);
+	if (_costType2 == ct) popup2->setItemChecked(200+i,true);
+      }
+    }
+    popup->insertItem(i18n("Primary Event Type"), popup1);
+    connect(popup1,SIGNAL(activated(int)),this,SLOT(setCostType(int)));
+    if (popup2) {
+      popup->insertItem(i18n("Secondary Event Type"), popup2);
+      connect(popup2,SIGNAL(activated(int)),this,SLOT(setCostType2(int)));
+    }
+  }
+  if (_showPercentage)
+    popup->insertItem(i18n("Show Absolute Cost"),
+		      this, SLOT(setAbsoluteCost()));
+  else
+    popup->insertItem(i18n("Show Relative Cost"),
+		      this, SLOT(setRelativeCost()));
+}
+
+bool TopLevel::setCostType(int id)
+{
+  if (!_data) return false;
+
+  TraceCostMapping* m = _data->mapping();
+  TraceCostType* ct=0;
+  if (id >=100 && id<199) ct = m->realType(id-100);
+  if (id >=200 && id<299) ct = m->virtualType(id-200);
+
+  return ct ? setCostType(ct) : false;
+}
+
+bool TopLevel::setCostType2(int id)
+{
+  if (!_data) return false;
+
+  TraceCostMapping* m = _data->mapping();
+  TraceCostType* ct=0;
+  if (id >=100 && id<199) ct = m->realType(id-100);
+  if (id >=200 && id<299) ct = m->virtualType(id-200);
+
+  return setCostType2(ct);
+}
+
+void TopLevel::addGoMenu(QPopupMenu* popup)
+{
+  popup->insertItem(i18n("Go Back"), this, SLOT(goBack()));
+  popup->insertItem(i18n("Go Forward"), this, SLOT(goForward()));
+  popup->insertItem(i18n("Go Up"), this, SLOT(goUp()));
+}
+
+void TopLevel::goBack()
+{
+  setDirectionDelayed(TraceItemView::Back);
+}
+
+void TopLevel::goForward()
+{
+  setDirectionDelayed(TraceItemView::Forward);
+}
+
+void TopLevel::goUp()
+{
+  setDirectionDelayed(TraceItemView::Up);
+}
+
 QString TopLevel::traceKey()
 {
   if (!_data || _data->command().isEmpty()) return QString::null;
@@ -1405,20 +1749,28 @@ void TopLevel::restoreTraceTypes()
   KConfigGroup cConfig(KGlobal::config(), QCString("CurrentState"));
   KConfigGroup pConfig(KGlobal::config(), QCString("TracePositions"));
 
-  QString groupType, costType;
+  QString groupType, costType, costType2;
   groupType =  pConfig.readEntry(QString("GroupType%1").arg(key));
   costType  =  pConfig.readEntry(QString("CostType%1").arg(key));
+  costType2 =  pConfig.readEntry(QString("CostType2%1").arg(key));
 
   if (groupType.isEmpty()) groupType = cConfig.readEntry("GroupType");
   if (costType.isEmpty()) costType = cConfig.readEntry("CostType");
+  if (costType2.isEmpty()) costType2 = cConfig.readEntry("CostType2");
 
   setGroupType(groupType);
   setCostType(costType);
+  setCostType2(costType2);
 
   // if still no cost type set, use first available
-  if (!_costType && !saCost->items().isEmpty())
-      costTypeSelected(saCost->items().first());
+  if (!_costType && !_saCost->items().isEmpty())
+      costTypeSelected(_saCost->items().first());
 
+  KConfigGroup aConfig(KGlobal::config(), QCString("Layouts"));
+  _layoutCount = aConfig.readNumEntry(QString("Count%1").arg(key), 0);
+  _layoutCurrent = aConfig.readNumEntry(QString("Current%1").arg(key), 0);
+  if (_layoutCount == 0) layoutRestore();
+  updateLayoutActions();
 }
 
 
@@ -1449,10 +1801,156 @@ void TopLevel::restoreTraceSettings()
 }
 
 
+/* Layout */
+
+void TopLevel::layoutDuplicate()
+{
+  // save current and allocate a new slot
+  _multiView->saveViewConfig(KGlobal::config(),
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     traceKey(), false);
+  _layoutCurrent = _layoutCount;
+  _layoutCount++;
+
+  updateLayoutActions();
+
+  kdDebug() << "TopLevel::layoutDuplicate: count " << _layoutCount << endl;
+}
+
+void TopLevel::layoutRemove()
+{
+  if (_layoutCount <2) return;
+
+  int from = _layoutCount-1;
+  if (_layoutCurrent == from) { _layoutCurrent--; from--; }
+  // restore from last and decrement count
+  _multiView->readViewConfig(KGlobal::config(),
+			     QString("Layout%1-MainView").arg(from),
+			     traceKey(), false);
+  _layoutCount--;
+
+  updateLayoutActions();
+}
+
+void TopLevel::layoutNext()
+{
+  if (_layoutCount <2) return;
+
+  KConfig* config = KGlobal::config();
+  QString key = traceKey();
+
+  _multiView->saveViewConfig(config,
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     key, false);
+  _layoutCurrent++;
+  if (_layoutCurrent == _layoutCount) _layoutCurrent = 0;
+
+  _multiView->readViewConfig(config,
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     key, false);
+
+  if (0) kdDebug() << "TopLevel::layoutNext: current "
+		   << _layoutCurrent << endl;
+}
+
+void TopLevel::layoutPrevious()
+{
+  if (_layoutCount <2) return;
+
+  KConfig* config = KGlobal::config();
+  QString key = traceKey();
+
+  _multiView->saveViewConfig(config,
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     key, false);
+  _layoutCurrent--;
+  if (_layoutCurrent <0) _layoutCurrent = _layoutCount-1;
+
+  _multiView->readViewConfig(config,
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     key, false);
+
+  if (0) kdDebug() << "TopLevel::layoutPrevious: current "
+		   << _layoutCurrent << endl;
+}
+
+void TopLevel::layoutSave()
+{
+  KConfig* config = KGlobal::config();
+  QString key = traceKey();
+
+  _multiView->saveViewConfig(config,
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     key, false);
+
+  for(int i=0;i<_layoutCount;i++) {
+    _multiView->readViewConfig(config,
+			       QString("Layout%1-MainView").arg(i),
+			       key, false);
+    _multiView->saveViewConfig(config,
+			       QString("Layout%1-MainView").arg(i),
+			       QString(), false);
+  }
+
+  _multiView->readViewConfig(config,
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     key, false);
+
+  KConfigGroup aConfig(config, QCString("Layouts"));
+  aConfig.writeEntry("DefaultCount", _layoutCount);
+  aConfig.writeEntry("DefaultCurrent", _layoutCurrent);
+}
+
+void TopLevel::layoutRestore()
+{
+  KConfig* config = KGlobal::config();
+  KConfigGroup aConfig(config, QCString("Layouts"));
+  _layoutCount = aConfig.readNumEntry("DefaultCount", 0);
+  _layoutCurrent = aConfig.readNumEntry("DefaultCurrent", 0);
+  if (_layoutCount == 0) {
+    _layoutCount++;
+    return;
+  }
+
+  QString key = traceKey();
+  for(int i=0;i<_layoutCount;i++) {
+    _multiView->readViewConfig(config,
+			       QString("Layout%1-MainView").arg(i),
+			       QString(), false);
+    _multiView->saveViewConfig(config,
+			       QString("Layout%1-MainView").arg(i),
+			       key, false);
+  }
+
+  _multiView->readViewConfig(config,
+			     QString("Layout%1-MainView").arg(_layoutCurrent),
+			     key, false);
+
+  updateLayoutActions();
+}
+
+
+void TopLevel::updateLayoutActions()
+{
+  KAction* ka;
+
+  ka = actionCollection()->action("layout_next");
+  if (ka) ka->setEnabled(_layoutCount>1);
+
+  ka = actionCollection()->action("layout_previous");
+  if (ka) ka->setEnabled(_layoutCount>1);
+
+  ka = actionCollection()->action("layout_remove");
+  if (ka) ka->setEnabled(_layoutCount>1);
+
+  _statusbar->message(i18n("Layout Count: %1").arg(_layoutCount), 1000);
+}
+
+
 void TopLevel::updateStatusBar()
 {
   if (!_data || _data->parts().count()==0) {
-    _statusLabel->setText(i18n("No trace file loaded."));
+    _statusLabel->setText(i18n("No profile data file loaded."));
     return;
   }
 
@@ -1460,18 +1958,29 @@ void TopLevel::updateStatusBar()
                    .arg(_data->shortTraceName())
                    .arg(_data->activePartRange());
 
-  if (_costType)
+  if (_costType) {
     status += i18n("Total %1 Cost: %2")
       .arg(_costType->longName())
       .arg(_data->prettySubCost(_costType));
-  else
-    status += i18n("No cost type selected");
 
+    /* this gets too long...
+    if (_costType2 && (_costType2 != _costType))
+      status += i18n(", %1 Cost: %2")
+	.arg(_costType2->longName())
+	.arg(_data->prettySubCost(_costType2));
+    */
+  }
+  else
+    status += i18n("No event type selected");
+
+  /* Not working... should give group of selected function
+  
   if (_groupType != TraceItem::Function) {
     status += QString(" - %1 '%2'")
               .arg(TraceItem::i18nTypeName(_groupType))
               .arg(_group ? _group->prettyName() : i18n("(None)"));
   }
+  */
 
   _statusLabel->setText(status);
 }
@@ -1667,7 +2176,8 @@ void TopLevel::forceTrace()
 {
 //  qDebug("forceTrace");
 
-  QFile cmd("cachegrind.cmd");
+  // Needs Callgrind now...
+  QFile cmd("callgrind.cmd");
   if (!cmd.exists()) {
     cmd.open(IO_WriteOnly);
     cmd.writeBlock("DUMP\n", 5);
@@ -1686,7 +2196,7 @@ void TopLevel::forceTraceReload()
 {
 //  qDebug("forceTraceReload");
 
-  QFile cmd("cachegrind.cmd");
+  QFile cmd("callgrind.cmd");
   if (cmd.exists()) {
     if (_taDump->isChecked())
       QTimer::singleShot( 1000, this, SLOT(forceTraceReload()) );
@@ -1834,8 +2344,8 @@ void TopLevel::upActivated(int id)
 
   StackBrowser* b = _stackSelection ? _stackSelection->browser() : 0;
   HistoryItem* hi = b ? b->current() : 0;
-  if (  !hi )
-      return;
+  if (!hi) return;
+
   TraceFunction* f = hi->function();
 
   while (id>0 && f) {
