@@ -4,8 +4,10 @@
         begin                   : Sun Mar 4 2001
         copyright               : (C) 2001-2004 Otto Bruggeman
                                   (C) 2001-2003 John Firebaugh
+                                  (C) 2004      Jeff Snyder
         email                   : otto.bruggeman@home.nl
                                   jfirebaugh@kde.org
+                                  jeff@caffeinated.me.uk
 ****************************************************************************/
 
 /***************************************************************************
@@ -17,10 +19,13 @@
 **
 ***************************************************************************/
 
+#include "kompare_qsplitter.h" // make sure we get there first
+
 #include <qlayout.h>
 #include <qwidget.h>
 
 #include <kaction.h>
+#include <kapplication.h>
 #include <kdebug.h>
 #include <kfiletreeview.h>
 #include <kfiledialog.h>
@@ -29,26 +34,30 @@
 #include <kstdaction.h>
 #include <kinstance.h>
 #include <ktempfile.h>
+#include <kparts/genericfactory.h>
 //#include <ktempdir.h>
 
 #include <kio/netaccess.h>
 
 #include "diffmodel.h"
-#include "kompare_part.h"
 #include "komparelistview.h"
 #include "kompareconnectwidget.h"
 #include "diffsettings.h"
 #include "viewsettings.h"
 #include "kompareprefdlg.h"
 #include "komparesaveoptionswidget.h"
-#include "kompareview.h"
+#include "komparesplitter.h"
 
+#include "kompare_part.h"
+
+typedef KParts::GenericFactory<KomparePart> KomparePartFactory;
+K_EXPORT_COMPONENT_FACTORY( libkomparepart, KomparePartFactory )
 
 ViewSettings* KomparePart::m_viewSettings = 0L;
 DiffSettings* KomparePart::m_diffSettings = 0L;
 
 KomparePart::KomparePart( QWidget *parentWidget, const char *widgetName,
-                      QObject *parent, const char *name ) :
+                          QObject *parent, const char *name, const QStringList & /*args*/ ) :
 	KParts::ReadWritePart(parent, name),
 	m_tempDiff( 0 ),
 	m_info()
@@ -63,8 +72,10 @@ KomparePart::KomparePart( QWidget *parentWidget, const char *widgetName,
 		m_diffSettings = new DiffSettings( 0 );
 	}
 
+	readProperties( kapp->config() );
+
 	// This creates the "Model creator" and connects the signals and slots
-	m_modelList = new Diff2::KompareModelList( m_diffSettings, &m_info, this, "komparemodellist" );
+	m_modelList = new Diff2::KompareModelList( m_diffSettings, m_info, this, "komparemodellist" );
 	connect( m_modelList, SIGNAL(status( Kompare::Status )),
 	         this, SLOT(slotSetStatus( Kompare::Status )) );
 	connect( m_modelList, SIGNAL(setStatusBarModelInfo( int, int, int, int, int )),
@@ -84,8 +95,8 @@ KomparePart::KomparePart( QWidget *parentWidget, const char *widgetName,
 	         this, SLOT( slotSetModified( bool ) ) );
 
 	// This is the stuff to connect the "interface" of the kompare part to the model inside
-	connect( m_modelList, SIGNAL(modelsChanged(const QPtrList<Diff2::DiffModel>*)),
-	         this, SIGNAL(modelsChanged(const QPtrList<Diff2::DiffModel>*)) );
+	connect( m_modelList, SIGNAL(modelsChanged(const Diff2::DiffModelList*)),
+	         this, SIGNAL(modelsChanged(const Diff2::DiffModelList*)) );
 
 	connect( m_modelList, SIGNAL(setSelection(const Diff2::DiffModel*, const Diff2::Difference*)),
 	         this, SIGNAL(setSelection(const Diff2::DiffModel*, const Diff2::Difference*)) );
@@ -104,31 +115,31 @@ KomparePart::KomparePart( QWidget *parentWidget, const char *widgetName,
 	connect( m_modelList, SIGNAL(applyDifference(const Diff2::Difference*, bool)),
 	         this, SIGNAL(applyDifference(const Diff2::Difference*, bool)) );
 
-	// This creates the viewwidget and connects the signals and slots
-	m_diffView = new KompareView( m_viewSettings, parentWidget, widgetName );
+	// This creates the splitterwidget and connects the signals and slots
+	m_splitter = new KompareSplitter ( m_viewSettings, parentWidget, widgetName );
 
 	connect( m_modelList, SIGNAL(setSelection(const Diff2::DiffModel*, const Diff2::Difference*)),
-	         m_diffView, SLOT(slotSetSelection(const Diff2::DiffModel*, const Diff2::Difference*)) );
+	         m_splitter,  SLOT(slotSetSelection(const Diff2::DiffModel*, const Diff2::Difference*)) );
+//	connect( m_splitter,  SIGNAL(selectionChanged(const Diff2::Difference*, const Diff2::Difference*)),
+//	         m_modelList, SLOT(slotSelectionChanged(const Diff2::Difference*, const Diff2::Difference*)) );
 	connect( m_modelList, SIGNAL(setSelection(const Diff2::Difference*)),
-	         m_diffView, SLOT(slotSetSelection(const Diff2::Difference*)) );
-	connect( m_diffView, SIGNAL(selectionChanged(const Diff2::Difference*)),
+	         m_splitter,  SLOT(slotSetSelection(const Diff2::Difference*)) );
+	connect( m_splitter,  SIGNAL(selectionChanged(const Diff2::Difference*)),
 	         m_modelList, SLOT(slotSelectionChanged(const Diff2::Difference*)) );
 
 	connect( m_modelList, SIGNAL(applyDifference(bool)),
-	         m_diffView, SLOT(slotApplyDifference(bool)) );
+	         m_splitter, SLOT(slotApplyDifference(bool)) );
 	connect( m_modelList, SIGNAL(applyAllDifferences(bool)),
-	         m_diffView, SLOT(slotApplyAllDifferences(bool)) );
+	         m_splitter, SLOT(slotApplyAllDifferences(bool)) );
 	connect( m_modelList, SIGNAL(applyDifference(const Diff2::Difference*, bool)),
-	         m_diffView, SLOT(slotApplyDifference(const Diff2::Difference*, bool)) );
-	connect( this, SIGNAL(configChanged()), m_diffView, SLOT(slotConfigChanged()) );
+	         m_splitter, SLOT(slotApplyDifference(const Diff2::Difference*, bool)) );
+	connect( this, SIGNAL(configChanged()), m_splitter, SIGNAL(configChanged()) );
 
 	// notify the part that this is our internal widget
-	setWidget( m_diffView );
+	setWidget( m_splitter->parentWidget() );
 
 	setupActions();
 
-	readProperties( instance()->config() );
-
 	// set our XML-UI resource file
 	setXMLFile( "komparepartui.rc" );
 
@@ -142,6 +153,9 @@ KomparePart::KomparePart( QWidget *parentWidget, const char *widgetName,
 
 KomparePart::~KomparePart()
 {
+	// This is the only place allowed to call cleanUpTemporaryFiles
+	// because before there might still be a use for them (when swapping)
+	cleanUpTemporaryFiles();
 }
 
 void KomparePart::setupActions()
@@ -151,7 +165,7 @@ void KomparePart::setupActions()
 	m_saveAll         = new KAction( i18n("Save &All"), "save_all", 0,
 	                                 this, SLOT(saveAll()),
 	                                 actionCollection(), "file_save_all" );
-	m_saveDiff        = new KAction( i18n("Save .&diff"), 0,
+	m_saveDiff        = new KAction( i18n("Save .&diff..."), 0,
 	                                 this, SLOT(saveDiff()),
 	                                 actionCollection(), "file_save_diff" );
 	m_swap            = new KAction( i18n( "Swap Source with Destination" ), 0,
@@ -172,6 +186,12 @@ void KomparePart::updateActions()
 	m_diffStats->setEnabled( m_modelList->modelCount() > 0 );
 }
 
+void KomparePart::setEncoding( const QString& encoding )
+{
+	kdDebug() << "Encoding: " << encoding << endl;
+	m_modelList->setEncoding( encoding );
+}
+
 bool KomparePart::openDiff( const KURL& url )
 {
 	kdDebug(8103) << "Url = " << url.url() << endl;
@@ -236,64 +256,60 @@ bool KomparePart::openDiff3( const QString& diff3Output )
 bool KomparePart::exists( const QString& url )
 {
 	QFileInfo fi( url );
-	if ( fi.exists() )
-		return true;
-	else
-		return false;
+	return fi.exists();
 }
 
-const QString& KomparePart::fetchURL( const KURL& url )
+const QString KomparePart::fetchURL( const KURL& url )
 {
-	QString* tempFile = new QString( "" );
+	QString tempFileName( "" );
 	if ( !url.isLocalFile() )
 	{
-		if ( ! KIO::NetAccess::download( url, *tempFile, widget() ) )
+		if ( ! KIO::NetAccess::download( url, tempFileName, widget() ) )
 		{
-			// FIXME: %1 should be bold
-			slotShowError( i18n( "The url %1 can not be downloaded." ).arg( url.prettyURL() ) );
-			*tempFile = "";
-			return *tempFile;
+			slotShowError( i18n( "<qt>The URL <b>%1</b> cannot be downloaded.</qt>" ).arg( url.prettyURL() ) );
+			tempFileName = "";
 		}
-		else
-			return *tempFile;
+		return tempFileName;
 	}
 	else
 	{
 		// is Local already, check if exists
 		if ( exists( url.path() ) )
-		{
-			*tempFile = url.path();
-			return *tempFile;
-		}
+			return url.path();
 		else
 		{
-			// FIXME: %1 should be bold
-			slotShowError( i18n( "The url %1 does not exist on your system." ).arg( url.prettyURL() ) );
-			return *tempFile;
+			slotShowError( i18n( "<qt>The URL <b>%1</b> does not exist on your system.</qt>" ).arg( url.prettyURL() ) );
+			return tempFileName;
 		}
 	}
 }
 
+void KomparePart::cleanUpTemporaryFiles()
+{
+	// i hope a local file will not be removed if it was not downloaded...
+	if ( !m_info.localSource.isEmpty() )
+		KIO::NetAccess::removeTempFile( m_info.localSource );
+	if ( !m_info.localDestination.isEmpty() )
+		KIO::NetAccess::removeTempFile( m_info.localDestination );
+}
+
 void KomparePart::compare( const KURL& source, const KURL& destination )
 {
-	emit kompareInfo( &m_info );
-
 	m_info.source = source;
 	m_info.destination = destination;
 
-	if ( ( m_info.localSource = fetchURL( source ) ).isEmpty() )
-	{
-		return;
-	}
+	m_info.localSource = fetchURL( source );
+	m_info.localDestination = fetchURL( destination );
 
-	if ( ( m_info.localDestination = fetchURL( destination ) ).isEmpty() )
-	{
-		// i hope a local file will not be removed if it was not downloaded...
-		KIO::NetAccess::removeTempFile( m_info.localSource );
-		return;
-	}
+	emit kompareInfo( &m_info );
 
-	m_modelList->compare( m_info.localSource, m_info.localDestination );
+	if ( !m_info.localSource.isEmpty() && !m_info.localDestination.isEmpty() )
+	{
+		m_modelList->compare( m_info.localSource, m_info.localDestination );
+		updateActions();
+		updateCaption();
+		updateStatus();
+	}
 }
 
 void KomparePart::compareFiles( const KURL& sourceFile, const KURL& destinationFile )
@@ -315,7 +331,6 @@ void KomparePart::compareFiles( const KURL& sourceFile, const KURL& destinationF
 		updateCaption();
 		updateStatus();
 	}
-	// Clean up needed ???
 }
 
 void KomparePart::compareDirs( const KURL& sourceDirectory, const KURL& destinationDirectory )
@@ -337,7 +352,6 @@ void KomparePart::compareDirs( const KURL& sourceDirectory, const KURL& destinat
 		updateCaption();
 		updateStatus();
 	}
-	// Clean up needed ???
 }
 
 void KomparePart::compare3Files( const KURL& /*originalFile*/, const KURL& /*changedFile1*/, const KURL& /*changedFile2*/ )
@@ -352,76 +366,47 @@ void KomparePart::openFileAndDiff( const KURL& file, const KURL& diffFile )
 {
 	emit kompareInfo( &m_info );
 
-	if ( ( m_info.localSource = fetchURL( file ) ).isEmpty() )
-		return;
-
-	if ( ( m_info.localDestination = fetchURL( diffFile ) ).isEmpty() )
-	{
-		KIO::NetAccess::removeTempFile( m_info.localSource );
-		return;
-	}
+	m_info.source = file;
+	m_info.destination = diffFile;
 
+	m_info.localSource = fetchURL( file );
+	m_info.localDestination = fetchURL( diffFile );
 	m_info.mode = Kompare::BlendingFile;
 
-	if ( m_modelList->openFileAndDiff( m_info.localSource, m_info.localDestination ) )
+	if ( !m_info.localSource.isEmpty() && !m_info.localDestination.isEmpty() )
 	{
-		kdDebug(8103) << "File merged with diff output..." << endl;
+		m_modelList->openFileAndDiff( m_info.localSource, m_info.localDestination );
+		updateActions();
+		updateCaption();
+		updateStatus();
 	}
-	else
-	{
-		kdDebug(8103) << "Could not merge file with diff output..." << endl;
-	}
-
-	// Clean up after ourselves
-	KIO::NetAccess::removeTempFile( m_info.localSource );
-	KIO::NetAccess::removeTempFile( m_info.localDestination );
 }
 
 void KomparePart::openDirAndDiff ( const KURL& dir,  const KURL& diffFile )
 {
 	emit kompareInfo( &m_info );
 
-	if ( ( m_info.localSource = fetchURL( dir ) ).isEmpty() )
-		return;
-
-	if ( ( m_info.localDestination = fetchURL( diffFile ) ).isEmpty() )
-	{
-		KIO::NetAccess::removeTempFile( m_info.localSource );
-		return;
-	}
+	m_info.source = dir;
+	m_info.destination = diffFile;
 
+	m_info.localSource = fetchURL( dir );
+	m_info.localDestination = fetchURL( diffFile );
 	m_info.mode = Kompare::BlendingDir;
 
-	// Still need to show the models even if something went wrong,
-	// kompare will then act as if openDiff was called
-	m_modelList->openDirAndDiff( m_info.localSource, m_info.localDestination );
-
-	m_modelList->show();
-
-	// Clean up after ourselves
-	KIO::NetAccess::removeTempFile( m_info.localSource );
-	KIO::NetAccess::removeTempFile( m_info.localDestination );
-}
-
-QString KomparePart::readFile()
-{
-	QFile file( m_file );
-	file.open( IO_ReadOnly );
-	QTextStream stream( &file );
-
-	kdDebug(8103) << "Reading from m_file = " << m_file << endl;
-
-	QString contents = stream.read();
-
-	file.close();
-
-	return contents;
+	if ( !m_info.localSource.isEmpty() && !m_info.localDestination.isEmpty() )
+	{
+		m_modelList->openDirAndDiff( m_info.localSource, m_info.localDestination );
+		updateActions();
+		updateCaption();
+		updateStatus();
+	}
 }
 
 bool KomparePart::openFile()
 {
 	// This is called from openURL
-	openDiff( readFile() );
+	// This is a little inefficient but i will do it anyway
+	openDiff( KURL( m_file ) );
 	return true;
 }
 
@@ -451,25 +436,52 @@ void KomparePart::saveDiff()
 		KConfig* config = instance()->config();
 		saveProperties( config );
 		config->sync();
-		KURL url = KFileDialog::getSaveURL( m_info.destination.url(),
-		              i18n("*.diff *.dif *.patch|Patch files"), widget(), i18n( "Save .diff" ) );
-		// FIXME: if url is remote we still need to upload it
-		kdDebug(8103) << "URL = " << url.prettyURL() << endl;
-		kdDebug(8103) << "Directory = " << w->directory() << endl;
-		kdDebug(8103) << "DiffSettings = " << m_diffSettings << endl;
 
-		m_modelList->saveDiff( url.url(), w->directory(), m_diffSettings );
+		while ( 1 )
+		{
+			KURL url = KFileDialog::getSaveURL( m_info.destination.url(),
+			              i18n("*.diff *.dif *.patch|Patch Files"), widget(), i18n( "Save .diff" ) );
+			if ( KIO::NetAccess::exists( url, false, widget() ) )
+			{
+				int result = KMessageBox::warningYesNoCancel( widget(), i18n("The file exists or is write-protected; do you want to overwrite it?"), i18n("File Exists") );
+				if ( result == KMessageBox::Cancel )
+				{
+					break;
+				}
+				else if ( result == KMessageBox::No )
+				{
+					continue;
+				}
+				else
+				{
+					kdDebug(8103) << "URL = " << url.prettyURL() << endl;
+					kdDebug(8103) << "Directory = " << w->directory() << endl;
+					kdDebug(8103) << "DiffSettings = " << m_diffSettings << endl;
+
+					m_modelList->saveDiff( url.url(), w->directory(), m_diffSettings );
+					break;
+				}
+			}
+			else
+			{
+				kdDebug(8103) << "URL = " << url.prettyURL() << endl;
+				kdDebug(8103) << "Directory = " << w->directory() << endl;
+				kdDebug(8103) << "DiffSettings = " << m_diffSettings << endl;
+
+				m_modelList->saveDiff( url.url(), w->directory(), m_diffSettings );
+				break;
+			}
+		}
 	}
 	delete dlg;
 }
 
-KURL KomparePart::diffURL()
+KAboutData *KomparePart::createAboutData()
 {
-	// This should just call url from the ReadOnlyPart, or leave it out completely
-	if( !m_info.source.isEmpty() ) {
-		saveDiff(); // Why are we saving here ???
-	}
-	return m_info.source;
+    KAboutData *about = new KAboutData("kompare", I18N_NOOP("KomparePart"), "3.2");
+    about->addAuthor("John Firebaugh", "Author", "jfirebaugh@kde.org");
+    about->addAuthor("Otto Bruggeman", "Author", "otto.bruggeman@home.nl" );
+    return about;
 }
 
 void KomparePart::slotSetStatus( enum Kompare::Status status )
@@ -571,7 +583,7 @@ void KomparePart::slotSwap()
 		int query = KMessageBox::warningYesNoCancel
 		            (
 		                widget(),
-		                i18n( "You have made changes to the destination.\n"
+		                i18n( "You have made changes to the destination file(s).\n"
 		                      "Would you like to save them?" ),
 		                i18n(  "Save Changes?" ),
 		                KStdGuiItem::save(),
@@ -694,7 +706,7 @@ bool KomparePart::queryClose()
 	int query = KMessageBox::warningYesNoCancel
 	            (
 	                widget(),
-	                i18n("You have made changes to the destination.\n"
+	                i18n("You have made changes to the destination file(s).\n"
 	                     "Would you like to save them?" ),
 	                i18n( "Save Changes?" ),
 	                KStdGuiItem::save(),
@@ -712,20 +724,16 @@ bool KomparePart::queryClose()
 
 int KomparePart::readProperties( KConfig *config )
 {
-	config->setGroup( "View" );
 	m_viewSettings->loadSettings( config );
-	config->setGroup( "DiffSettings" );
-	m_diffSettings->loadSettings   ( config );
+	m_diffSettings->loadSettings( config );
 	emit configChanged();
 	return 0;
 }
 
 int KomparePart::saveProperties( KConfig *config )
 {
-	config->setGroup( "View" );
 	m_viewSettings->saveSettings( config );
-	config->setGroup( "DiffSettings" );
-	m_diffSettings->saveSettings   ( config );
+	m_diffSettings->saveSettings( config );
 	return 0;
 }
 
@@ -734,78 +742,18 @@ void KomparePart::optionsPreferences()
 	// show preferences
 	KomparePrefDlg* pref = new KomparePrefDlg( m_viewSettings, m_diffSettings );
 
-	if ( pref->exec() ) {
-		KConfig* config = instance()->config();
-		saveProperties( config );
-		config->sync();
-		//FIXME: maybe this signal should also be emitted when
-		// Apply is pressed. I'll figure it out this week.
+	connect( pref, SIGNAL(applyClicked()), this, SIGNAL(configChanged()) );
+
+	if ( pref->exec() )
 		emit configChanged();
-	}
 }
 
 void KomparePart::slotSetModified( bool modified )
-{
+{ 
 	kdDebug() << "KomparePart::slotSetModified( " << modified << " );" << endl;
 	setModified( modified );
 	updateActions();
 	updateCaption();
 }
 
-// It's usually safe to leave the factory code alone.. with the
-// notable exception of the KAboutData data
-#include <kaboutdata.h>
-
-KInstance*  KomparePartFactory::s_instance = 0L;
-KAboutData* KomparePartFactory::s_about = 0L;
-
-KomparePartFactory::KomparePartFactory()
-    : KParts::Factory()
-{
-}
-
-KomparePartFactory::~KomparePartFactory()
-{
-	delete s_instance;
-	delete s_about;
-
-	s_instance = 0L;
-}
-
-KParts::Part* KomparePartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
-                                                  QObject *parent, const char *name,
-                                                  const char *classname, const QStringList & /*args*/ )
-{
-	// Create an instance of our Part
-	KomparePart* obj = new KomparePart( parentWidget, widgetName, parent, name );
-
-	// See if we are to be read-write or not
-	if (QCString(classname) == "KParts::ReadOnlyPart")
-		obj->setReadWrite(false);
-
-	KGlobal::locale()->insertCatalogue("kompare");
-
-	return obj;
-}
-
-KInstance* KomparePartFactory::instance()
-{
-	if( !s_instance )
-	{
-		s_about = new KAboutData("komparepart", I18N_NOOP("KomparePart"), "3.2");
-		s_about->addAuthor("John Firebaugh", "Author", "jfirebaugh@kde.org");
-		s_about->addAuthor("Otto Bruggeman", "Author", "otto.bruggeman@home.nl" );
-		s_instance = new KInstance(s_about);
-	}
-	return s_instance;
-}
-
-extern "C"
-{
-	void* init_libkomparepart()
-	{
-		return new KomparePartFactory;
-	}
-}
-
 #include "kompare_part.moc"
