@@ -1,5 +1,5 @@
 /*
- * $Id: scheck.cpp,v 1.58 2003/09/01 20:50:38 orlovich Exp $
+ * $Id: scheck.cpp,v 1.63 2004/11/21 20:13:05 mueller Exp $
  *
  * KDE3 Style Guide compliance check "Style", v0.0.1
  *        Copyright (C) 2002 Maksim Orlovich <orlovich@cs.rochester.edu>
@@ -49,6 +49,7 @@
 #include <qheader.h>
 #include <qlabel.h>
 #include <qmenubar.h>
+#include <qobjectlist.h>
 #include <qpushbutton.h>
 #include <qradiobutton.h>
 #include <qregexp.h>
@@ -60,6 +61,7 @@
 #include <qtoolbutton.h>
 #include <qtoolbar.h>
 #include <qpopupmenu.h>
+#include <qwidgetlist.h>
 
 #include <kdrawutil.h>
 #include <kaccelmanager.h>
@@ -93,17 +95,17 @@ class StyleCheckStylePlugin : public QStylePlugin
 		}
 };
 
-Q_EXPORT_PLUGIN( StyleCheckStylePlugin )
+KDE_Q_EXPORT_PLUGIN( StyleCheckStylePlugin )
 // ---------------------------------------------------
 
 
 // ### Remove globals
-QBitmap lightBmp;
-QBitmap grayBmp;
-QBitmap dgrayBmp;
-QBitmap centerBmp;
-QBitmap maskBmp;
-QBitmap xBmp;
+static QBitmap lightBmp;
+static QBitmap grayBmp;
+static QBitmap dgrayBmp;
+static QBitmap centerBmp;
+static QBitmap maskBmp;
+static QBitmap xBmp;
 
 static const int itemFrame       = 1;
 static const int itemHMargin     = 3;
@@ -494,7 +496,7 @@ static QValueVector<StyleGuideViolation>  checkSentenceStyle(QString str, ColonM
 					}
 				}
 
-				if (word == QString::null) //Looks like goes to end of string..
+				if (word.isNull()) //Looks like goes to end of string..
 				{
 					word = str.mid ( c );
 				}
@@ -656,7 +658,7 @@ static QValueVector<StyleGuideViolation>  checkTitleStyle(QString str, TitleType
 					}
 				}
 
-				if (word == QString::null) //Looks like goes to end of string..
+				if (word.isNull()) //Looks like goes to end of string..
 				{
 					word = str.mid ( c );
 					lastWord = true;
@@ -877,6 +879,8 @@ StyleCheckStyle::StyleCheckStyle(  )
 	: KStyle( 0 , ThreeButtonScrollBar )
 {
 	hoverWidget = 0L;
+	topLevelAccelManageTimer = new QTimer(this);
+	connect(topLevelAccelManageTimer, SIGNAL(timeout()), this, SLOT(slotAccelManage()));
 	watcher = new StyleCheckTitleWatcher;
 	xxMode = (QString(getenv("KDE_LANG"))=="xx");
 }
@@ -887,17 +891,58 @@ StyleCheckStyle::~StyleCheckStyle()
 	delete watcher;
 }
 
+//We walk down the widget tree until we find something we render, and sic KAccelManager in programmer's mode on those
+void StyleCheckStyle::accelManageRecursive(QWidget* widget)
+{
+	if (&widget->style() == this)
+	{
+		KAcceleratorManager::manage(widget, true);
+		return;
+	}
+	
+	const QObjectList* children = widget->children();
+	if (!children)
+		return;
+	QObjectListIterator iter(*children);
+	
+	QObject* walk;
+	while ((walk = iter.current()))
+	{
+		if (walk->isWidgetType())
+			accelManageRecursive(static_cast<QWidget*>(walk));
+		++iter;
+	}
+}
+
+
+void StyleCheckStyle::slotAccelManage()
+{
+	//Walk through top-levels
+	QWidgetList* topLevels = QApplication::topLevelWidgets();
+	if (!topLevels)
+		return;
+	
+	QWidgetListIt iter(*topLevels);
+	
+	QWidget* walk;
+	while ((walk = iter.current()))
+	{
+		accelManageRecursive(walk);
+		++iter;
+	}
+	
+}
+
 
 void StyleCheckStyle::polish(QWidget* widget)
 {
-        // having a global view on the widget makes accel
-        // easier to catch
-        QWidget* actWin = qApp->activeWindow();
-        if (actWin)
-            KAcceleratorManager::manage(actWin, true);
-        // theorecticaly KAcceleratorManager should be correct by now
-        // for this widget
-        KAcceleratorManager::manage(widget, true);
+	/* Having a global view on the widget makes accel
+	   easier to catch. However, just intruding on the main window
+	   is wrong since a style can be used for a subwindow. The upshot is that we defer 
+	   accel management to a timer, until things stabilize, and then walk from top-levels down
+	*/
+	topLevelAccelManageTimer->start(200, true);
+	//
 
 	// Put in order of highest occurance to maximise hit rate
 	if (widget->inherits("QPushButton")) {
@@ -2597,7 +2642,7 @@ bool StyleCheckStyle::eventFilter( QObject *object, QEvent *event )
 
 		QValueVector<StyleGuideViolation> violations;
 
-		if (QCString(lb->name()) == "KJanusWidgetTitleLabel")
+		if (QCString(lb->name()) == "KJanusWidgetTitleLabel" || lb->font().bold())
 		{
 			// We're a page title
 			violations = checkTitleStyle(lb->text(), LongTitle, lb->buddy() ? HasAccels : NoAccels);
@@ -2714,4 +2759,4 @@ void StyleCheckStyle::renderGradient( QPainter* p, const QRect& r,
 }
 
 // vim: set noet ts=4 sw=4:
-// kate: indent-width 4; replace-tabs off; tab-width 4;
+// kate: indent-width 4; replace-tabs off; tab-width 4; space-indent off;
