@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *									   *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -17,11 +22,10 @@
 class UMLDoc;
 class UMLRole;
 
-using namespace Uml;
-
 /**
  * This class contains the non-graphic representation of an association.
- * An association can be a generalization, aggregation, or composition.
+ * An association can be a generalization, realization, simple association,
+ * directed association, aggregation, or composition.
  *
  * @short Sets up association information.
  * @author Oliver Kellogg <okellogg@users.sourceforge.net>
@@ -35,12 +39,23 @@ class UMLAssociation : public UMLObject {
 public:
 	/**
 	 * Sets up an association.
+	 * A new unique ID is assigned internally.
 	 *
 	 * @param type		The Uml::Association_Type to construct.
 	 * @param roleA		Pointer to the UMLObject in role A.
 	 * @param roleB		Pointer to the UMLObject in role B.
 	 */
-	UMLAssociation(Association_Type type, UMLObject *roleA = 0, UMLObject *roleB = 0);
+	UMLAssociation(Uml::Association_Type type, UMLObject *roleA, UMLObject *roleB);
+
+	/**
+	 * Constructs an association - for loading only.
+	 * This constructor should not normally be used as it constructs
+	 * an incomplete association (i.e. the role objects are missing.)
+	 *
+	 * @param type		The Uml::Association_Type to construct.
+	 *			Default: Uml::at_Unknown.
+	 */
+	UMLAssociation(Uml::Association_Type type = Uml::at_Unknown);
 
 	/**
 	 * Overloaded '==' operator
@@ -83,49 +98,57 @@ public:
 	 *
 	 * @return	Pointer to the UMLObject in the given role.
 	 */
-	UMLObject* getObject(Role_Type role);
+	UMLObject* getObject(Uml::Role_Type role);
+
+	/**
+	 * Returns the ID of the UMLObject assigned to the given role.
+	 * Shorthand for getObject(role)->getID().
+	 *
+	 * @return	ID of the UMLObject in the given role.
+	 */
+	Uml::IDType getObjectId(Uml::Role_Type role);
 
 	/**
 	 * Returns the ID of the UMLObject assigned to the given role.
 	 *
 	 * @return	ID of the UMLObject of the given role.
 	 */
-	int getRoleId(Role_Type role) const;
+	Uml::IDType getRoleId(Uml::Role_Type role) const;
 
 	/**
 	 * Returns the Changeablity of the given role.
 	 *
 	 * @return	Changeability_Type of the given role.
 	 */
-	Changeability_Type getChangeability(Role_Type role) const;
+	Uml::Changeability_Type getChangeability(Uml::Role_Type role) const;
 
 	/**
 	 * Returns the Visibility of the given role.
 	 *
 	 * @return	Scope of the given role.
 	 */
-	Scope getVisibility(Role_Type role) const;
+	Uml::Scope getVisibility(Uml::Role_Type role) const;
 
 	/**
 	 * Returns the multiplicity assigned to the given role.
 	 *
 	 * @return	The multiplicity assigned to the given role.
 	 */
-	QString getMulti(Role_Type role) const;
+	QString getMulti(Uml::Role_Type role) const;
 
 	/**
 	 * Returns the name assigned to the role A.
 	 *
 	 * @return	The name assigned to the given role.
 	 */
-	QString getRoleName(Role_Type role) const;
+	QString getRoleName(Uml::Role_Type role) const;
 
 	/**
 	 * Returns the documentation assigned to the given role.
 	 *
 	 * @return	Documentation text of given role.
 	 */
-	QString getRoleDoc(Role_Type role) const;
+	QString getRoleDoc(Uml::Role_Type role) const;
 
 	/**
 	 * Sets the assocType of the UMLAssociation.
@@ -139,42 +162,42 @@ public:
 	 *
 	 * @param obj		Pointer to the UMLObject of the given role.
 	 */
-	void setObject(UMLObject *obj, Role_Type role);
+	void setObject(UMLObject *obj, Uml::Role_Type role);
 
 	/**
 	 * Sets the visibility of the given role of the UMLAssociation.
 	 *
 	 * @param value 	Scope of role A visibility.
 	 */
-	void setVisibility(Scope value, Role_Type role);
+	void setVisibility(Uml::Scope value, Uml::Role_Type role);
 
 	/**
 	 * Sets the changeability of the given role of the UMLAssociation.
 	 *
 	 * @param value 	Changeability_Type of the given role.
 	 */
-	void setChangeability(Changeability_Type value, Role_Type role);
+	void setChangeability(Uml::Changeability_Type value, Uml::Role_Type role);
 
 	/**
 	 * Sets the multiplicity of the given role of the UMLAssociation.
 	 *
 	 * @param multiA	The multiplicity of the given role.
 	 */
-	void setMulti(QString multi, Role_Type role);
+	void setMulti(const QString &multi, Uml::Role_Type role);
 
 	/**
 	 * Sets the name of the given role of the UMLAssociation.
 	 *
 	 * @param roleNameA	The name of the given role.
 	 */
-	void setRoleName(QString roleName, Role_Type role);
+	void setRoleName(const QString &roleName, Uml::Role_Type role);
 
 	/**
 	 * Sets the documentation on the given role in the association.
 	 *
 	 * @param doc		The string with the documentation.
 	 */
-	void setRoleDoc(QString doc, Role_Type role);
+	void setRoleDoc(const QString &doc, Uml::Role_Type role);
 
 	/**
          * Convert Changeability_Type value into QString representation.
@@ -183,19 +206,24 @@ public:
 	 */
 	static QString ChangeabilityToString(Uml::Changeability_Type type);
 
-        /**
-         * Convert Scope value into QString representation.
-	 *
-	 * @param scope		The Scope enum value to convert.
-         */
-	static QString ScopeToString(Uml::Scope scope);
-
 	/**
 	 * Get the underlying UMLRole object for the given role.
 	 *
 	 * @return	Pointer to the UMLRole object for the given role.
 	 */
-	UMLRole * getUMLRole(Role_Type role);
+	UMLRole * getUMLRole(Uml::Role_Type role);
+
+	/**
+	 * Set backward compatibility flag for loading version 1.3.x files.
+	 * This flag is necessary because the handling of the isNavigable
+	 * attribute of <AssociationEnd> was incorrect.
+	 */
+	void setOldLoadMode(bool value = true);
+
+	/**
+	 * Return the backward compatibility flag for loading files.
+	 */
+	bool getOldLoadMode() const;
 
 	/**
 	 * Make a clone of this object.
@@ -203,6 +231,16 @@ public:
 	 */
 	virtual UMLObject* clone() const { return NULL; }
 
+	/**
+	 * Resolve types. Required when dealing with foreign XMI files.
+	 * Needs to be called after all UML objects are loaded from file.
+	 * Overrides the method from UMLObject.
+	 * Calls resolveRef() for each role.
+	 *
+	 * @return	True for success.
+	 */
+	virtual bool resolveRef();
+
 	/**
 	 * Creates the <UML:Generalization> or <UML:Association> XMI element
 	 * including its role objects.
@@ -226,7 +264,7 @@ protected:
 	 * @param roleAObj	Pointer to the role A UMLObject.
 	 * @param roleBObj	Pointer to the role B UMLObject.
 	 */
-	void init(Association_Type type, UMLObject *roleAObj, UMLObject *roleBObj);
+	void init(Uml::Association_Type type, UMLObject *roleAObj, UMLObject *roleBObj);
 
 	/* If the type Uml::Association_Type is changed then also the following
 	   must be changed accordingly:
@@ -236,14 +274,14 @@ protected:
 	 */
 	static const Uml::Association_Type atypeFirst;
 	static const Uml::Association_Type atypeLast;
-	static const unsigned nAssocTypes;
+	static const unsigned int nAssocTypes;
 	static const QString assocTypeStr[];
 
 	UMLRole * m_pRole[2];
 
 	Uml::Association_Type m_AssocType;
 	QString m_Name;
-
+	bool m_bOldLoadMode;
 };
 
 #endif
