@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,24 +15,22 @@
 #ifndef ASSOCIATIONWIDGET_H
 #define ASSOCIATIONWIDGET_H
 
-#include "umlnamespace.h"
+#include "widgetbase.h"
+#include "linkwidget.h"
 #include "umlwidgetlist.h"
 #include "messagewidgetlist.h"
+#include "associationwidgetlist.h"
 #include "linepath.h"
 
-// qt includes
-#include <qobject.h>
-
-class FloatingText;
 class IDChangeLog;
 class ListPopupMenu;
 class QBitmap;
 class QPixmap;
 class QDataStream;
+class UMLDoc;
 class UMLView;
 class UMLAssociation;
-
-using namespace Uml;
+class UMLAttribute;
 
 /**
  * This class represents an association inside a diagram.
@@ -46,10 +49,9 @@ using namespace Uml;
  *
  * @author Gustavo Madrigal
  * @short This class represents an association inside a diagram.
- * @version $Revision: 1.35 $
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
-class AssociationWidget : public QObject {
+class AssociationWidget : public WidgetBase, public LinkWidget {
 	Q_OBJECT
 public:
 	/**
@@ -76,9 +78,11 @@ public:
 	 * @param WidgetA	Pointer the role A widget for the association.
 	 * @param Type		The Association_Type for this association.
 	 * @param WidgetB	Pointer the role B widget for the association.
+	 * @param umlassoc	Pointer the underlying UMLAssociation (if applicable.)
 	 */
 	AssociationWidget(UMLView *view, UMLWidget* WidgetA,
-			  Association_Type Type, UMLWidget* WidgetB);
+			  Uml::Association_Type Type, UMLWidget* WidgetB,
+			  UMLAssociation *umlassoc = NULL);
 
 	/**
 	 * Deconstructor.
@@ -108,46 +112,25 @@ public:
 	bool activate();
 
 	/**
-	 * Write property of m_pWidgetA.
+	 * Set the widget of the given role.
 	 *
-	 * @param widgetA	Pointer to the role A UMLWidget.
+	 * @param widget	Pointer to the UMLWidget.
 	 */
-	void setWidgetA( UMLWidget* widgetA);
+	void setWidget(UMLWidget* widget, Uml::Role_Type role);
 
 	/**
-	 * Write property of m_pWidgetB.
+	 * Return the multiplicity FloatingText widget of the given role.
 	 *
-	 * @param widgetB	Pointer to the role B UMLWidget.
+	 * @return	Pointer to the multiplicity FloatingText object.
 	 */
-	void setWidgetB( UMLWidget* widgetB);
+	FloatingText* getMultiWidget(Uml::Role_Type role);
 
 	/**
-	 * Read property of FloatingText* m_pMultiA.
+	 * Return the given role's multiplicity text.
 	 *
-	 * @return	Pointer to the multiA FloatingText.
+	 * @return	Text of the given role's multiplicity widget.
 	 */
-	FloatingText* getMultiAWidget();
-
-	/**
-	 * Returns the m_pMultiA's text.
-	 *
-	 * @return	Text of the multiA widget.
-	 */
-	QString getMultiA() const;
-
-	/**
-	 * Read property of FloatingText* m_pMultiB.
-	 *
-	 * @return	Pointer to the multiB FloatingText.
-	 */
-	FloatingText* getMultiBWidget();
-
-	/**
-	 * Returns the m_pMultiB's text.
-	 *
-	 * @return	Text of the multiB widget.
-	 */
-	QString getMultiB() const;
+	QString getMulti(Uml::Role_Type role) const;
 
 	/**
 	 * Read property of FloatingText* m_pName.
@@ -164,18 +147,11 @@ public:
 	QString getName() const;
 
 	/**
-	 * Read property of FloatingText* m_pRoleA.
+	 * Return the given role's FloatingText object.
 	 *
-	 * @return	Pointer to the role A FloatingText widget.
+	 * @return	Pointer to the role's FloatingText widget.
 	 */
-	FloatingText* getRoleAWidget();
-
-	/**
-	 * Read property of FloatingText* m_pRoleB.
-	 *
-	 * @return	Pointer to the role B FloatingText widget.
-	 */
-	FloatingText* getRoleBWidget();
+	FloatingText* getRoleWidget(Uml::Role_Type role);
 
 	/**
 	 * Returns the documentation about this association.
@@ -185,103 +161,60 @@ public:
 	QString getDoc() const;
 
 	/**
-	 * Returns the m_pRoleA's text.
+	 * Return the given role's FloatingText widget text.
 	 *
-	 * @return	The name set at the roleA FloatingText.
+	 * @return	The name set at the FloatingText.
 	 */
-	QString getRoleNameA() const;
+	QString getRoleName(Uml::Role_Type role) const;
 
 	/**
-	 * Returns the documentation about RoleA.
+	 * Returns the given role's documentation.
 	 */
-	QString getRoleADoc() const;
-
-	/**
-	 * Returns the m_pRoleB's text.
-	 *
-	 * @return	The name set at the roleB FloatingText.
-	 */
-	QString getRoleNameB() const;
-
-	/**
-	 * Returns the documentation about Role B.
-	 */
-	QString getRoleBDoc() const;
+	QString getRoleDoc(Uml::Role_Type role) const;
 
 	/**
 	 * Sets the text in the FloatingText widget representing the Name
 	 * of this association.
 	 */
-	void setName (QString strRole);
+	void setName (const QString &strRole);
 
 	/**
-	 * Sets the text in the FloatingText representing the Multiplicity
-	 * at the ending side of the association.
+	 * Sets the text in the FloatingText representing the multiplicity
+	 * at the given side of the association.
 	 */
-	bool setMultiB(QString strMultiB);
+	void setMulti(const QString &strMulti, Uml::Role_Type role);
 
 	/**
-	 * Sets the text in the FloatingText representing the Multiplicity
-	 * at the starting side of the association.
+	 * Gets the visibility on the given role of the association.
 	 */
-	bool setMultiA(QString strMultiA);
+	Uml::Scope getVisibility (Uml::Role_Type role) const;
 
 	/**
-	 * Gets the visibility on the rolename A end of the Association.
+	 * Sets the visibility on the given role of the association.
 	 */
-	Scope getVisibilityA () const;
-
-	/*
-	 * Gets the visibility on the rolename B end of the Association.
-	 */
-	Scope getVisibilityB () const;
+	void setVisibility (Uml::Scope visibility, Uml::Role_Type role );
 
 	/**
-	 * Sets the visibility on the rolename 'A' end of the Association.
+	 * Gets the changeability on the the given end of the Association.
 	 */
-	void setVisibilityA ( Scope visibility );
-
-	/*
-	 * Sets the visibility on the rolename 'B' end of the Association.
-	 */
-	void setVisibilityB (Scope visibility);
+	Uml::Changeability_Type getChangeability(Uml::Role_Type role) const;
 
 	/**
-	 * Gets the changeability on the rolename A end of the Association.
+	 * Sets the changeability on the the given end of the Association.
 	 */
-	Changeability_Type getChangeabilityA() const;
+	void setChangeability (Uml::Changeability_Type value, Uml::Role_Type role);
 
 	/**
-	 * Gets the changeability on the rolename B end of the Association.
+	 * Gets the ID of the given role widget.
 	 */
-	Changeability_Type getChangeabilityB() const;
+	Uml::IDType getWidgetID(Uml::Role_Type role) const;
 
 	/**
-	 * Sets the changeability on the rolename 'B' end of the Association.
-	 */
-	void setChangeabilityA (Changeability_Type value);
-
-	/**
-	 * Sets the changeability on the rolename 'B' end of the Association.
-	 */
-	void setChangeabilityB (Changeability_Type value);
-
-	/**
-	 * Gets the ID of role A.
-	 */
-	int getWidgetAID() const;
-
-	/**
-	 * Gets the ID of role B.
-	 */
-	int getWidgetBID() const;
-
-	/**
-	 * Read property of UMLWidget* m_pWidgetA.
+	 * Gets the given role widget.
 	 *
-	 * @return	Pointer to the role A UMLWidget.
+	 * @return	Pointer to the role's UMLWidget.
 	 */
-	UMLWidget* getWidgetA();
+	UMLWidget* getWidget(Uml::Role_Type role);
 
 	/**
 	 * Sets the associated widgets.
@@ -290,14 +223,7 @@ public:
 	 * @param assocType	The Association_Type for this association.
 	 * @param widgetB	Pointer the role B widget for the association.
 	 */
-	bool setWidgets( UMLWidget* widgetA, Association_Type assocType, UMLWidget* widgetB);
-
-	/**
-	 * Read property of UMLWidget* m_pWidgetB.
-	 *
-	 * @return	Pointer to the role B UMLWidget.
-	 */
-	UMLWidget* getWidgetB();
+	bool setWidgets( UMLWidget* widgetA, Uml::Association_Type assocType, UMLWidget* widgetB);
 
 	/**
 	 * Returns true if this association associates widgetA to widgetB,
@@ -322,14 +248,14 @@ public:
 	 *
 	 * @return	This AssociationWidget's Association_Type.
 	 */
-	Association_Type getAssocType() const;
+	Uml::Association_Type getAssocType() const;
 
 	/**
 	 * Sets the association's type.
 	 *
 	 * @param type		The Association_Type to set.
 	 */
-	void setAssocType(Association_Type type);
+	void setAssocType(Uml::Association_Type type);
 
 	/**
 	 * Returns a QString object representing this AssociationWidget.
@@ -394,6 +320,9 @@ public:
 	 * be the RoleTextSegment (if this segment moves then the RoleText
 	 * will move with it). It sets m_unNameLineSegment to the start point
 	 * of the chosen segment.
+	 *
+	 * Overrides operation from LinkWidget (i.e. this method is also
+	 * required by FloatingText.)
 	 */
 	void calculateNameTextSegment();
 
@@ -432,12 +361,6 @@ public:
 	 */
 	void moveEntireAssoc( int x, int y );
 
-	/**
-	 * Sets the new font for the text widgets used.
-	 * Called by the text widgets when the menu was selected on one of them.
-	 */
-	void changeFont(QFont font);
-
 	/**
 	 * Returns the bounding rectangle of all segments of the association.
 	 */
@@ -452,43 +375,50 @@ public:
 	/**
 	 * Set all 'owned' child widgets to this font.
 	 */
-	void setFont (QFont font);
-
+	void lwSetFont (QFont font);
 
 	/**
-	 * Read property of FloatingText * m_pChangeWidgetA.
+	 * Return the given role's changeability FloatingText widget.
 	 */
-	FloatingText* getChangeWidgetA();
-
-	/**
-	 * Read property of FloatingText * m_pChangeWidgetB.
-	 */
-	FloatingText* getChangeWidgetB();
-
+	FloatingText* getChangeWidget(Uml::Role_Type role);
 
 	/**
 	 * Sets the text to the FloatingText that display the Role text of this
 	 * association.
-	 * For this funtion to work properly, the associated widgets
-	 * (m_pWidgetA and m_pWidgetB) should already be set.
+	 * For this function to work properly, the associated widget
+	 *  should already be set.
 	 */
-	bool setRoleNameA(QString strRole);
-	bool setRoleNameB(QString strRole);
+	bool setRoleName(const QString &strRole, Uml::Role_Type role);
 
 	/**
 	 * Set the documentation on this association.
 	 */
-	void setDoc(QString doc);
+	void setDoc(const QString &doc);
 
 	/**
-	 * Set the documentation on role A.
+	 * Set the documentation on the given role.
 	 */
-	void setRoleADoc(QString doc);
+	void setRoleDoc(const QString &doc, Uml::Role_Type role);
 
 	/**
-	 * Set the documentation on role B.
+	 * Overrides operation from LinkWidget.
+	 * Required by FloatingText.
 	 */
-	void setRoleBDoc(QString doc);
+	UMLClassifier *getOperationOwner(FloatingText *ft);
+
+	/**
+	 * Overrides operation from LinkWidget.
+	 * Required by FloatingText.
+	 */
+	void setOperationText(FloatingText *ft, const QString &opText);
+
+	/**
+	 * Overrides operation from LinkWidget.
+	 * Required by FloatingText.
+	 *
+	 * @param ft	The text widget which to update.
+	 */
+	void setMessageText(FloatingText *ft);
 
 	/**
 	 * Returns the UMLAssociation representation of this object.
@@ -498,50 +428,78 @@ public:
 	 */
 	UMLAssociation * getAssociation ();
 
+	/**
+	 * Returns the UMLAttribute representation of this object.
+	 *
+	 * @return	Pointer to the UMLAttribute that is represented by
+	 *		this AsociationWidget.
+	 */
+	UMLAttribute * getAttribute ();
+
+	/**
+	 * Sets the text of the FloatingText identified by the ft's Text_Role.
+	 * Overrides operation from LinkWidget.
+	 * Required by FloatingText.
+	 */
+	void setText(FloatingText *ft, const QString &text);
+
 	/**
 	 * Calls @ref setTextPosition() on all the labels.
+	 * Overrides operation from LinkWidget.
 	 */
 	void resetTextPositions();
 
 	/**
-	 * Sets the Association line index for widgetA.
+	 * Shows the association properties dialog and updates the
+	 * corresponding texts if its execution is successful.
+	 * Returns true for success.
 	 */
-	void setIndexA(int index);
+	bool showDialog();
 
 	/**
-	 * Returns the Association line index for widgetA.
+	 * Sets the Association line index for the given role.
 	 */
-	int getIndexA() const;
+	void setIndex(int index, Uml::Role_Type role);
 
 	/**
-	 * Sets the Association line index for widgetB.
+	 * Returns the Association line index for the given role.
 	 */
-	void setIndexB(int index);
+	int getIndex(Uml::Role_Type role) const;
 
 	/**
-	 * Returns the Association line index for widgetB.
+	 * Sets the total count on the Association region.
 	 */
-	int getIndexB() const;
+	void setTotalCount(int count, Uml::Role_Type role);
 
 	/**
-	 * Sets the total count on the Association region for widgetA.
+	 * Returns the total count on the Association region.
 	 */
-	void setTotalCountA(int count);
-
-	/**
-	 * Returns the total count on the Association region for widgetA.
-	 */
-	int getTotalCountA() const;
+	int getTotalCount(Uml::Role_Type role) const;
 
 	/**
 	 * Sets the total count on the Association region for widgetB.
 	 */
-	void setTotalCountB(int count);
+	void setTotalCount(int count);
 
 	/**
-	 * Region the total count on the Association region for widgetB
+	 * Overrides operation from LinkWidget.
+	 * Required by FloatingText.
+	 *
+	 * @param seqNum	The new sequence number string to set.
+	 * @param op		The new operation string to set.
 	 */
-	int getTotalCountB() const;
+	void setSeqNumAndOp(const QString &seqNum, const QString &op);
+
+	/**
+	 * Overrides operation from LinkWidget.
+	 * Required by FloatingText.
+	 *
+	 * @param ft		The calling FloatingText (unused here.)
+	 * @param seqNum	Return this AssociationWidget's sequence number string.
+	 * @param op		Return this AssociationWidget's operation string.
+	 */
+	UMLClassifier * getSeqNumAndOp(FloatingText *ft, QString& seqNum,
+							 QString& op);
 
 	/**
 	 * Calculates and sets the first and last point in the association's
@@ -553,12 +511,12 @@ public:
 	void calculateEndingPoints();
 
 	/**
-	 * Saves this widget to the <UML:AssociationWidget> XMI element.
+	 * Saves this widget to the <assocwidget> XMI element.
 	 */
-	bool saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
+	void saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
 
 	/**
-	 * Loads this widget from the <UML:AssociationWidget> XMI element.
+	 * Loads this widget from the <assocwidget> XMI element.
 	 */
 	bool loadFromXMI( QDomElement & qElement );
 
@@ -622,6 +580,18 @@ private:
 	 */
 	static Region findPointRegion(QRect Rect, int PosX, int PosY);
 
+	/**
+	 * Given a rectangle and a point, findInterceptOnEdge computes the
+	 * connecting line between the middle point of the rectangle and
+	 * the point, and returns the intercept of this line with the
+	 * the edge of the rectangle identified by `region'.
+	 * When the region is North or South, the X value is returned (Y is
+	 * constant.)
+	 * When the region is East or West, the Y value is returned (X is
+	 * constant.)
+	 */
+	static int findInterceptOnEdge(const QRect &rect, Region region, const QPoint &point);
+
 	/**
 	 * Overrides moveEvent.
 	 */
@@ -631,7 +601,7 @@ private:
 	 * This function calculates which role should be set for the m_pName
 	 * FloatingText.
 	 */
-	Text_Role CalculateNameType(Text_Role defaultRoleType);
+	Uml::Text_Role CalculateNameType(Uml::Text_Role defaultRoleType);
 
 	/**
 	 * Returns true if point (PosX, PosY) is close enough to any of the
@@ -643,19 +613,7 @@ private:
 	/**
 	 * Returns a point with interchanged X and Y coordinates.
 	 */
-	static QPoint swapXY(QPoint p);
-
-	/**
-	 * Returns the intersection point between line P1P2 and the bounding
-	 * rectangle of pWidget.
-	 * Not currently used.
-	 */
-	QPoint findRectIntersectionPoint(UMLWidget* pWidget, QPoint P1, QPoint P2);
-
-	/**
-	 * Returns the intersection point between lines P1P2 and P3P4.
-	 */
-	static QPoint findIntersection(QPoint P1, QPoint P2, QPoint P3, QPoint P4);
+	static QPoint swapXY(const QPoint &p);
 
 	/**
 	 * Returns the total length of the association's LinePath:
@@ -670,7 +628,7 @@ private:
 	 * to Distance and if PX is not a point of the segment P1P2 then the
 	 * function returns (-1,-1).
 	 */
-	static QPoint calculatePointAtDistance(QPoint P1, QPoint P2, float Distance);
+	static QPoint calculatePointAtDistance(const QPoint &P1, const QPoint &P2, float Distance);
 
 	/**
 	 * Calculates which point of a perpendicular line to segment P1P2 that
@@ -678,7 +636,7 @@ private:
 	 * Let's say such point is PX, the distance from P2 to PX must be equal
 	 * to Distance.
 	 */
-	static QPoint calculatePointAtDistanceOnPerpendicular(QPoint P1, QPoint P2, float Distance);
+	static QPoint calculatePointAtDistanceOnPerpendicular(const QPoint &P1, const QPoint &P2, float Distance);
 
 	/**
 	 * Calculates the intersection between line P1P2 and a perpendicular
@@ -694,23 +652,27 @@ private:
 	 * that widget is playing.
 	 * Returns the point at which to put the widget.
 	 */
-	QPoint calculateTextPosition(Text_Role role);
+	QPoint calculateTextPosition(Uml::Text_Role role);
+
+	/**
+	 * Returns the FloatingText identified by the given text role.
+	 * Returns NULL if there is no FloatingText active for the text role.
+	 */
+	FloatingText* floatingText(Uml::Text_Role role);
 
 	/**
 	 * Puts the text widget with the given role at the given position.
-	 * This method is usually called after calling @ref calculateTextPostion
-	 * to get the needed postion.  It is a separate function in case there
-	 * is a need to place the text without calculation.
+	 * This method calls @ref calculateTextPostion to get the needed position.
 	 * I.e. the line segment it is on has moved and it should move the same
 	 * amount as the line.
 	 */
-	void setTextPosition(Text_Role role, QPoint pos);
+	void setTextPosition(Uml::Text_Role role);
 
 	/**
 	 * Moves the text widget with the given role by the difference between
 	 * the two points.
 	 */
-	void setTextPositionRelatively(Text_Role role, QPoint pos, QPoint oldPosition);
+	void setTextPositionRelatively(Uml::Text_Role role, const QPoint &oldPosition);
 
 	/**
 	 * Returns the Region the widget to line intersection is for the given
@@ -767,13 +729,6 @@ private:
 		 */
 		QPoint m_OldCorner;
 
-		/**
-		 * This role's old top left region according to the other role's bounding
-		 * rectangle.
-		 * Not currently used. (findRectIntersectionPoint() is inactive.)
-		 */
-		int m_nCornerRegion;
-
 		/**
 		 * The region of this role's widget.
 		 */
@@ -789,20 +744,15 @@ private:
 		 */
 		int m_nTotalCount;
 
-		// The following items are only used if m_pAssociation is not set.
-		Scope m_Visibility;
-		Changeability_Type m_Changeability;
+		// The following items are only used if m_pObject is not set.
+		Uml::Scope m_Visibility;
+		Uml::Changeability_Type m_Changeability;
 		QString m_RoleDoc;
 		QString m_RoleName;
 		QString m_Multi;
 
 	} m_role[2];
 
-	/**
-	 * Constants used for indexing the roles. 
-	 */
-	enum Role_Type { A, B };
-
 	/**
 	 * Called to tell the association that another association has added
 	 * a line to the region of one of its widgets. The widget is identified
@@ -812,19 +762,19 @@ private:
 	 * @ref calculateEndingPoints when required.
 	 */
 	void updateRegionLineCount(int index, int totalCount,
-				   AssociationWidget::Region region, Role_Type role);
+				   AssociationWidget::Region region, Uml::Role_Type role);
 
 	/**
 	 * Tells all the other view associations the new count for the
 	 * given widget on a certain region. And also what index they should be.
 	 */
-	void updateAssociations(int totalCount, Region region, Role_Type role);
+	void updateAssociations(int totalCount, Region region, Uml::Role_Type role);
 
 	/**
 	 * Returns the number of lines there are on the given region for
 	 * either widget A or B of the association.
 	 */
-	int getRegionCount(Region region, Role_Type role);
+	int getRegionCount(Region region, Uml::Role_Type role);
 
 	/**
 	 * Initialize attributes of this class at construction time.
@@ -834,38 +784,13 @@ private:
 	/**
 	 * Auxiliary method for calculateEndingPoints().
 	 */
-	void doUpdates(int otherX, int otherY, Role_Type role);
-
-	/**
-	 * Auxiliary to setMulti{A,B}.
-	 */
-	bool setMulti(QString strMulti, Role_Type role);
-
-	/**
-	 * Auxiliary to setRoleName{A,B}.
-	 */
-	bool setRoleName (QString strRole, Role_Type role);
-
-	/**
-	 * Auxiliary to setVisibility{A,B}.
-	 */
-	void setVisibility (Scope value, Role_Type role);
-
-	/**
-	 * Auxiliary to setChangeability{A,B}.
-	 */
-	void setChangeability (Changeability_Type value, Role_Type role);
+	void doUpdates(int otherX, int otherY, Uml::Role_Type role);
 
 	/**
 	 * For internal purposes only.
-	 * Other classes/users should use setChangeability[AB] instead.
+	 * Other classes/users should use setChangeability() instead.
 	 */
-	bool setChangeWidget(QString strChangeWidget, Role_Type role);
-
-	/**
-	 * Auxiliary to getWidget{A,B}ID.
-	 */
-	int getWidgetID(Role_Type role) const;
+	void setChangeWidget(const QString &strChangeWidget, Uml::Role_Type role);
 
 	/**
 	 * Checks to see if the given point is one of the points of the line.
@@ -873,24 +798,29 @@ private:
 	 * This is only valid if no other point id being moved and only
 	 * while the left mouse button is down.
 	 */
-	void checkPoints(QPoint p);
+	void checkPoints(const QPoint &p);
 
 	/**
-	 * Utility: Find the widget identified by the given ID in the given
-	 * widget or message list.
-	 * It's somewhat arbitrary that this utility is here as it has no
-	 * dependencies whatsoever on AssociationWidget.  It just happens
-	 * to be needed here.  We should pull it out from here if it also
-	 * becomes needed somewhere else.
+	 * Returns true if the line path starts at the given widget.
 	 */
-	static UMLWidget* findWidget(int id, const UMLWidgetList& widgets,
-					     const MessageWidgetList* pMessages = NULL);
+	bool linePathStartsAt(const UMLWidget* widget);
+
+	/**
+	 * Auxiliary method for updateAssociations():
+	 * Put position into m_positions and assoc into m_ordered at the
+	 * correct index.
+	 * m_positions and m_ordered move in parallel and are sorted by
+	 * ascending position.
+	 */
+	void insertIntoLists(int position, const AssociationWidget* assoc);
+
+	int m_positions[100];             ///< auxiliary variable for updateAssociations()
+	int m_positions_len;              ///< auxiliary variable for updateAssociations()
+	AssociationWidgetList m_ordered;  ///< auxiliary variable for updateAssociations()
 
 	/*QPixmap 	*m_pPix;
 	QBitmap 	*m_pMask;*/
 
-	UMLView 	* m_pView;
-
 	/**
 	 * Flag which is true if the activate method has been called for this
 	 * class instance.
@@ -905,10 +835,9 @@ private:
 	 * m_LinePath[m_unNameLineSegment] -- m_LinePath[m_unNameLineSegment+1]
 	 */
 	uint 		m_unNameLineSegment;
-
+	UMLDoc * m_umldoc;  ///< just a shorthand for UMLApp::app()->getDocument()
 	bool 		m_bFocus;
 	ListPopupMenu 	*m_pMenu;
-	UMLAssociation  *m_pAssociation;
 	bool 		m_bSelected;
 	int 		m_nMovingPoint;
 
@@ -917,7 +846,7 @@ private:
 	 */
 	LinePath m_LinePath;
 
-	// The following items are only used if m_pAssociation is not set.
+	// The following items are only used if m_pObject is not set.
 	QString m_Doc;
 	Uml::Association_Type m_AssocType;
 
@@ -941,8 +870,8 @@ public slots:
 	void slotClearAllSelected();
 
 	/**
-	 * Merge together the UMLAssociation representation and this widget
+	 * Synchronize this widget from the UMLAssociation.
 	 */
-	void mergeUMLRepresentationIntoAssociationData();
+	void syncToModel();
 };
 #endif
