@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -22,7 +27,6 @@
  * @see UMLObject
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
-
 class UMLAttribute : public UMLClassifierListItem {
 public:
 	/**
@@ -31,12 +35,12 @@ public:
 	 * @param parent	The parent of this UMLAttribute.
 	 * @param name		The name of this UMLAttribute.
 	 * @param id		The unique id given to this UMLAttribute.
-	 * @param type		The type of this UMLAttribute.
 	 * @param s		The scope of the UMLAttribute.
+	 * @param type		The type of this UMLAttribute.
 	 * @param iv		The initial value of the attribute.
 	 */
-	UMLAttribute(const UMLObject *parent, QString name, int id, QString type = "int",
-		     Scope s = Private, QString iv = 0);
+	UMLAttribute(const UMLObject *parent, QString name, Uml::IDType id = Uml::id_None,
+		     Uml::Scope s = Uml::Private, QString type = "int", QString iv = 0);
 
 	/**
 	 * Sets up an attribute.
@@ -66,13 +70,6 @@ public:
 	 */
 	virtual UMLObject* clone() const;
 
-	/**
-	 * Returns the type of the UMLAttribute.
-	 *
-	 * @return	The type of the UMLAttribute.
-	 */
-	QString getTypeName();
-
 	/**
 	 * Returns The initial value of the UMLAttribute.
 	 *
@@ -80,19 +77,12 @@ public:
 	 */
 	QString getInitialValue();
 
-	/**
-	 * Sets the type of the UMLAttribute.
-	 *
-	 * @param type		The type of the UMLAttribute.
-	 */
-	void setTypeName(QString type);
-
 	/**
 	 * Sets the initial value of the UMLAttribute.
 	 *
 	 * @param iv		The inital value of the UMLAttribute.
 	 */
-	void setInitialValue(QString iv);
+	void setInitialValue( const QString &iv );
 
 	/**
 	 * Returns a string representation of the UMLAttribute.
@@ -101,7 +91,12 @@ public:
 	 *			inital value.
 	 * @return	Returns a string representation of the UMLAttribute.
 	 */
-	QString toString(Signature_Type sig);
+	QString toString(Uml::Signature_Type sig = Uml::st_NoSig);
+
+	/**
+	 * Reimplement method from UMLObject.
+	 */
+	QString getFullyQualifiedName(const QString &separator = "::") const;
 
 	/**
 	 * Creates the <UML:Attribute> XMI element.
@@ -113,8 +108,8 @@ public:
 	 */
 	bool showPropertiesDialogue(QWidget* parent);
 
-	void setParmKind (Uml::Parameter_Kind pk);
-	Uml::Parameter_Kind getParmKind () const;
+	void setParmKind (Uml::Parameter_Direction pk);
+	Uml::Parameter_Direction getParmKind () const;
 
 protected:
 	/**
@@ -123,9 +118,8 @@ protected:
 	bool load( QDomElement & element );
 
 private:
-	QString m_TypeName;     ///< text for the attribute type.
 	QString m_InitialValue; ///< text for the attribute's initial value.
-	Uml::Parameter_Kind m_ParmKind;
+	Uml::Parameter_Direction m_ParmKind;
 };
 
 #endif
