@@ -14,6 +14,8 @@
 #include "umlattributelist.h"
 #include "umloperationlist.h"
 #include "umlclassifierlistitemlist.h"
+#include "classifierlistitem.h"
+#include "umltemplatelist.h"
 
 class IDChangeLog;
 
@@ -49,7 +51,7 @@ public:
 	 * @param name		The name of the Concept.
 	 * @param id		The unique id of the Concept.
 	 */
-	UMLClassifier(const QString & name = "", int id = -1);
+	UMLClassifier(const QString & name = "", Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Standard deconstructor.
@@ -147,14 +149,97 @@ public:
 	 *
 	 * @return	The list of operations for the Classifier.
 	 */
-	UMLClassifierListItemList getOpList(bool includeInherited = false);
+	UMLOperationList getOpList(bool includeInherited = false);
 
 	/**
-	 * Returns the entries in m_pOpsList that are actually operations.
+	 * Adds a template to the class if it is not there yet.
+	 *
+	 * @param name		The name of the template.
+	 * @param id		The id of the template.
+	 * @return	Pointer to the UMLTemplate object created.
+	 */
+	UMLTemplate* addTemplate(const QString &name, Uml::IDType id = Uml::id_None);
+
+	/**
+	 * Adds an already created template.
+	 * The template object must not belong to any other concept.
+	 *
+	 * @param newTemplate	Pointer to the UMLTemplate object to add.
+	 * @param log		Pointer to the IDChangeLog.
+	 * @return	True if the template was sucessfully added.
+	 */
+	bool addTemplate(UMLTemplate* newTemplate, IDChangeLog* log = 0);
+
+	/**
+	 * Adds an template to the class.
+	 * The template object must not belong to any other class.
+	 *
+	 * @param Template	Pointer to the UMLTemplate to add.
+	 * @param position	The position of the template in the list.
+	 *			A value of -1 will add the template at the end.
+	 * @return	True if the template was sucessfully added.
+	 */
+	 //TODO: if the param IDChangeLog from the method above is not being used,
+	  // give position a default value of -1 and the method can replace the above one
+	bool addTemplate(UMLTemplate* Template, int position);
+
+	/**
+	 * Removes a template from the class.
+	 *
+	 * @param template	The template to remove.
+	 * @return	Count of the remaining templates after removal.
+	 *		Returns -1 if the given template was not found.
+	 */
+	int removeTemplate(UMLTemplate* umltemplate);
+
+	/**
+	 * Take and return a template parameter from class.
+	 * It is the callers responsibility to pass on ownership of
+	 * the returned template (or to delete the template)
+	 *
+	 * @param t		template to take
+	 * @return	pointer to the template or null if not found.
+	 */
+	UMLTemplate* takeTemplate(UMLTemplate* t);
+
+	/**
+	 * Seeks the template parameter of the given name.
+	 */
+	UMLTemplate *findTemplate(QString name);
+
+	/**
+	 * Returns the number of templates for the class.
+	 *
+	 * @return	The number of templates for the class.
+	 */
+	int templates();
+
+	/**
+	 * Returns the templates.
+	 * Same as UMLClassifier::getFilteredList(ot_Template) but
+	 * return type is a true UMLTemplateList.
+	 *
+	 * @return	Pointer to the list of true templates for the class.
+	 */
+	UMLTemplateList getTemplateList();
+
+
+	/**
+	 * Returns the entries in m_List that are of the requested type.
 	 *
 	 * @return	The list of true operations for the Concept.
 	 */
-	UMLOperationList getFilteredOperationsList(bool includeInherited = false);
+	UMLClassifierListItemList getFilteredList(Uml::Object_Type ot);
+
+	/**
+	 * Needs to be called after all UML objects are loaded from file.
+	 * Calls the parent resolveRef(), and calls resolveRef() on all
+	 * UMLClassifierListItems.
+	 * Overrides the method from UMLObject.
+	 *
+	 * @return 	true for success.
+	 */
+	virtual bool resolveRef();
 
 	/**
 	 * Find a list of attributes, operations, associations or
@@ -162,12 +247,9 @@ public:
 	 *
 	 * @param t		The type to find.
 	 * @param n		The name of the object to find.
-	 * @param seekStereo	Set this true if we should look at the object's
-	 *			stereotype instead of its name.
 	 * @return	The list of objects found; will be empty if none found.
 	 */
-	virtual UMLObjectList findChildObject(UMLObject_Type t, QString n,
-					      bool seekStereo = false);
+	virtual UMLObjectList findChildObject(Uml::Object_Type t, const QString &n);
 
 	/**
 	 * Find an attribute, operation, association or template.
@@ -176,18 +258,7 @@ public:
 	 *
 	 * @return	The object found.  Will return 0 if none found.
 	 */
-	virtual UMLObject* findChildObject(int id);
-
-	/**
-	 * Find the object of the given (non-numeric) auxiliary ID
-	 * in m_OpsList.  The auxiliary ID is the ID returned by
-	 * UMLObject::getAuxId() and is currently only used for
-	 * loading foreign XMI files.
-	 *
-	 * @param idStr		The ID to seek.
-	 * @return	Pointer to the UMLObject found or NULL if not found.
-	 */
-	UMLObject* findChildObjectByIdStr(QString idStr);
+	virtual UMLObject* findChildObject(Uml::IDType id);
 
 	/**
 	 * Returns a list of concepts which this concept inherits from.
@@ -234,12 +305,15 @@ signals:
 	 */
 	void operationRemoved(UMLOperation *);
 
+	void templateAdded(UMLObject*);
+	void templateRemoved(UMLObject*);
+
 protected:
 
 	/**
-	 * List of all the operations in this classifier.
+	 * List of items (operations/attributes/templates) in this classifier.
 	 */
-	UMLClassifierListItemList m_OpsList;
+	UMLClassifierListItemList m_List;
 
 private:
 
@@ -249,6 +323,13 @@ private:
 	void init();
 
 protected:
+
+	/**
+	 * Auxiliary to saveToXMI of inheriting classes:
+	 * Saves template parameters to the given QDomElement.
+	 */
+	void saveToXMI(QDomDocument& qDoc, QDomElement& qElement);
+
 	/**
 	 * Auxiliary to loadFromXMI:
 	 * The loading of operations is implemented here.
