@@ -1,23 +1,33 @@
 /***************************************************************************
- *   cpptree2uml.cpp - based on kdevelop/languages/cpp/store_walker.cpp,   *
- *		     Copyright (C) 2003 by Roberto Raggi		 *
+ *   Based on kdevelop/languages/cpp/store_walker.cpp,                     *
+ *	             Copyright (C) 2003 by Roberto Raggi                   *
  *   Copyright (C) 2004 by Oliver Kellogg <okellogg@users.sourceforge.net> *
- *									 *
+ *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.				   *
- *									 *
+ *                                                                         *
  ***************************************************************************/
 
+// own header
 #include "cpptree2uml.h"
-#include "ast_utils.h"
-#include "../classimport.h"
-#include "urlutil.h"
-
-#include <kdebug.h>
+// qt/kde includes
 #include <qfileinfo.h>
 #include <qdir.h>
+#include <qregexp.h>
+#include <kdebug.h>
+// app includes
+#include "ast_utils.h"
+#include "urlutil.h"
+#include "../classimport.h"
+// FIXME: The sole reason for the next 2 includes is parseTypedef().
+// Make capsule methods in ClassImport, and remove these includes.
+#include "../classifier.h"
+#include "../datatype.h"
+// FIXME The next 2 includes are motivated by template params
+#include "../template.h"
+#include "../class.h"
 
 CppTree2Uml::CppTree2Uml( const QString& fileName, ClassImport* store )
     : m_importer( store ), m_anon( 0 ), m_nsCnt( 0 ), m_clsCnt( 0 )
@@ -69,12 +79,16 @@ void CppTree2Uml::parseNamespace( NamespaceAST* ast )
 #ifdef DEBUG_CPPTREE2UML
     kdDebug() << "CppTree2Uml::parseNamespace: " << nsName << endl;
 #endif
-    UMLObject * o = m_importer->createUMLObject( Uml::ot_Package, nsName, "" /*stereotype*/,
-						 ast->comment(),
-						 m_currentNamespace[m_nsCnt] );
+    UMLObject * o = m_importer->createUMLObject( Uml::ot_Package, nsName,
+						 m_currentNamespace[m_nsCnt],
+						 ast->comment());
     UMLPackage *ns = (UMLPackage *)o;
     m_currentScope.push_back( nsName );
-    m_currentNamespace[++m_nsCnt] = ns;
+    if (++m_nsCnt > STACKSIZE) {
+	kdError() << "CppTree2Uml::parseNamespace: excessive namespace nesting" << endl;
+	m_nsCnt = STACKSIZE;
+    }
+    m_currentNamespace[m_nsCnt] = ns;
 
     TreeParser::parseNamespace( ast );
 
@@ -126,13 +140,27 @@ void CppTree2Uml::parseTypedef( TypedefAST* ast )
 	       if( d->declaratorId() )
 		  id = d->declaratorId()->text();
 	    }
-#ifdef DEBUG_CPPTREE2UML
+//#ifdef DEBUG_CPPTREE2UML
 	    kdDebug() << "CppTree2Uml::parseTypedef: name=" << id << ", type=" << type << endl;
-#endif
-	    m_importer->createUMLObject( Uml::ot_Class, id,
-					 "typedef" /* stereotype */,
-	 				 "" /* doc */,
-					 m_currentNamespace[m_nsCnt] );
+//#endif
+	    /* @todo Trace typedefs back to their root type for deciding
+	             whether to build a Datatype (for pointers.)  */
+	    if (type.contains('*')) {
+		UMLObject *inner =
+	        m_importer->createUMLObject( Uml::ot_Class, typeId,
+					     m_currentNamespace[m_nsCnt] );
+	        UMLObject *typedefObj =
+		m_importer->createUMLObject( Uml::ot_Datatype, id,
+					     m_currentNamespace[m_nsCnt] );
+	        UMLDatatype *dt = static_cast<UMLDatatype*>(typedefObj);
+		dt->setIsReference();
+		dt->setOriginType(static_cast<UMLClassifier*>(inner));
+	    } else {
+	        m_importer->createUMLObject( Uml::ot_Class, id,
+					     m_currentNamespace[m_nsCnt],
+	 				     "" /* doc */,
+					     "typedef" /* stereotype */);
+	    }
 	    ++it;
 	}
 
@@ -142,10 +170,51 @@ void CppTree2Uml::parseTypedef( TypedefAST* ast )
 
 void CppTree2Uml::parseTemplateDeclaration( TemplateDeclarationAST* ast )
 {
+    TemplateParameterListAST* parmListAST = ast->templateParameterList();
+    QPtrList<TemplateParameterAST> parmList = parmListAST->templateParameterList();
+    for (QPtrListIterator<TemplateParameterAST> it(parmList); it.current(); ++it) {
+	// The template is either a typeParameter or a typeValueParameter.
+
+    	TemplateParameterAST* tmplParmNode = it.current();
+	TypeParameterAST* typeParmNode = tmplParmNode->typeParameter();
+	if (typeParmNode) {
+	    NameAST* nameNode = typeParmNode->name();
+	    if (nameNode) {
+		QString typeName = nameNode->unqualifiedName()->text();
+		Umbrello::NameAndType nt(typeName, NULL);
+		m_templateParams.append(nt);
+	    } else {
+		kdError() << "CppTree2Uml::parseTemplateDeclaration(type):"
+			  << " nameNode is NULL" << endl;
+	    }
+	}
+
+	ParameterDeclarationAST* valueNode = tmplParmNode->typeValueParameter();
+	if (valueNode) {
+	    TypeSpecifierAST* typeSpec = valueNode->typeSpec();
+	    if (typeSpec == NULL) {
+		kdError() << "CppTree2Uml::parseTemplateDeclaration(value):"
+			  << " typeSpec is NULL" << endl;
+		continue;
+	    }
+	    QString typeName = typeSpec->name()->text();
+	    UMLObject *t = m_importer->createUMLObject( Uml::ot_UMLObject, typeName,
+							m_currentNamespace[m_nsCnt] );
+	    DeclaratorAST* declNode = valueNode->declarator();
+	    NameAST* nameNode = declNode->declaratorId();
+	    if (nameNode == NULL) {
+		kdError() << "CppTree2Uml::parseTemplateDeclaration(value):"
+			  << " nameNode is NULL" << endl;
+		continue;
+	    }
+	    QString paramName = nameNode->unqualifiedName()->text();
+	    Umbrello::NameAndType nt(paramName, t);
+	    m_templateParams.append(nt);
+	}
+    }
+
     if( ast->declaration() )
 	TreeParser::parseDeclaration( ast->declaration() );
-
-    TreeParser::parseTemplateDeclaration( ast );
 }
 
 void CppTree2Uml::parseSimpleDeclaration( SimpleDeclarationAST* ast )
@@ -214,16 +283,16 @@ void CppTree2Uml::parseFunctionDefinition( FunctionDefinitionAST* ast )
 
     UMLClass *c = m_currentClass[m_clsCnt];
     if (c == NULL) {
-        kdDebug() << "CppTree2Uml::parseFunctionDefinition (" << id
+	kdDebug() << "CppTree2Uml::parseFunctionDefinition (" << id
 		  << "): need a surrounding class." << endl;
 	return;
     }
     QString returnType = typeOfDeclaration( typeSpec, d );
     UMLOperation *m = m_importer->makeOperation(c, id);
-    m_importer->insertMethod( c, m, (Uml::Scope)m_currentAccess, returnType,
-			      isStatic, false /*isAbstract*/, m_comment );
-    m_comment = "";
     parseFunctionArguments( d, m );
+    m_importer->insertMethod( c, m, (Uml::Scope)m_currentAccess, returnType,
+			      isStatic, false /*isAbstract*/, m_comment);
+    m_comment = "";
 
 /* For reference, Kdevelop does some more:
     method->setFileName( m_fileName );
@@ -258,25 +327,37 @@ void CppTree2Uml::parseClassSpecifier( ClassSpecifierAST* ast )
     } else {
 	className = ast->name()->unqualifiedName()->text().stripWhiteSpace();
     }
-#ifdef DEBUG_CPPTREE2UML
+//#ifdef DEBUG_CPPTREE2UML
     kdDebug() << "CppTree2Uml::parseClassSpecifier: name=" << className << endl;
-#endif
+//#endif
     if( !scopeOfName( ast->name(), QStringList() ).isEmpty() ){
 	kdDebug() << "skip private class declarations" << endl;
 	return;
     }
 
-    UMLObject * o = m_importer->createUMLObject( Uml::ot_Class, className, "" /*stereotype*/,
-						 ast->comment(),
-						 m_currentNamespace[m_nsCnt] );
+    if (className.isEmpty()) {
+	className = "anon_" + QString::number(m_anon);
+	m_anon++;
+    }
+    UMLObject * o = m_importer->createUMLObject( Uml::ot_Class, className,
+						 m_currentNamespace[m_nsCnt],
+						 ast->comment() );
     UMLClass *klass = (UMLClass *)o;
-
+    flushTemplateParams(klass);
     if ( ast->baseClause() )
 	parseBaseClause( ast->baseClause(), klass );
 
     m_currentScope.push_back( className );
-    m_currentClass[++m_clsCnt] = klass;
-    m_currentNamespace[++m_nsCnt] = (UMLPackage*)klass;
+    if (++m_clsCnt > STACKSIZE) {
+	kdError() << "CppTree2Uml::parseNamespace: excessive class nesting" << endl;
+	m_clsCnt = STACKSIZE;
+    }
+    m_currentClass[m_clsCnt] = klass;
+    if (++m_nsCnt > STACKSIZE) {
+	kdError() << "CppTree2Uml::parseNamespace: excessive namespace nesting" << endl;
+	m_nsCnt = STACKSIZE;
+    }
+    m_currentNamespace[m_nsCnt] = (UMLPackage*)klass;
 
     TreeParser::parseClassSpecifier( ast );
 
@@ -292,11 +373,15 @@ void CppTree2Uml::parseClassSpecifier( ClassSpecifierAST* ast )
 
 void CppTree2Uml::parseEnumSpecifier( EnumSpecifierAST* ast )
 {
-    QString typeName = ast->name()->unqualifiedName()->text().stripWhiteSpace();
+    NameAST *nameNode = ast->name();
+    if (nameNode == NULL)
+	return;  // skip constants
+    QString typeName = nameNode->unqualifiedName()->text().stripWhiteSpace();
+    if (typeName.isEmpty())
+	return;  // skip constants
     UMLObject *o = m_importer->createUMLObject( Uml::ot_Enum, typeName,
-						"" /* stereotype */,
-						ast->comment(),
-						m_currentNamespace[m_nsCnt] );
+						m_currentNamespace[m_nsCnt],
+						ast->comment() );
 
     QPtrList<EnumeratorAST> l = ast->enumeratorList();
     QPtrListIterator<EnumeratorAST> it( l );
@@ -307,6 +392,19 @@ void CppTree2Uml::parseEnumSpecifier( EnumSpecifierAST* ast )
     }
 }
 
+void CppTree2Uml::parseElaboratedTypeSpecifier( ElaboratedTypeSpecifierAST* typeSpec )
+{
+    // This is invoked for forward declarations.
+    /// @todo Refine - Currently only handles class foward declarations.
+    ///              - Using typeSpec->text() is probably not good, decode
+    ///                the kind() instead.
+    QString text = typeSpec->text();
+    kdDebug() << "CppTree2Uml::parseElaboratedTypeSpecifier: text is " << text << endl;
+    text.remove(QRegExp("^class\\s+"));
+    UMLObject *o = m_importer->createUMLObject( Uml::ot_Class, text );
+    flushTemplateParams( static_cast<UMLClass*>(o) );
+}
+
 void CppTree2Uml::parseDeclaration( GroupAST* funSpec, GroupAST* storageSpec,
 				    TypeSpecifierAST* typeSpec, InitDeclaratorAST* decl )
 {
@@ -337,7 +435,7 @@ void CppTree2Uml::parseDeclaration( GroupAST* funSpec, GroupAST* storageSpec,
 
     UMLClass *c = m_currentClass[m_clsCnt];
     if (c == NULL) {
-        kdDebug() << "CppTree2Uml::parseDeclaration (" << id
+	kdDebug() << "CppTree2Uml::parseDeclaration (" << id
 		  << "): need a surrounding class." << endl;
 	return;
     }
@@ -419,7 +517,7 @@ void CppTree2Uml::parseFunctionDeclaration(  GroupAST* funSpec, GroupAST* storag
 
     UMLClass *c = m_currentClass[m_clsCnt];
     if (c == NULL) {
-        kdDebug() << "CppTree2Uml::parseFunctionDeclaration (" << id
+	kdDebug() << "CppTree2Uml::parseFunctionDeclaration (" << id
 		  << "): need a surrounding class." << endl;
 	return;
     }
@@ -428,7 +526,7 @@ void CppTree2Uml::parseFunctionDeclaration(  GroupAST* funSpec, GroupAST* storag
     UMLOperation *m = m_importer->makeOperation(c, id);
     parseFunctionArguments( d, m );
     m_importer->insertMethod( c, m, (Uml::Scope)m_currentAccess, returnType,
-			      isStatic, isPure, m_comment );
+			      isStatic, isPure, m_comment);
     m_comment = "";
 }
 
@@ -451,8 +549,8 @@ void CppTree2Uml::parseFunctionArguments(DeclaratorAST* declarator,
 
 	    QString tp = typeOfDeclaration( param->typeSpec(), param->declarator() );
 
-	    m_importer->addMethodParameter( method, tp, name, "" /*initialValue*/,
-	    							 "" /*doc*/);
+	    if (tp != "void")
+		m_importer->addMethodParameter( method, tp, name );
 	}
     }
 }
@@ -516,3 +614,18 @@ QStringList CppTree2Uml::scopeOfDeclarator( DeclaratorAST* d, const QStringList&
 {
     return scopeOfName( d->declaratorId(), startScope );
 }
+
+void CppTree2Uml::flushTemplateParams(UMLClass *klass) {
+    if (m_templateParams.count()) {
+	Umbrello::NameAndType_ListIt it;
+	for (it = m_templateParams.begin(); it != m_templateParams.end(); ++it) {
+	    const Umbrello::NameAndType &nt = *it;
+	    kdDebug() << "CppTree2Uml::parseClassSpecifier: adding template param: "
+	    	      << nt.first << endl;
+	    UMLTemplate *tmpl = klass->addTemplate(nt.first);
+	    tmpl->setType(nt.second);
+	}
+	m_templateParams.clear();
+    }
+}
+
