@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -189,8 +194,7 @@ private:
 	/**
 	 * Pastes the children of a UMLListViewItem (The Parent)
 	 */
-	bool pasteChildren(UMLListViewItem* Parent, UMLListViewItemListIt* It,
-			   IDChangeLog& ChangeLog, UMLDoc * Doc);
+	bool pasteChildren(UMLListViewItem* parent, IDChangeLog *chgLog, UMLDoc *doc);
 
 	/**
 	 * Gives a `sorry' message box if you're pasting an item which
