@@ -1,7 +1,11 @@
 /***************************************************************************
 			  cppwriter.cpp  -  description
-			     -------------------
+    This is the "old" code generator that does not support code editing
+    in the Modeller but uses significantly less file space because the
+    source code is not replicated in the XMI file.
+                             -------------------
     copyright	    : (C) 2003 Brian Thomas brian.thomas@gsfc.nasa.gov
+         (C) 2004  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
 ***************************************************************************/
 
 /***************************************************************************
@@ -13,41 +17,40 @@
  *									 *
  ***************************************************************************/
 
+// own header
 #include "cppwriter.h"
-
-#include <kdebug.h>
-
-#include <klocale.h>
-#include <kmessagebox.h>
+// qt/kde includes
 #include <qfile.h>
 #include <qtextstream.h>
 #include <qregexp.h>
-
+#include <kdebug.h>
+// app includes
 #include "classifierinfo.h"
+#include "codegen_utils.h"
 #include "../umldoc.h"
+#include "../classifier.h"
 #include "../class.h"
 #include "../interface.h"
 #include "../operation.h"
+#include "../template.h"
+#include "../umltemplatelist.h"
+#include "../umlclassifierlistitemlist.h"
 #include "../classifierlistitem.h"
-#include "../umlnamespace.h"
+#include "../model_utils.h"
 
 // 3-14-2003: this code developed from the javawriter with parts of the
 // original cppwriter by Luis De la Parra Blum
 
-CppWriter::CppWriter( QObject *parent, const char *name ) : CodeGenerator(parent, name)
+CppWriter::CppWriter( UMLDoc *parent, const char *name )
+  : SimpleCodeGenerator(parent, name)
 {
 
 	// set some general parameters for how to generate code in this class
-	INDENT = "\t";
-	IndentLevel = 0;
+	STRING_TYPENAME = "string";
+	STRING_TYPENAME_INCLUDE = "<string>";
 
-	STRING_TYPENAME = "QString";
-	STRING_TYPENAME_INCLUDE = "<qstring.h>";
-
-//	VECTOR_TYPENAME = "vector"; // std::vector
-//	VECTOR_TYPENAME_INCLUDE = "<vector.h>"; // std::vector
-	VECTOR_TYPENAME = "QPtrList"; // Qt library
-	VECTOR_TYPENAME_INCLUDE = "<qptrlist.h>"; // Qt library
+	VECTOR_TYPENAME = "vector"; // std::vector
+	VECTOR_TYPENAME_INCLUDE = "<vector>"; // std::vector
 
 	// Probably we could resolve this better through the use of templates,
 	// but its a quick n dirty fix for the timebeing.. until codegeneration
@@ -85,6 +88,17 @@ CppWriter::CppWriter( QObject *parent, const char *name ) : CodeGenerator(parent
 
 CppWriter::~CppWriter() { }
 
+QString CppWriter::getLanguage() {
+	return "C++";
+}
+
+bool CppWriter::isType (QString & type)
+{
+	if (type == "CppWriter")
+		return true;
+	return false;
+}
+
 void CppWriter::writeClass(UMLClassifier *c)
 {
 
@@ -143,39 +157,39 @@ void CppWriter::writeClass(UMLClassifier *c)
 
 void CppWriter::writeHeaderFile (UMLClassifier *c, QFile &fileh) {
 
-	// open stream for writting
+	// open stream for writing
 	QTextStream h (&fileh);
 
 	// up the indent level to one
-	IndentLevel = 1;
+	m_indentLevel = 1;
 
 	// write header blurb
 	QString str = getHeadingFile(".h");
 	if(!str.isEmpty()) {
 		str.replace(QRegExp("%filename%"),classifierInfo->fileName+".h");
 		str.replace(QRegExp("%filepath%"),fileh.name());
-		h<<str<<endl;
+		h<<str<< m_endl;
 	}
 
 	// Write the hash define stuff to prevent multiple parsing/inclusion of header
 	QString hashDefine = classifierInfo->className.upper().simplifyWhiteSpace().replace(QRegExp(" "),  "_");
 	writeBlankLine(h);
-	h << "#ifndef "<< hashDefine + "_H" << endl;
-	h << "#define "<< hashDefine + "_H" << endl;
+	h << "#ifndef "<< hashDefine + "_H" << m_endl;
+	h << "#define "<< hashDefine + "_H" << m_endl;
 
 	UMLClassifierList superclasses = classifierInfo->superclasses;
 	for(UMLClassifier *classifier = superclasses.first(); classifier ;classifier = superclasses.next()) {
 		QString headerName = findFileName(classifier, ".h");
 		if (headerName.isEmpty()) {
-			h<<"#include \""<<findFileName(classifier,".h")<<".h\""<<endl;
+			h<<"#include \""<<findFileName(classifier,".h")<<".h\""<<m_endl;
 		}
 	}
 
 	writeBlankLine(h);
-	h<<"#include "<<STRING_TYPENAME_INCLUDE<<endl;
+	h<<"#include "<<STRING_TYPENAME_INCLUDE<<m_endl;
 	if(classifierInfo->hasVectorFields)
 	{
-		h<<"#include "<<VECTOR_TYPENAME_INCLUDE<<endl;
+		h<<"#include "<<VECTOR_TYPENAME_INCLUDE<<m_endl;
 		writeBlankLine(h);
 	}
 
@@ -192,17 +206,17 @@ void CppWriter::writeHeaderFile (UMLClassifier *c, QFile &fileh) {
 
 	for(UMLClassifier *classifier = superclasses.first(); classifier ; classifier = superclasses.next()) {
 		if(classifier->getPackage()!=c->getPackage() && !classifier->getPackage().isEmpty()) {
-			h<<"using "<<cleanName(classifier->getPackage())<<"::"<<cleanName(classifier->getName())<<";"<<endl;
+			h<<"using "<<cleanName(classifier->getPackage())<<"::"<<cleanName(classifier->getName())<<";"<<m_endl;
 		}
 	}
 
 	if(!c->getPackage().isEmpty() && WRITE_PACKAGE_NAMESPACE)
-		h<<endl<<"namespace "<<cleanName(c->getPackage())<<" {"<<endl<<endl;
+		h<<m_endl<<"namespace "<<cleanName(c->getPackage())<<" {"<<m_endl<<m_endl;
 
 	//Write class Documentation if there is somthing or if force option
 	if(forceDoc() || !c->getDoc().isEmpty()) {
-		h<<endl<<"/**"<<endl;
-		h<<"  * class "<<classifierInfo->className<<endl;
+		h<<m_endl<<"/**"<<m_endl;
+		h<<"  * class "<<classifierInfo->className<<m_endl;
 		h<<formatDoc(c->getDoc(),"  * ");
 		h<<"  */";
 		writeBlankLine(h);
@@ -212,33 +226,63 @@ void CppWriter::writeHeaderFile (UMLClassifier *c, QFile &fileh) {
 	//check if class is abstract and / or has abstract methods
 	if((c->getAbstract() || classifierInfo->isInterface )
 			&& !hasAbstractOps(c))
-		h<<"/******************************* Abstract Class ****************************"<<endl
-		<<classifierInfo->className<<" does not have any pure virtual methods, but its author"<<endl
-		<<"  defined it as an abstract class, so you should not use it directly."<<endl
-		<<"  Inherit from it instead and create only objects from the derived classes"<<endl
-		<<"*****************************************************************************/"<<endl<<endl;
+		h<<"/******************************* Abstract Class ****************************"<<m_endl
+		<<classifierInfo->className<<" does not have any pure virtual methods, but its author"<<m_endl
+		<<"  defined it as an abstract class, so you should not use it directly."<<m_endl
+		<<"  Inherit from it instead and create only objects from the derived classes"<<m_endl
+		<<"*****************************************************************************/"<<m_endl<<m_endl;
 
 	if (!classifierInfo->isInterface) {
 		UMLClass* k = dynamic_cast<UMLClass*>(c);
 		if (k->isEnumeration()) {
-			h << "enum " << classifierInfo->className << " {" << endl;
-			UMLAttributeList* atl = k->getFilteredAttributeList();
-			for (UMLAttribute *at=atl->first(); at ; ) {
+			h << "enum " << classifierInfo->className << " {" << m_endl;
+			UMLAttributeList atl = k->getAttributeList();
+			UMLAttribute *at = atl.first();
+			while (at) {
 				QString attrName = cleanName(at->getName());
 				h << getIndent() << attrName;
-				if ((at=atl->next()) != 0)
-					h << ",";
-				else
+				if ((at = atl.next()) == NULL)
 					break;
-				h << endl;
+				h << "," << m_endl;
 			}
-			h << endl << "};" << endl;	// end of class header
+			h << m_endl << "};" << m_endl;	// end of class header
 	                if(!c->getPackage().isEmpty() && WRITE_PACKAGE_NAMESPACE)
-				h << "}  // end of package namespace" << endl;
-			h << endl << "#endif // " << hashDefine + "_H" << endl;
+				h << "}  // end of package namespace" << m_endl;
+			h << m_endl << "#endif // " << hashDefine + "_H" << m_endl;
+			return;
+		} else if (c->getBaseType() == Uml::ot_Enum) {
+			UMLClassifierListItemList litList = c->getFilteredList(Uml::ot_EnumLiteral);
+			uint i = 0;
+			h << "enum " << classifierInfo->className << " {" << m_endl;
+			for (UMLClassifierListItem *lit = litList.first(); lit; lit = litList.next()) {
+				QString enumLiteral = cleanName(lit->getName());
+				h << getIndent() << enumLiteral;
+				if (++i < litList.count())
+					h << ",";
+				h << m_endl;
+			}
+			h << m_endl << "};" << m_endl;	// end of class header
+	                if(!c->getPackage().isEmpty() && WRITE_PACKAGE_NAMESPACE)
+				h << "}  // end of package namespace" << m_endl;
+			h << m_endl << "#endif // " << hashDefine + "_H" << m_endl;
 			return;
 		}
 	}
+
+	// Generate template parameters.
+	UMLTemplateList template_params = c->getTemplateList();
+	if (template_params.count()) {
+		h << "template<";
+		for (UMLTemplate *t = template_params.first(); t; ) {
+			QString formalName = t->getName();
+			QString typeName = t->getTypeName();
+			h << typeName << " " << formalName;
+			if ((t = template_params.next()) != NULL)
+				h << ", ";
+		}
+		h << ">" << m_endl;
+	}
+
 	h << "class " << classifierInfo->className;
 	if (classifierInfo->superclasses.count() > 0)
 		h << " : ";
@@ -255,7 +299,7 @@ void CppWriter::writeHeaderFile (UMLClassifier *c, QFile &fileh) {
 			h << ", ";
 	}
 
-	h<<endl<<"{"<<endl; // begin the body of the class
+	h<<m_endl<<"{"<<m_endl; // begin the body of the class
 
 
 	//declarations of operations
@@ -266,7 +310,7 @@ void CppWriter::writeHeaderFile (UMLClassifier *c, QFile &fileh) {
 	//
 
 	// PUBLIC attribs/methods
-	h<<scopeToCPPDecl(Uml::Public)<<":"<<endl<<endl; // print visibility decl.
+	h<<"public:"<<m_endl<<m_endl; // print visibility decl.
 	// for public: constructors are first ops we print out
 	if(!classifierInfo->isInterface)
 		writeConstructorDecls(h);
@@ -276,33 +320,33 @@ void CppWriter::writeHeaderFile (UMLClassifier *c, QFile &fileh) {
 
 	// PROTECTED attribs/methods
 	//
-	h<<scopeToCPPDecl(Uml::Protected)<<":"<<endl<<endl; // print visibility decl.
+	h<<"protected"<<":"<<m_endl<<m_endl; // print visibility decl.
 	writeHeaderFieldDecl(c,Uml::Protected, h);
         writeHeaderAccessorMethodDecl(c, Uml::Protected, h);
 	writeOperations(c,true,Uml::Protected,h);
 
 	// PRIVATE attribs/methods
 	//
-	h<<scopeToCPPDecl(Uml::Private)<<":"<<endl<<endl; // print visibility decl.
+	h<<"private"<<":"<<m_endl<<m_endl; // print visibility decl.
 	writeHeaderFieldDecl(c,Uml::Private, h);
         writeHeaderAccessorMethodDecl(c, Uml::Private, h);
 	writeOperations(c,true,Uml::Private,h);
         writeInitAttibuteDecl(h); // this is always private, used by constructors to initialize class
 
 	// end of class header
-	h<<endl<<"};"<<endl;
+	h<<m_endl<<"};"<<m_endl;
 
 	// end of class namespace, if any
 	if(!c->getPackage().isEmpty() && WRITE_PACKAGE_NAMESPACE)
-		h<<"}; // end of package namespace"<<endl;
+		h<<"}; // end of package namespace"<<m_endl;
 
 	// last thing..close our hashdefine
-	h << endl << "#endif // " << hashDefine + "_H" << endl;
+	h << m_endl << "#endif // " << hashDefine + "_H" << m_endl;
 
 
 }
 
-void CppWriter::writeHeaderAccessorMethodDecl(UMLClassifier *c, Scope permitScope, QTextStream &stream)
+void CppWriter::writeHeaderAccessorMethodDecl(UMLClassifier *c, Uml::Scope permitScope, QTextStream &stream)
 {
 
 	// attributes
@@ -321,7 +365,7 @@ void CppWriter::writeHeaderAccessorMethodDecl(UMLClassifier *c, Scope permitScop
 
 }
 
-void CppWriter::writeHeaderFieldDecl(UMLClassifier *c, Scope permitScope, QTextStream &stream)
+void CppWriter::writeHeaderFieldDecl(UMLClassifier *c, Uml::Scope permitScope, QTextStream &stream)
 {
 	// attributes
         writeAttributeDecls(permitScope, true, stream); // write static attributes first
@@ -336,11 +380,11 @@ void CppWriter::writeHeaderFieldDecl(UMLClassifier *c, Scope permitScope, QTextS
 
 void CppWriter::writeSourceFile (UMLClassifier *c, QFile &filecpp ) {
 
-	// open writing stream
-	QTextStream cpp(&filecpp);
+	// open stream for writing
+	QTextStream cpp (&filecpp);
 
 	// set the starting indentation at zero
-	IndentLevel = 0;
+	m_indentLevel = 0;
 
 	//try to find a heading file (license, coments, etc)
 	QString str;
@@ -348,14 +392,14 @@ void CppWriter::writeSourceFile (UMLClassifier *c, QFile &filecpp ) {
 	if(!str.isEmpty()) {
 		str.replace(QRegExp("%filename%"),classifierInfo->fileName+".cpp");
 		str.replace(QRegExp("%filepath%"),filecpp.name());
-		cpp<<str<<endl;
+		cpp<<str<<m_endl;
 	}
 
 	// IMPORT statements
 	// Q: Why all utils? Isnt just List and Vector the only classes we are using?
 	// Our import *should* also look at operations, and check that objects being
 	// used arent in another package (and thus need to be explicitly imported here).
-	cpp<<"#include \""<<(classifierInfo->className).lower()<<".h\""<<endl;
+	cpp<<"#include \""<<(classifierInfo->className).lower()<<".h\""<<m_endl;
 	writeBlankLine(cpp);
 
 	// Start body of class
@@ -483,7 +527,7 @@ void CppWriter::writeClassDecl(UMLClassifier *c, QTextStream &cpp)
 
 }
 
-void CppWriter::writeAttributeDecls (Scope visibility, bool writeStatic, QTextStream &stream )
+void CppWriter::writeAttributeDecls (Uml::Scope visibility, bool writeStatic, QTextStream &stream )
 {
 
 	if(classifierInfo->isInterface)
@@ -518,7 +562,7 @@ void CppWriter::writeAttributeDecls (Scope visibility, bool writeStatic, QTextSt
         //write documentation
         if(forceDoc() || list->count() > 0)
         {
-                QString strVis = capitalizeFirstLetter(scopeToCPPDecl(visibility));
+                QString strVis = capitalizeFirstLetter(Umbrello::scopeToString(visibility));
                 QString strStatic = writeStatic ? "Static ":"";
                 writeComment(strStatic+strVis+" attributes",getIndent(), stream);
                 writeComment(" ",getIndent(), stream);
@@ -549,7 +593,7 @@ void CppWriter::writeAttributeDecls (Scope visibility, bool writeStatic, QTextSt
 	                QString typeName = fixTypeName(at->getTypeName());
 	                if(!documentation.isEmpty())
 	                        writeComment(documentation, getIndent(), stream);
-	                stream<<getIndent()<<staticValue<<typeName<<" "<<varName<<";"<<endl;
+	                stream<<getIndent()<<staticValue<<typeName<<" "<<varName<<";"<<m_endl;
 
 	        }
 
@@ -564,7 +608,7 @@ void CppWriter::writeAttributeDecls (Scope visibility, bool writeStatic, QTextSt
 
 }
 
-void CppWriter::writeHeaderAttributeAccessorMethods (Scope visibility, bool writeStatic, QTextStream &stream )
+void CppWriter::writeHeaderAttributeAccessorMethods (Uml::Scope visibility, bool writeStatic, QTextStream &stream )
 {
 
         UMLAttributeList * list;
@@ -601,7 +645,7 @@ void CppWriter::writeHeaderAttributeAccessorMethods (Scope visibility, bool writ
 // this is for writing *source* or *header* file attribute methods
 //
 void CppWriter::writeAttributeMethods(UMLAttributeList *attribs,
-					Scope visibility, bool isHeaderMethod,
+					Uml::Scope visibility, bool isHeaderMethod,
  					bool isStatic,
 					bool writeMethodBody, QTextStream &stream)
 {
@@ -611,7 +655,7 @@ void CppWriter::writeAttributeMethods(UMLAttributeList *attribs,
 
 	if(forceDoc() || attribs->count()>0)
 	{
-		QString strVis = capitalizeFirstLetter(scopeToCPPDecl(visibility));
+		QString strVis = capitalizeFirstLetter(Umbrello::scopeToString(visibility));
 		QString strStatic = (isStatic ? " static" : "");
         	writeBlankLine(stream);
 		writeComment(strVis+strStatic+" attribute accessor methods",getIndent(),stream);
@@ -637,7 +681,7 @@ void CppWriter::writeAttributeMethods(UMLAttributeList *attribs,
 		methodBaseName.replace(0,1,methodBaseName.at(0).upper());
 
 		writeSingleAttributeAccessorMethods(at->getTypeName(), varName,
-						    methodBaseName, at->getDoc(), chg_Changeable, isHeaderMethod,
+						    methodBaseName, at->getDoc(), Uml::chg_Changeable, isHeaderMethod,
 						    at->getStatic(), writeMethodBody, stream);
 	}
 
@@ -653,12 +697,12 @@ void CppWriter::writeComment(QString comment, QString myIndent, QTextStream &cpp
 		QStringList lines = QStringList::split( "\n", comment);
 		for(uint i= 0; i < lines.count(); i++)
 		{
-			cpp<<myIndent<<"// "<<lines[i] << endl;
+			cpp<<myIndent<<"// "<<lines[i] << m_endl;
 		}
 	} else {
 		// this should be more fancy in the future, breaking it up into 80 char
 		// lines so that it doesnt look too bad
-		cpp<<myIndent<<"// "<< comment << endl;
+		cpp<<myIndent<<"// "<< comment << m_endl;
 	}
 }
 
@@ -667,7 +711,7 @@ void CppWriter::writeDocumentation(QString header, QString body, QString end, QT
 	writeBlankLine(cpp);
 	QString indent = getIndent();
 
-	cpp<<indent<<"/**"<<endl;
+	cpp<<indent<<"/**"<<m_endl;
 	if (!header.isEmpty())
 		cpp<<formatDoc(header, indent+" * ");
 	if (!body.isEmpty())
@@ -678,10 +722,10 @@ void CppWriter::writeDocumentation(QString header, QString body, QString end, QT
 		for(uint i= 0; i < lines.count(); i++)
 			cpp<<formatDoc(lines[i], indent+" * ");
 	}
-	cpp<<indent<<" */"<<endl;
+	cpp<<indent<<" */"<<m_endl;
 }
 
-void CppWriter::writeAssociationDecls(UMLAssociationList associations, Scope permitScope, int id, QTextStream &h)
+void CppWriter::writeAssociationDecls(UMLAssociationList associations, Uml::Scope permitScope, Uml::IDType id, QTextStream &h)
 {
 
 	if( forceSections() || !associations.isEmpty() )
@@ -692,10 +736,10 @@ void CppWriter::writeAssociationDecls(UMLAssociationList associations, Scope per
 
 			// it may seem counter intuitive, but you want to insert the role of the
 			// *other* class into *this* class.
-			if (a->getRoleAId() == id && a->getRoleNameB() != "")
+			if (a->getRoleId(Uml::A) == id && a->getRoleName(Uml::B) != "")
 				printRoleB = true;
 
-			if (a->getRoleBId() == id && a->getRoleNameA() != "")
+			if (a->getRoleId(Uml::B) == id && a->getRoleName(Uml::A) != "")
 				printRoleA = true;
 
 			// First: we insert documentaion for association IF it has either role AND some documentation (!)
@@ -703,18 +747,18 @@ void CppWriter::writeAssociationDecls(UMLAssociationList associations, Scope per
 				writeComment(a->getDoc(), getIndent(), h);
 
 			// print RoleB decl
-			if (printRoleB && a->getVisibilityB() == permitScope)
+			if (printRoleB && a->getVisibility(Uml::B) == permitScope)
 			{
 
-				QString fieldClassName = cleanName(getUMLObjectName(a->getObjectB()));
-				writeAssociationRoleDecl(fieldClassName, a->getRoleNameB(), a->getMultiB(), a->getRoleBDoc(), h);
+				QString fieldClassName = cleanName(getUMLObjectName(a->getObject(Uml::B)));
+				writeAssociationRoleDecl(fieldClassName, a->getRoleName(Uml::B), a->getMulti(Uml::B), a->getRoleDoc(Uml::B), h);
 			}
 
 			// print RoleA decl
-			if (printRoleA && a->getVisibilityA() == permitScope)
+			if (printRoleA && a->getVisibility(Uml::A) == permitScope)
 			{
-				QString fieldClassName = cleanName(getUMLObjectName(a->getObjectA()));
-				writeAssociationRoleDecl(fieldClassName, a->getRoleNameA(), a->getMultiA(), a->getRoleADoc(), h);
+				QString fieldClassName = cleanName(getUMLObjectName(a->getObject(Uml::A)));
+				writeAssociationRoleDecl(fieldClassName, a->getRoleName(Uml::A), a->getMulti(Uml::A), a->getRoleDoc(Uml::A), h);
 			}
 
 			// reset for next association in our loop
@@ -758,7 +802,7 @@ void CppWriter::writeAssociationRoleDecl(QString fieldClassName, QString roleNam
                 	ObjectFieldVariables.append(fieldClassName);
 		}
 
-		stream<<indent<<fieldClassName<<" * "<<fieldVarName<<";"<<endl;
+		stream<<indent<<fieldClassName<<" * "<<fieldVarName<<";"<<m_endl;
 	}
 	else
 	{
@@ -770,17 +814,17 @@ void CppWriter::writeAssociationRoleDecl(QString fieldClassName, QString roleNam
                 	VectorFieldVariables.append(fieldVarName);
 
 		stream << indent << VECTOR_TYPENAME <<"<" << fieldClassName << "*";
-	       	stream << "> " << fieldVarName << ";" << endl;
+		stream << "> " << fieldVarName << ";" << m_endl;
 	}
 }
 
 // for either source or header files
 void CppWriter::writeAssociationMethods (UMLAssociationList associations,
-					Scope permitVisib,
+					Uml::Scope permitVisib,
 					bool isHeaderMethod,
 					bool writeMethodBody,
 					bool writePointerVar,
-					int myID, QTextStream &stream)
+					Uml::IDType myID, QTextStream &stream)
 {
 	if( forceSections() || !associations.isEmpty() )
 	{
@@ -789,32 +833,32 @@ void CppWriter::writeAssociationMethods (UMLAssociationList associations,
 
 			// insert the methods to access the role of the other
 			// class in the code of this one
-			if (a->getRoleAId() == myID && a->getVisibilityB() == permitVisib)
+			if (a->getRoleId(Uml::A) == myID && a->getVisibility(Uml::B) == permitVisib)
 			{
 				// only write out IF there is a rolename given
-				if(!a->getRoleNameB().isEmpty()) {
-					QString fieldClassName = getUMLObjectName(a->getObjectB()) + (writePointerVar ? " *":"");
+				if(!a->getRoleName(Uml::B).isEmpty()) {
+					QString fieldClassName = getUMLObjectName(a->getObject(Uml::B)) + (writePointerVar ? " *":"");
 					writeAssociationRoleMethod(fieldClassName,
 									isHeaderMethod,
 									writeMethodBody,
-								   a->getRoleNameB(),
-								   a->getMultiB(), a->getRoleBDoc(),
-								   a->getChangeabilityB(), stream);
+								   a->getRoleName(Uml::B),
+								   a->getMulti(Uml::B), a->getRoleDoc(Uml::B),
+								   a->getChangeability(Uml::B), stream);
 				}
 			}
 
-			if (a->getRoleBId() == myID && a->getVisibilityA() == permitVisib)
+			if (a->getRoleId(Uml::B) == myID && a->getVisibility(Uml::A) == permitVisib)
 			{
 				// only write out IF there is a rolename given
-				if(!a->getRoleNameA().isEmpty()) {
-					QString fieldClassName = getUMLObjectName(a->getObjectA()) + (writePointerVar ? " *":"");
+				if(!a->getRoleName(Uml::A).isEmpty()) {
+					QString fieldClassName = getUMLObjectName(a->getObject(Uml::A)) + (writePointerVar ? " *":"");
 					writeAssociationRoleMethod(fieldClassName,
 									isHeaderMethod,
 									writeMethodBody,
-								   a->getRoleNameA(),
-								   a->getMultiA(),
-								   a->getRoleADoc(),
-								   a->getChangeabilityA(),
+								   a->getRoleName(Uml::A),
+								   a->getMulti(Uml::A),
+								   a->getRoleDoc(Uml::A),
+								   a->getChangeability(Uml::A),
 								   stream);
 				}
 			}
@@ -824,11 +868,11 @@ void CppWriter::writeAssociationMethods (UMLAssociationList associations,
 }
 
 void CppWriter::writeAssociationRoleMethod (QString fieldClassName,
-						bool isHeaderMethod,
-						bool writeMethodBody,
-						QString roleName, QString multi,
-					     QString description, Changeability_Type change,
-					     QTextStream &stream)
+					    bool isHeaderMethod,
+					    bool writeMethodBody,
+					    QString roleName, QString multi,
+					    QString description, Uml::Changeability_Type change,
+					    QTextStream &stream)
 {
 	if(multi.isEmpty() || multi.contains(QRegExp("^[01]$")))
 	{
@@ -846,7 +890,7 @@ void CppWriter::writeAssociationRoleMethod (QString fieldClassName,
 
 void CppWriter::writeVectorAttributeAccessorMethods (QString fieldClassName, QString fieldVarName,
 						      QString fieldName, QString description,
-						      Changeability_Type changeType,
+						      Uml::Changeability_Type changeType,
 						      bool isHeaderMethod,
 						      bool writeMethodBody,
 						      QTextStream &stream)
@@ -857,7 +901,7 @@ void CppWriter::writeVectorAttributeAccessorMethods (QString fieldClassName, QSt
 	QString indent = getIndent();
 
 	// ONLY IF changeability is NOT Frozen
-	if (changeType != chg_Frozen)
+	if (changeType != Uml::chg_Frozen)
 	{
 		writeDocumentation("Add a "+fieldName+" object to the "+fieldVarName+" List",description,"",stream);
 		stream<<indent<<"void ";
@@ -869,17 +913,17 @@ void CppWriter::writeVectorAttributeAccessorMethods (QString fieldClassName, QSt
 			method.replace(QRegExp("%VARNAME%"),fieldVarName);
 			method.replace(QRegExp("%VECTORTYPENAME%"), VECTOR_TYPENAME);
 			method.replace(QRegExp("%ITEMCLASS%"),fieldClassName);
-			stream<<indent<<" {"<<endl;
-	                IndentLevel++;
+			stream<<indent<<" {"<<m_endl;
+	                m_indentLevel++;
 			printTextAsSeparateLinesWithIndent(method,getIndent(),stream);
-	                IndentLevel--;
-			stream<<indent<<"}"<<endl;
+	                m_indentLevel--;
+			stream<<indent<<"}"<<m_endl;
 		} else
-			stream<<";"<<endl;
+			stream<<";"<<m_endl;
 	}
 
 	// ONLY IF changeability is Changeable
-	if (changeType == chg_Changeable)
+	if (changeType == Uml::chg_Changeable)
 	{
 		writeDocumentation("Remove a "+fieldName+" object from "+fieldVarName+" List",description,"",stream);
 		stream<<indent<<"void ";
@@ -891,13 +935,13 @@ void CppWriter::writeVectorAttributeAccessorMethods (QString fieldClassName, QSt
 			method.replace(QRegExp("%VARNAME%"),fieldVarName);
 			method.replace(QRegExp("%VECTORTYPENAME%"), VECTOR_TYPENAME);
 			method.replace(QRegExp("%ITEMCLASS%"),fieldClassName);
-			stream<<indent<<" {"<<endl;
-	                IndentLevel++;
+			stream<<indent<<" {"<<m_endl;
+	                m_indentLevel++;
 			printTextAsSeparateLinesWithIndent(method,getIndent(),stream);
-	                IndentLevel--;
-			stream<<indent<<"}"<<endl;
+	                m_indentLevel--;
+			stream<<indent<<"}"<<m_endl;
 		} else
-			stream<<";"<<endl;
+			stream<<";"<<m_endl;
 	}
 
 	// always allow getting the list of stuff
@@ -908,20 +952,20 @@ void CppWriter::writeVectorAttributeAccessorMethods (QString fieldClassName, QSt
 		stream<<classifierInfo->className<<"::";
 	stream<<"get"<<fieldName<<"List ( )";
 	if(writeMethodBody) {
-		stream<<indent<<" {"<<endl;
-		IndentLevel++;
-		stream<<getIndent()<<"return "<<fieldVarName<<";"<<endl;
-		IndentLevel--;
-		stream<<indent<<"}"<<endl;
+		stream<<indent<<" {"<<m_endl;
+		m_indentLevel++;
+		stream<<getIndent()<<"return "<<fieldVarName<<";"<<m_endl;
+		m_indentLevel--;
+		stream<<indent<<"}"<<m_endl;
 	} else
-		stream<<";"<<endl;
+		stream<<";"<<m_endl;
 
 }
 
 
 void CppWriter::writeSingleAttributeAccessorMethods(QString fieldClassName, QString fieldVarName,
 						     QString fieldName, QString description,
-						     Changeability_Type change,
+						     Uml::Changeability_Type change,
 						     bool isHeaderMethod,
 						     bool isStatic,
 						     bool writeMethodBody,
@@ -937,7 +981,7 @@ void CppWriter::writeSingleAttributeAccessorMethods(QString fieldClassName, QStr
 	QString indent = getIndent();
 
 	// set method
-	if (change == chg_Changeable && !isStatic) {
+	if (change == Uml::chg_Changeable && !isStatic) {
 		writeDocumentation("Set the value of "+fieldVarName,description,"@param new_var the new value of "+fieldVarName,stream);
 		stream<<indent<<"void ";
 		if(!isHeaderMethod)
@@ -945,16 +989,16 @@ void CppWriter::writeSingleAttributeAccessorMethods(QString fieldClassName, QStr
 		stream<<"set"<<fieldName<<" ( "<<fieldClassName<<" new_var )";
 
 		if(writeMethodBody) {
-			stream<<indent<<" {"<<endl;
-			IndentLevel++;
+			stream<<indent<<" {"<<m_endl;
+			m_indentLevel++;
 			stream<<getIndent()<<indent;
-			IndentLevel--;
+			m_indentLevel--;
 			if(isStatic)
 				stream<<classifierInfo->className<<"::";
-			stream<<fieldVarName<<" = new_var;"<<endl;
-			stream<<indent<<"}"<<endl;
+			stream<<fieldVarName<<" = new_var;"<<m_endl;
+			stream<<indent<<"}"<<m_endl;
 		} else
-		       stream<<";"<<endl;
+		       stream<<";"<<m_endl;
 	}
 
 	// get method
@@ -965,16 +1009,16 @@ void CppWriter::writeSingleAttributeAccessorMethods(QString fieldClassName, QStr
 	stream<<"get"<<fieldName<<" ( )";
 
 	if(writeMethodBody) {
-		stream<<indent<<" {"<<endl;
-		IndentLevel++;
+		stream<<indent<<" {"<<m_endl;
+		m_indentLevel++;
 		stream<<getIndent()<<"return ";
-		IndentLevel--;
+		m_indentLevel--;
 		if(isStatic)
 			stream<<classifierInfo->className<<"::";
-		stream<<fieldVarName<<";"<<endl;
+		stream<<fieldVarName<<";"<<m_endl;
 		stream<<indent<<"}";
 	} else
-		       stream<<";"<<endl;
+		       stream<<";"<<m_endl;
 
 	writeBlankLine(stream);
 }
@@ -993,7 +1037,7 @@ void CppWriter::writeConstructorDecls(QTextStream &stream)
 	if(WRITE_EMPTY_CONSTRUCTOR)
 	{
 		writeDocumentation("", "Empty Constructor", "", stream);
-		stream<<getIndent()<<classifierInfo->className<<" ( );"<<endl;
+		stream<<getIndent()<<classifierInfo->className<<" ( );"<<m_endl;
 	}
 
 	if(WRITE_EMPTY_DESTRUCTOR)
@@ -1002,7 +1046,7 @@ void CppWriter::writeConstructorDecls(QTextStream &stream)
 		stream<<getIndent();
 		if (WRITE_VIRTUAL_DESTRUCTORS)
 			stream<<"virtual ";
-		stream<<"~"<<classifierInfo->className<<" ( );"<<endl;
+		stream<<"~"<<classifierInfo->className<<" ( );"<<m_endl;
 	}
 
 	if(WRITE_EMPTY_DESTRUCTOR || WRITE_EMPTY_CONSTRUCTOR)
@@ -1012,7 +1056,7 @@ void CppWriter::writeConstructorDecls(QTextStream &stream)
 void CppWriter::writeInitAttibuteDecl (QTextStream &stream)
 {
 	if(WRITE_EMPTY_CONSTRUCTOR && classifierInfo->hasAttributes)
-		stream<<getIndent()<<"void initAttributes ( ) ;"<<endl;
+		stream<<getIndent()<<"void initAttributes ( ) ;"<<m_endl;
 }
 
 void CppWriter::writeInitAttibuteMethod (QTextStream &stream)
@@ -1024,49 +1068,49 @@ void CppWriter::writeInitAttibuteMethod (QTextStream &stream)
 		QString className = classifierInfo->className;
 		QString indent = getIndent();
 
-		stream<<indent<<"void "<<className<<"::"<<"initAttributes ( ) {"<<endl;
+		stream<<indent<<"void "<<className<<"::"<<"initAttributes ( ) {"<<m_endl;
 
-		IndentLevel++;
+		m_indentLevel++;
 		// first, initiation of fields derived from attributes
 		UMLAttributeList* atl = classifierInfo->getAttList();
 		for(UMLAttribute *at = atl->first(); at ; at = atl->next()) {
 			if(!at->getInitialValue().isEmpty()) {
 				QString varName = getAttributeVariableName(at);
-				stream<<getIndent()<<varName<<" = "<<at->getInitialValue()<<";"<<endl;
+				stream<<getIndent()<<varName<<" = "<<at->getInitialValue()<<";"<<m_endl;
 			}
 		}
 		// Now initialize the association related fields (e.g. vectors)
 		if (VECTOR_METHOD_INIT != "") {
   			QStringList::Iterator it;
-	        	for( it = VectorFieldVariables.begin(); it != VectorFieldVariables.end(); ++it ) {
-	                	QString fieldVarName = *it;
-	                        QString method = VECTOR_METHOD_INIT;
-	                        method.replace(QRegExp("%VARNAME%"),fieldVarName);
-	                        method.replace(QRegExp("%VECTORTYPENAME%"), VECTOR_TYPENAME);
-				stream<<getIndent()<<method<<endl;
-	        	}
+			for( it = VectorFieldVariables.begin(); it != VectorFieldVariables.end(); ++it ) {
+				QString fieldVarName = *it;
+				QString method = VECTOR_METHOD_INIT;
+				method.replace(QRegExp("%VARNAME%"),fieldVarName);
+				method.replace(QRegExp("%VECTORTYPENAME%"), VECTOR_TYPENAME);
+				stream<<getIndent()<<method<<m_endl;
+			}
 		}
 
 		if (OBJECT_METHOD_INIT != "") {
   			QStringList::Iterator it;
-	        	for( it = ObjectFieldVariables.begin(); it != ObjectFieldVariables.end(); ++it ) {
-	                	QString fieldVarName = *it;
+			for( it = ObjectFieldVariables.begin(); it != ObjectFieldVariables.end(); ++it ) {
+				QString fieldVarName = *it;
 				it++;
-	                	QString fieldClassName = *it;
-	                        QString method = OBJECT_METHOD_INIT;
-	                        method.replace(QRegExp("%VARNAME%"),fieldVarName);
+				QString fieldClassName = *it;
+				QString method = OBJECT_METHOD_INIT;
+				method.replace(QRegExp("%VARNAME%"),fieldVarName);
 				method.replace(QRegExp("%ITEMCLASS%"),fieldClassName);
-				stream<<getIndent()<<method<<endl;
-	        	}
+				stream<<getIndent()<<method<<m_endl;
+			}
 		}
 
 		// clean up
 		ObjectFieldVariables.clear(); // shouldnt be needed?
 		VectorFieldVariables.clear(); // shouldnt be needed?
 
-		IndentLevel--;
+		m_indentLevel--;
 
-		stream<<indent<<"}"<<endl;
+		stream<<indent<<"}"<<m_endl;
 	}
 }
 
@@ -1087,17 +1131,17 @@ void CppWriter::writeConstructorMethods(QTextStream &stream)
 	if(WRITE_EMPTY_CONSTRUCTOR)
 	{
 		QString indent = getIndent();
-		stream<<indent<<className<<"::"<<className<<" ( ) {"<<endl;
+		stream<<indent<<className<<"::"<<className<<" ( ) {"<<m_endl;
 		if(classifierInfo->hasAttributes)
-			stream<<indent<<INDENT<<"initAttributes();"<<endl;
-		stream<<indent<<"};"<<endl;
+			stream<<indent<<INDENT<<"initAttributes();"<<m_endl;
+		stream<<indent<<"};"<<m_endl;
 		writeBlankLine(stream);
 	}
 
 	// empty destructor
 	if(WRITE_EMPTY_DESTRUCTOR)
 	{
-		stream<<getIndent()<<className<<"::~"<<className<<" ( ) { };"<<endl;
+		stream<<getIndent()<<className<<"::~"<<className<<" ( ) { };"<<m_endl;
 		writeBlankLine(stream);
 	}
 
@@ -1107,20 +1151,20 @@ void CppWriter::writeConstructorMethods(QTextStream &stream)
 // the Java Object "String" (there is no string primative in Java).
 QString CppWriter::fixTypeName(QString string)
 {
+	if (string.isEmpty())
+		return "void";
 	string.replace(QRegExp("^string$"), STRING_TYPENAME);
 	return string;
 }
 
-void CppWriter::writeOperations(UMLClassifier *c, bool isHeaderMethod, Scope permitScope, QTextStream &cpp) {
+void CppWriter::writeOperations(UMLClassifier *c, bool isHeaderMethod,
+				Uml::Scope permitScope, QTextStream &cpp) {
 
-	UMLOperationList *opl;
 	UMLOperationList oplist;
-	oplist.setAutoDelete(false);
 
-	UMLOperation *op;
-	//sort operations by scope first and see if there are abstrat methods
-	opl = c->getFilteredOperationsList();
-	for(op = opl->first(); op ; op=opl->next()) {
+	//sort operations by scope first and see if there are abstract methods
+	UMLOperationList inputlist = c->getOpList();
+	for (UMLOperation *op = inputlist.first(); op; op = inputlist.next()) {
 		switch(op->getScope()) {
 		case Uml::Public:
 			if(permitScope == Uml::Public)
@@ -1147,7 +1191,6 @@ void CppWriter::writeOperations(UMLClassifier *c, bool isHeaderMethod, Scope per
 	*/
 	writeOperations(oplist,isHeaderMethod, cpp);
 
-
 }
 
 // write operation in either header or
@@ -1167,26 +1210,26 @@ void CppWriter::writeOperations(UMLOperationList &oplist, bool isHeaderMethod, Q
 		QString returnStr = "";
 		// write documentation
 
-		QString methodReturnType = fixTypeName(op->getReturnType());
+		QString methodReturnType = fixTypeName(op->getTypeName());
 		if(methodReturnType != "void")
 			returnStr += "@return	"+methodReturnType+"\n";
 
 		str = ""; // reset for next method
 		if (op->getAbstract() || classifierInfo->isInterface) {
-                	if (isHeaderMethod) {
+			if (isHeaderMethod) {
  				// declare abstract method as 'virtual'
 				str += "virtual ";
 			}
-                }
+		}
 
 		// static declaration for header file
-                if (isHeaderMethod)
+		if (isHeaderMethod)
 			str += ((op->getStatic() && isHeaderMethod) ? "static ":"");
 
 		// returntype of method
 		str += methodReturnType + " ";
 
-                if (!isHeaderMethod)
+		if (!isHeaderMethod)
 			str += className + "::";
 
 		str += cleanName(op->getName()) + " (";
@@ -1215,7 +1258,7 @@ void CppWriter::writeOperations(UMLOperationList &oplist, bool isHeaderMethod, Q
 
 		// write it out
 		writeDocumentation("", op->getDoc(), returnStr, cpp);
-		cpp<<getIndent()<<str<<endl;
+		cpp<<getIndent()<<str<<m_endl;
 		writeBlankLine(cpp);
 	}
 }
@@ -1224,7 +1267,7 @@ void CppWriter::writeOperations(UMLOperationList &oplist, bool isHeaderMethod, Q
 // of an association have roles we need to have forward declaration of
 // the other class...but only IF its not THIS class (as could happen
 // in self-association relationship).
-void CppWriter::printAssociationIncludeDecl (UMLAssociationList list, int myId, QTextStream &stream)
+void CppWriter::printAssociationIncludeDecl (UMLAssociationList list, Uml::IDType myId, QTextStream &stream)
 {
 
 	for (UMLAssociation *a = list.first(); a; a = list.next()) {
@@ -1233,20 +1276,20 @@ void CppWriter::printAssociationIncludeDecl (UMLAssociationList list, int myId,
 
 		// only use OTHER classes (e.g. we dont need to write includes for ourselves!!
 		// AND only IF the roleName is defined, otherwise, its not meant to be noticed.
-		if (a->getRoleAId() == myId && a->getRoleNameB() != "") {
-			current = dynamic_cast<UMLClassifier*>(a->getObjectB());
-		} else if (a->getRoleBId() == myId && a->getRoleNameA() != "") {
-			current = dynamic_cast<UMLClassifier*>(a->getObjectA());
+		if (a->getRoleId(Uml::A) == myId && a->getRoleName(Uml::B) != "") {
+			current = dynamic_cast<UMLClassifier*>(a->getObject(Uml::B));
+		} else if (a->getRoleId(Uml::B) == myId && a->getRoleName(Uml::A) != "") {
+			current = dynamic_cast<UMLClassifier*>(a->getObject(Uml::A));
 			isFirstClass = false;
 		}
 
 		// as header doc for this method indicates, we need to be a bit sophisticated on
 		// how to declare some associations.
 		if( current )
-			if( !isFirstClass && !a->getRoleNameA().isEmpty() && !a->getRoleNameB().isEmpty())
-				stream<<"class "<<current->getName()<<";"<<endl; // special case: use forward declaration
+			if( !isFirstClass && !a->getRoleName(Uml::A).isEmpty() && !a->getRoleName(Uml::B).isEmpty())
+				stream<<"class "<<current->getName()<<";"<<m_endl; // special case: use forward declaration
 			else
-				stream<<"#include \""<<current->getName().lower()<<".h\""<<endl; // just the include statement
+				stream<<"#include \""<<current->getName().lower()<<".h\""<<m_endl; // just the include statement
 	}
 }
 
@@ -1262,25 +1305,6 @@ QString CppWriter::fixInitialStringDeclValue(QString value, QString type)
 	return value;
 }
 
-QString CppWriter::scopeToCPPDecl(Uml::Scope scope)
-{
-	QString scopeString;
-	switch(scope)
-	{
-		case Uml::Public:
-			scopeString = "public";
-			break;
-		case Uml::Protected:
-			scopeString = "protected";
-			break;
-		case Uml::Private:
-		default:
-			scopeString = "private";
-			break;
-	}
-	return scopeString;
-}
-
 // methods like this _shouldnt_ be needed IF we properly did things thruought the code.
 QString CppWriter::getUMLObjectName(UMLObject *obj)
 {
@@ -1298,15 +1322,7 @@ QString CppWriter::capitalizeFirstLetter(QString string)
 
 void CppWriter::writeBlankLine(QTextStream &stream)
 {
-	stream<<endl;
-}
-
-QString CppWriter::getIndent ()
-{
-	QString myIndent = "";
-	for (int i = 0 ; i < IndentLevel ; i++)
-		myIndent.append(INDENT);
-	return myIndent;
+	stream<<m_endl;
 }
 
 void CppWriter::printTextAsSeparateLinesWithIndent (QString text, QString indent, QTextStream &stream)
@@ -1316,7 +1332,7 @@ void CppWriter::printTextAsSeparateLinesWithIndent (QString text, QString indent
 
 	QStringList lines = QStringList::split( "\n", text);
 	for(uint i= 0; i < lines.count(); i++)
-		stream<<indent<<lines[i]<<endl;
+		stream<<indent<<lines[i]<<m_endl;
 }
 
 QString CppWriter::getAttributeVariableName (UMLAttribute *at)
@@ -1324,3 +1340,12 @@ QString CppWriter::getAttributeVariableName (UMLAttribute *at)
 	QString fieldName = "m_" + cleanName(at->getName());
 	return fieldName;
 }
+
+void CppWriter::createDefaultDatatypes() {
+	Umbrello::createCppDatatypes();
+}
+
+const QStringList CppWriter::reservedKeywords() const {
+	return Umbrello::reservedCppKeywords();
+}
+
