@@ -1,6 +1,4 @@
 /***************************************************************************
-                          idlwriter.cpp  -  description
-                             -------------------
     begin                : Sat Jan 4 2003
     copyright            : (C) 2003 by Oliver Kellogg
     email                : okellogg@users.sourceforge.net
@@ -27,8 +25,8 @@
 #include "../umldoc.h"
 #include "../class.h"
 #include "../enum.h"
-#include "../enumliteral.h"
-#include "../umlenumliterallist.h"
+#include "../classifierlistitem.h"
+#include "../umlclassifierlistitemlist.h"
 #include "../package.h"
 #include "../association.h"
 #include "../attribute.h"
@@ -37,29 +35,16 @@
 
 IDLWriter::IDLWriter(UMLDoc *parent, const char *name)
 		: SimpleCodeGenerator(parent, name, false) {
-	indentlevel = 0;
-	// FIXME: Eventually we should fabricate an Indenter class
-	// that can be used by all code generators.
-	// NOTE: this has now been done in new code gen class. -b.t.
 }
 
 IDLWriter::~IDLWriter() {}
 
-QString IDLWriter::spc() {
-	QString s;
-	if (m_indentation == " ")
-		s.fill(' ', indentlevel * m_indentationAmount);
-	else
-		s.fill('\t', indentlevel * m_indentationAmount);
-	return s;
-}
-
 bool IDLWriter::isOOClass(UMLClassifier *c) {
-	QString stype = c->getStereotype();
+	QString stype = c->getStereotype(false);
 	if (stype == "CORBAConstant" || stype == "CORBAEnum" ||
-	        stype == "CORBAStruct" || stype == "CORBAUnion" ||
-	        stype == "CORBASequence" || stype == "CORBAArray" ||
-	        stype == "CORBATypedef")
+	    stype == "CORBAStruct" || stype == "CORBAUnion" ||
+	    stype == "CORBASequence" || stype == "CORBAArray" ||
+	    stype == "CORBATypedef")
 		return false;
 
 	// CORBAValue, CORBAInterface, and all empty/unknown stereotypes are
@@ -67,6 +52,11 @@ bool IDLWriter::isOOClass(UMLClassifier *c) {
 	return true;
 }
 
+bool IDLWriter::assocTypeIsMappableToAttribute(Uml::Association_Type at) {
+	return (at == Uml::at_Aggregation || at == Uml::at_Association ||
+		at == Uml::at_Composition || at == Uml::at_UniAssociation);
+}
+
 QString IDLWriter::qualifiedName(UMLClassifier *c) {
 	UMLPackage *umlPkg = c->getUMLPackage();
 	QString className = cleanName(c->getName());
@@ -84,7 +74,7 @@ QString IDLWriter::qualifiedName(UMLClassifier *c) {
  * returns "IDL"
  */
 QString IDLWriter::getLanguage() {
-        return "IDL";
+	return "IDL";
 }
 
 /**
@@ -94,9 +84,9 @@ QString IDLWriter::getLanguage() {
  */
 bool IDLWriter::isType (QString & type)
 {
-   if(type == "IDLWriter")
-        return true;
-   return false;
+	if(type == "IDLWriter")
+		return true;
+	return false;
 }
 
 void IDLWriter::computeAssocTypeAndRole
@@ -104,25 +94,35 @@ void IDLWriter::computeAssocTypeAndRole
 {
 	// Determine which is the "remote" end of the association:
 	bool IAmRoleA = true;
-	UMLObject *other = a->getObject(B);
+	UMLObject *other = a->getObject(Uml::B);
+	Uml::Association_Type at = a->getAssocType();
 	if (c->getName() == other->getName()) {
+		if (at == Uml::at_Aggregation || at == Uml::at_Composition ||
+		    at == Uml::at_UniAssociation) {
+			// Assuming unidirectional association, and we are
+			// at the "wrong" side.
+			// Returning roleName = QString::null tells caller to
+			// skip this association at this side.
+			roleName = QString::null;
+			return;
+		}
 		IAmRoleA = false;
-		other = a->getObject(A);
+		other = a->getObject(Uml::A);
 	}
 	// Construct the type name:
 	typeName = cleanName(other->getName());
 	QString multiplicity;
 	if (IAmRoleA)
-		multiplicity = a->getMulti(B);
+		multiplicity = a->getMulti(Uml::B);
 	else
-		multiplicity = a->getMulti(A);
+		multiplicity = a->getMulti(Uml::A);
 	if (!multiplicity.isEmpty() && multiplicity != "1")
 		typeName.append("Vector");
 	// Construct the member name:
 	if (IAmRoleA)
-		roleName = a->getRoleName(B);
+		roleName = a->getRoleName(Uml::B);
 	else
-		roleName = a->getRoleName(A);
+		roleName = a->getRoleName(Uml::A);
 	if (roleName.isEmpty()) {
 		roleName = a->getName();
 		if (roleName.isEmpty()) {
@@ -162,7 +162,7 @@ void IDLWriter::writeClass(UMLClassifier *c) {
 	if (!str.isEmpty()) {
 		str.replace(QRegExp("%filename%"), fileName);
 		str.replace(QRegExp("%filepath%"), file.name());
-		idl << str << endl;
+		idl << str << m_endl;
 	}
 
 	// Write includes.
@@ -174,9 +174,9 @@ void IDLWriter::writeClass(UMLClassifier *c) {
 				continue;
 			QString baseName = findFileName(conc, ".idl");
 			if (!baseName.isEmpty())
-				idl << "#include \"" << baseName << ".idl\"" << m_newLineEndingChars;
+				idl << "#include \"" << baseName << ".idl\"" << m_endl;
 		}
-		idl << m_newLineEndingChars;
+		idl << m_endl;
 	}
 
 	// Generate the module declaration(s) for the package(s) in which
@@ -184,42 +184,41 @@ void IDLWriter::writeClass(UMLClassifier *c) {
 	UMLPackageList pkgList = c->getPackages();
 	UMLPackage *pkg;
 	for (pkg = pkgList.first(); pkg != NULL; pkg = pkgList.next()) {
-		idl << spc() << "module " << pkg->getName() << " {" << m_newLineEndingChars << m_newLineEndingChars;
-		indentlevel++;
+		idl << getIndent() << "module " << pkg->getName() << " {" << m_endl << m_endl;
+		m_indentLevel++;
 	}
 
 	// Write class Documentation if non-empty or if force option set.
 	if (forceDoc() || !c->getDoc().isEmpty()) {
-		idl << "//" << m_newLineEndingChars;
-		idl << "// class " << classname << endl;
+		idl << "//" << m_endl;
+		idl << "// class " << classname << m_endl;
 		idl << formatDoc(c->getDoc(), "// ");
-		idl << m_newLineEndingChars;
+		idl << m_endl;
 	}
 
 	if (c->getBaseType() == Uml::ot_Enum) {
-		UMLEnum *ue = static_cast<UMLEnum*>(c);
-		UMLEnumLiteralList litList = ue->getFilteredEnumLiteralList();
+		UMLClassifierListItemList litList = c->getFilteredList(Uml::ot_EnumLiteral);
 		uint i = 0;
-		idl << spc() << "enum " << classname << " {" << m_newLineEndingChars;
-		indentlevel++;
-		for (UMLEnumLiteral *lit = litList.first(); lit; lit = litList.next()) {
+		idl << getIndent() << "enum " << classname << " {" << m_endl;
+		m_indentLevel++;
+		for (UMLClassifierListItem *lit = litList.first(); lit; lit = litList.next()) {
 			QString enumLiteral = cleanName(lit->getName());
-			idl << spc() << enumLiteral;
+			idl << getIndent() << enumLiteral;
 			if (++i < litList.count())
 				idl << ",";
-			idl << endl;
+			idl << m_endl;
 		}
-		indentlevel--;
-		idl << spc() << "};" << m_newLineEndingChars << m_newLineEndingChars;
+		m_indentLevel--;
+		idl << getIndent() << "};" << m_endl << m_endl;
 		// Close the modules inside which we might be nested.
 		for (pkg = pkgList.first(); pkg != NULL; pkg = pkgList.next()) {
-			indentlevel--;
-			idl << spc() << "};" << m_newLineEndingChars << m_newLineEndingChars;
+			m_indentLevel--;
+			idl << getIndent() << "};" << m_endl << m_endl;
 		}
 		return;
 	}
 	if (! isOOClass(c)) {
-		QString stype = c->getStereotype();
+		QString stype = c->getStereotype(false);
 		if (stype == "CORBAConstant") {
 			kdError() << "The stereotype " << stype << " cannot be applied to "
 				  << c->getName() << ", but only to attributes." << endl;
@@ -232,73 +231,78 @@ void IDLWriter::writeClass(UMLClassifier *c) {
 			return;
 		}
 		if (stype == "CORBAEnum") {
-			UMLAttributeList *atl = myClass->getFilteredAttributeList();
+			UMLAttributeList atl = myClass->getAttributeList();
 			UMLAttribute *at;
-			idl << spc() << "enum " << classname << " {" << m_newLineEndingChars;
-			indentlevel++;
+			idl << getIndent() << "enum " << classname << " {" << m_endl;
+			m_indentLevel++;
 			uint i = 0;
-			for (at = atl->first(); at; at = atl->next()) {
+			for (at = atl.first(); at; at = atl.next()) {
 				QString enumLiteral = cleanName(at->getName());
-				idl << spc() << enumLiteral;
-				if (++i < atl->count())
+				idl << getIndent() << enumLiteral;
+				if (++i < atl.count())
 					idl << ",";
-				idl << endl;
+				idl << m_endl;
 			}
-			indentlevel--;
-			idl << spc() << "};" << m_newLineEndingChars << m_newLineEndingChars;
+			m_indentLevel--;
+			idl << getIndent() << "};" << m_endl << m_endl;
 		} else if (stype == "CORBAStruct") {
-			UMLAttributeList *atl = myClass->getFilteredAttributeList();
+			UMLAttributeList atl = myClass->getAttributeList();
 			UMLAttribute *at;
-			idl << spc() << "struct " << classname << " {" << m_newLineEndingChars;
-			indentlevel++;
-			for (at = atl->first(); at; at = atl->next()) {
+			idl << getIndent() << "struct " << classname << " {" << m_endl;
+			m_indentLevel++;
+			for (at = atl.first(); at; at = atl.next()) {
 				QString name = cleanName(at->getName());
-				idl << spc() << at->getTypeName() << " " << name << ";" << m_newLineEndingChars;
+				idl << getIndent() << at->getTypeName() << " " << name << ";" << m_endl;
 				// Initial value not possible in IDL.
 			}
 			UMLAssociationList compositions = c->getCompositions();
 			if (!compositions.isEmpty()) {
-				idl << spc() << "// Compositions." << m_newLineEndingChars;
+				idl << getIndent() << "// Compositions." << m_endl;
 				for (UMLAssociation *a = compositions.first(); a; a = compositions.next()) {
 					QString memberType, memberName;
 					computeAssocTypeAndRole(a, c, memberType, memberName);
-					idl << spc() << memberType << " " << memberName << ";" << m_newLineEndingChars;
+					idl << getIndent() << memberType << " " << memberName << ";" << m_endl;
 				}
 			}
 			UMLAssociationList aggregations = c->getAggregations();
 			if (!aggregations.isEmpty()) {
-				idl << spc() << "// Aggregations." << m_newLineEndingChars;
+				idl << getIndent() << "// Aggregations." << m_endl;
 				for (UMLAssociation *a = aggregations.first(); a; a = aggregations.next()) {
 					QString memberType, memberName;
 					computeAssocTypeAndRole(a, c, memberType, memberName);
-					idl << spc() << memberType << " " << memberName << ";" << m_newLineEndingChars;
+					idl << getIndent() << memberType << " " << memberName << ";" << m_endl;
 				}
 			}
-			indentlevel--;
-			idl << spc() << "};" << m_newLineEndingChars << m_newLineEndingChars;
+			m_indentLevel--;
+			idl << getIndent() << "};" << m_endl << m_endl;
 		} else if (stype == "CORBAUnion") {
-			idl << spc() << "// " << stype << " " << c->getName()
-			    << " is Not Yet Implemented" << m_newLineEndingChars << m_newLineEndingChars;
+			idl << getIndent() << "// " << stype << " " << c->getName()
+			    << " is Not Yet Implemented" << m_endl << m_endl;
 		} else if (stype == "CORBATypedef") {
 			UMLClassifierList superclasses = c->getSuperClasses();
 			UMLClassifier* firstParent = superclasses.first();
-			idl << spc() << "typedef " << firstParent->getName() << " "
-			    << c->getName() << ";" << m_newLineEndingChars << m_newLineEndingChars;
+			idl << getIndent() << "typedef " << firstParent->getName() << " "
+			    << c->getName() << ";" << m_endl << m_endl;
 		} else {
-			idl << spc() << "// " << stype << ": Unknown stereotype" << m_newLineEndingChars << m_newLineEndingChars;
+			idl << getIndent() << "// " << stype << ": Unknown stereotype" << m_endl << m_endl;
 		}
 		// Close the modules inside which we might be nested.
 		for (pkg = pkgList.first(); pkg != NULL; pkg = pkgList.next()) {
-			indentlevel--;
-			idl << spc() << "};" << m_newLineEndingChars << m_newLineEndingChars;
+			m_indentLevel--;
+			idl << getIndent() << "};" << m_endl << m_endl;
 		}
 		return;
 	}
 
-	idl << spc();
+	idl << getIndent();
 	if (c->getAbstract())
 		idl << "abstract ";
-	idl << "interface " << c->getName();
+	bool isValuetype = (c->getStereotype(false) == "CORBAValue");
+	if (isValuetype)
+		idl << "valuetype ";
+	else
+		idl << "interface ";
+	idl << c->getName();
 	UMLClassifierList superclasses = c->getSuperClasses();
 	if (! superclasses.isEmpty()) {
 		idl << " : ";
@@ -308,66 +312,63 @@ void IDLWriter::writeClass(UMLClassifier *c) {
 			idl << qualifiedName(parent);
 			if (n_parents)
 				idl << ", ";
-	        	parent = superclasses.next();
+			parent = superclasses.next();
 		}
 	}
-	idl << " {" << m_newLineEndingChars << m_newLineEndingChars;
-	indentlevel++;
+	idl << " {" << m_endl << m_endl;
+	m_indentLevel++;
 
 	// Generate auxiliary declarations for multiplicity of associations
 	UMLAssociation *a;
 	bool didComment = false;
-	UMLAssociationList aggregations = c->getAggregations();
-	for (a = aggregations.first(); a; a = aggregations.next()) {
-		QString multiplicity = a->getMulti(A);
+	UMLAssociationList assocs = c->getAssociations();
+	for (a = assocs.first(); a; a = assocs.next()) {
+		if (! assocTypeIsMappableToAttribute(a->getAssocType()))
+			continue;
+		QString multiplicity = a->getMulti(Uml::A);
 		if (multiplicity.isEmpty() || multiplicity == "1")
 			continue;
 		if (!didComment) {
-			idl << spc() << "// Types for association multiplicities" << m_newLineEndingChars << m_newLineEndingChars;
+			idl << getIndent() << "// Types for association multiplicities" << m_endl << m_endl;
 			didComment = true;
 		}
-		UMLClassifier* other = (UMLClassifier*)m_doc->findUMLObject(a->getRoleId(A));
+		UMLClassifier* other = (UMLClassifier*)m_doc->findObjectById(a->getRoleId(Uml::A));
 		QString bareName = cleanName(other->getName());
-		idl << spc() << "typedef sequence<" << qualifiedName(other) << "> "
-		    << bareName << "Vector;" << m_newLineEndingChars << m_newLineEndingChars;
-	}
-
-	UMLAssociationList compositions = c->getCompositions();
-	for (a = compositions.first(); a; a = compositions.next()) {
-		QString multiplicity = a->getMulti(A);
-		if (multiplicity.isEmpty() || multiplicity == "1")
-			continue;
-		if (!didComment) {
-			idl << spc() << "// Types for association multiplicities" << m_newLineEndingChars << m_newLineEndingChars;
-			didComment = true;
-		}
-		UMLClassifier* other = (UMLClassifier*)m_doc->findUMLObject(a->getRoleId(A));
-		QString bareName = cleanName(other->getName());
-		idl << spc() << "typedef sequence<" << qualifiedName(other) << "> "
-		    << bareName << "Vector;" << m_newLineEndingChars << m_newLineEndingChars;
+		idl << getIndent() << "typedef sequence<" << qualifiedName(other) << "> "
+		    << bareName << "Vector;" << m_endl << m_endl;
 	}
 
 	// Generate public attributes.
 	if(myClass) {
-		UMLAttributeList *atl = myClass->getFilteredAttributeList();
-		UMLAttributeList atpub;
-		UMLAttribute *at;
-		for (at = atl->first(); at; at = atl->next()) {
-			if (at->getScope() == Uml::Public)
-				atpub.append(at);
-		}
-		if (forceSections() || atpub.count()) {
-			idl << spc() << "// Public attributes:" << m_newLineEndingChars << m_newLineEndingChars;
-			for (at = atpub.first(); at; at = atpub.next()) {
+		UMLAttributeList atl = myClass->getAttributeList();
+		if (forceSections() || atl.count()) {
+			idl << getIndent() << "// Attributes:" << m_endl << m_endl;
+			for (UMLAttribute *at = atl.first(); at; at = atl.next()) {
 				QString attName = cleanName(at->getName());
-				idl << spc() << "attribute " << at->getTypeName()
-				    << " " << attName << ";" << m_newLineEndingChars << m_newLineEndingChars;
+				Uml::Scope scope = at->getScope();
+				idl << getIndent();
+				if (isValuetype) {
+					if (scope == Uml::Public)
+						idl << "public ";
+					else
+						idl << "private ";
+				} else {
+					if (scope != Uml::Public) {
+						idl << "// visibility should be: "
+						    << Umbrello::scopeToString(scope, false)
+						    << m_endl;
+						idl << getIndent();
+					}
+					idl << "attribute ";
+				}
+				idl << at->getTypeName() << " " << attName << ";"
+				    << m_endl << m_endl;
 			}
 		}
 	}
 
 	// Generate public operations.
-	UMLOperationList opl(c->getFilteredOperationsList());
+	UMLOperationList opl(c->getOpList());
 	UMLOperationList oppub;
 	UMLOperation *op;
 	for (op = opl.first(); op; op = opl.next()) {
@@ -375,39 +376,41 @@ void IDLWriter::writeClass(UMLClassifier *c) {
 			oppub.append(op);
 	}
 	if (forceSections() || oppub.count()) {
-		idl << spc() << "// Public methods:" << m_newLineEndingChars << m_newLineEndingChars;
+		idl << getIndent() << "// Public methods:" << m_endl << m_endl;
 		for (op = oppub.first(); op; op = oppub.next())
 			writeOperation(op, idl);
-		idl << m_newLineEndingChars;
+		idl << m_endl;
 	}
 
 
-	if (forceSections() || !aggregations.isEmpty()) {
-		idl << spc() << "// Aggregations:" << m_newLineEndingChars;
-		for (UMLAssociation *a = aggregations.first(); a; a = aggregations.next()) {
+	if (forceSections() || !assocs.isEmpty()) {
+		idl << getIndent() << "// Associations:" << m_endl << m_endl;
+		for (a = assocs.first(); a; a = assocs.next()) {
+			Uml::Association_Type at = a->getAssocType();
+			if (! assocTypeIsMappableToAttribute(at))
+				continue;
 			QString typeName, roleName;
 			computeAssocTypeAndRole(a, c, typeName, roleName);
-			idl << spc() << "// " << typeName << " " << roleName << ";" << m_newLineEndingChars;
+			if (roleName.isEmpty())  // presumably because we are at the "wrong" end
+				continue;
+			idl << getIndent() << "// " << UMLAssociation::typeAsString(at) << m_endl;
+			idl << getIndent();
+			if (isValuetype)
+				idl << "public ";
+			else
+				idl << "attribute ";
+			idl << typeName << " " << roleName << ";" << m_endl;
 		}
-		idl << endl;
-	}
-	if (forceSections() || !compositions.isEmpty()) {
-		idl << spc() << "// Compositions:" << m_newLineEndingChars;
-		for (UMLAssociation *a = compositions.first(); a; a = compositions.next()) {
-			QString typeName, roleName;
-			computeAssocTypeAndRole(a, c, typeName, roleName);
-			idl << spc() << "// " << typeName << " " << roleName << ";" << m_newLineEndingChars;
-		}
-		idl << endl;
+		idl << m_endl;
 	}
 
-	indentlevel--;
-	idl << spc() << "};" << m_newLineEndingChars << m_newLineEndingChars;
+	m_indentLevel--;
+	idl << getIndent() << "};" << m_endl << m_endl;
 
 	// Close the modules inside which we might be nested.
 	for (pkg = pkgList.first(); pkg != NULL; pkg = pkgList.next()) {
-		indentlevel--;
-		idl << spc() << "};" << m_newLineEndingChars << m_newLineEndingChars;
+		m_indentLevel--;
+		idl << getIndent() << "};" << m_endl << m_endl;
 	}
 	file.close();
 	emit codeGenerated(c, true);
@@ -416,34 +419,34 @@ void IDLWriter::writeClass(UMLClassifier *c) {
 
 void IDLWriter::writeOperation(UMLOperation *op, QTextStream &idl, bool is_comment) {
 	UMLAttributeList *atl = op->getParmList();
-	QString rettype = op->getReturnType();
+	QString rettype = op->getTypeName();
 
-	idl << spc();
+	idl << getIndent();
 	if (is_comment)
 		idl << "// ";
 	idl << rettype << " " << cleanName(op->getName()) << " (";
 	if (atl->count()) {
-		idl << m_newLineEndingChars;
-		indentlevel++;
+		idl << m_endl;
+		m_indentLevel++;
 		uint i = 0;
 		for (UMLAttribute *at = atl->first(); at; at = atl->next()) {
-			idl << spc();
+			idl << getIndent();
 			if (is_comment)
 				idl << "// ";
-			Uml::Parameter_Kind pk = at->getParmKind();
-			if (pk == Uml::pk_Out)
+			Uml::Parameter_Direction pk = at->getParmKind();
+			if (pk == Uml::pd_Out)
 				idl << "out ";
-			else if (pk == Uml::pk_InOut)
+			else if (pk == Uml::pd_InOut)
 				idl << "inout ";
 			else
 				idl << "in ";
 			idl << at->getTypeName() << " " << cleanName(at->getName());
 			if (++i < atl->count())
-				idl << "," << m_newLineEndingChars;
+				idl << "," << m_endl;
 		}
-		indentlevel--;
+		m_indentLevel--;
 	}
-	idl << ");" << m_newLineEndingChars << m_newLineEndingChars;
+	idl << ");" << m_endl << m_endl;
 }
 
 void IDLWriter::createDefaultDatatypes() {
@@ -459,55 +462,47 @@ void IDLWriter::createDefaultDatatypes() {
 	m_doc->createDatatype("string");
 }
 
-/**
- * List of reserved keywords for this code generator.
- *
- * Just add new keywords, then mark all lines and
- * pipe it through the external 'sort' program.
- */
-static const char *ReservedWords[] = {
-  "any",
-  "attribute",
-  "boolean",
-  "case",
-  "char",
-  "const",
-  "context",
-  "default",
-  "double",
-  "enum",
-  "exception",
-  "FALSE",
-  "float",
-  "in",
-  "inout",
-  "interface",
-  "long",
-  "module",
-  "octet",
-  "oneway",
-  "out",
-  "raises",
-  "readonly",
-  "sequence",
-  "short",
-  "string",
-  "struct",
-  "switch",
-  "TRUE",
-  "typedef",
-  "union",
-  "unsigned",
-  "void",
-  NULL
-};
+const QStringList IDLWriter::reservedKeywords() const {
 
-/**
- * get list of reserved keywords
- */
-const char **
-IDLWriter::getReservedKeywords() {
-  return ReservedWords;
+  static QStringList keywords;
+
+  if (keywords.isEmpty()) {
+    keywords << "any"
+	     << "attribute"
+	     << "boolean"
+	     << "case"
+	     << "char"
+	     << "const"
+	     << "context"
+	     << "default"
+	     << "double"
+	     << "enum"
+	     << "exception"
+	     << "FALSE"
+	     << "float"
+	     << "in"
+	     << "inout"
+	     << "interface"
+	     << "long"
+	     << "module"
+	     << "octet"
+	     << "oneway"
+	     << "out"
+	     << "raises"
+	     << "readonly"
+	     << "sequence"
+	     << "short"
+	     << "string"
+	     << "struct"
+	     << "switch"
+	     << "TRUE"
+	     << "typedef"
+	     << "union"
+	     << "unsigned"
+	     << "void";
+  }
+
+  return keywords;
 }
 
 
