@@ -42,24 +42,24 @@ public:
 	 */
 	virtual void writeClass (UMLClassifier *c);
 
-  /**
-   * checks whether type is "IDLWriter"
-   *
-   * @param type
-   */
+	/**
+	 * checks whether type is "IDLWriter"
+	 *
+	 * @param type
+	 */
 	virtual bool isType (QString & type);
 
-  /**
-   * returns "IDL"
-   */
-  virtual QString getLanguage();
+	/**
+	 * returns "IDL"
+	 */
+	virtual QString getLanguage();
 
 	void createDefaultDatatypes();
 
 	/**
 	 * get list of reserved keywords
 	 */
-	virtual const char **getReservedKeywords();
+	virtual const QStringList reservedKeywords() const;
 
 private:
 
@@ -73,13 +73,12 @@ private:
 	void computeAssocTypeAndRole(UMLAssociation* a, UMLClassifier *c,
 				     QString& typeName, QString& roleName);
 
-	bool isOOClass(UMLClassifier* c);
+	static bool isOOClass(UMLClassifier* c);
+
+	static bool assocTypeIsMappableToAttribute(Uml::Association_Type at);
 
 	QString qualifiedName(UMLClassifier* c);
 
-	QString spc();
-
-	int indentlevel;
 };
 
 #endif // IDLWRITER_H
