@@ -1,7 +1,11 @@
 /***************************************************************************
                           javawriter.cpp  -  description
+    This is the "old" code generator that does not support code editing
+    in the Modeller but uses significantly less file space because the
+    source code is not replicated in the XMI file.
                              -------------------
     copyright            : (C) 2003 Brian Thomas brian.thomas@gsfc.nasa.gov
+         (C) 2004  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
 ***************************************************************************/
 
 /***************************************************************************
@@ -13,29 +17,39 @@
  *                                                                         *
  ***************************************************************************/
 
+// own header
 #include "javawriter.h"
-
-#include <kdebug.h>
-
-#include <klocale.h>
-#include <kmessagebox.h>
+// qt includes
 #include <qfile.h>
 #include <qtextstream.h>
 #include <qregexp.h>
-
+// kde includes
+#include <kdebug.h>
+// app includes
 #include "../umldoc.h"
 #include "../class.h"
 #include "../interface.h"
 #include "../operation.h"
-#include "../umlnamespace.h"
+#include "../attribute.h"
+#include "../association.h"
 
-JavaWriter::JavaWriter( QObject *parent, const char *name ) : CodeGenerator(parent, name) {
-	indent = "\t";
-	startline = "\n"+indent; // using UNIX newLine standard.. bad
+JavaWriter::JavaWriter( UMLDoc *parent, const char *name ) : SimpleCodeGenerator(parent, name) {
+	startline = m_endl + m_indentation;
 }
 
 JavaWriter::~JavaWriter() {}
 
+QString JavaWriter::getLanguage() {
+	return "Java";
+}
+
+bool JavaWriter::isType (QString & type)
+{
+	if (type == "JavaCodeGenerator")
+		return true;
+	return false;
+}
+
 void JavaWriter::writeClass(UMLClassifier *c)
 {
 
@@ -69,24 +83,23 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	//
 
 	// sort attributes by Scope
-        UMLAttribute *at;
-        UMLAttributeList *atl;
-        UMLAttributeList  atpub, atprot, atpriv;
-        UMLAttributeList  final_atpub, final_atprot, final_atpriv;
-        atpub.setAutoDelete(false);
-        final_atpub.setAutoDelete(false);
-        atprot.setAutoDelete(false);
-        final_atprot.setAutoDelete(false);
-        atpriv.setAutoDelete(false);
-        final_atpriv.setAutoDelete(false);
+	UMLAttributeList  atl;
+	UMLAttributeList  atpub, atprot, atpriv;
+	UMLAttributeList  final_atpub, final_atprot, final_atpriv;
+	atpub.setAutoDelete(false);
+	final_atpub.setAutoDelete(false);
+	atprot.setAutoDelete(false);
+	final_atprot.setAutoDelete(false);
+	atpriv.setAutoDelete(false);
+	final_atpriv.setAutoDelete(false);
 
 	UMLClass * myClass = dynamic_cast<UMLClass *>(c);
 
 	if(myClass) {
-	        atl = myClass->getFilteredAttributeList();
-	        for(at=atl->first(); at ; at=atl->next()) {
-	                switch(at->getScope())
-	                {
+		UMLAttributeList atl = myClass->getAttributeList();
+		for (UMLAttribute *at = atl.first(); at ; at = atl.next()) {
+			switch(at->getScope())
+			{
 			case Uml::Public:
 				if(at->getStatic())
 					final_atpub.append(at);
@@ -105,8 +118,8 @@ void JavaWriter::writeClass(UMLClassifier *c)
 				else
 					atpriv.append(at);
 				break;
-	                }
-	        }
+			}
+		}
 	}
 
 	// another preparation, determine what we have
@@ -115,12 +128,9 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	UMLAssociationList compositions = c->getCompositions();
 
 	bool hasAssociations = aggregations.count() > 0 || associations.count() > 0 || compositions.count() > 0;
-	bool hasAttributes = atpub.count() > 0 || atprot.count() > 0 || atpriv.count() > 0
-			     || final_atpub.count() > 0
-			     || final_atprot.count() > 0
-			     || final_atpriv.count() > 0;
+	bool hasAttributes = (atl.count() > 0);
 	bool hasAccessorMethods = hasAttributes || hasAssociations;
-	bool hasOperationMethods = c->getFilteredOperationsList()->last() ? true : false;
+	bool hasOperationMethods = (c->getOpList().count() > 0);
 	// this is a bit too simplistic..some associations are for
 	// SINGLE objects, and WONT be declared as Vectors, so this
 	// is a bit overly inclusive
@@ -135,11 +145,11 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	if(!str.isEmpty()) {
 		str.replace(QRegExp("%filename%"),fileName);
 		str.replace(QRegExp("%filepath%"),file.name());
-		java<<str<<endl;
+		java<<str<<m_endl;
 	}
 
 	if(!c->getPackage().isEmpty())
-		java<<"package "<<c->getPackage()<<";"<<endl;
+		java<<"package "<<c->getPackage()<<";"<<m_endl;
 
 	// IMPORT statements
 	// Q: Why all utils? Isnt just List and Vector the only classes we are using?
@@ -148,15 +158,20 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	if (hasVectorFields )
 	{
 		writeBlankLine(java);
-		java<<"import java.util.*;"<<endl;
+		java<<"import java.util.*;"<<m_endl;
 	}
 
 	//only import classes in a different package as this class
 	UMLClassifierList imports;
 	findObjectsRelated(c,imports);
-	for(UMLClassifier *con = imports.first(); con ; con = imports.next())
-		if(con->getPackage() != c->getPackage())
-			java<<"import "<<con->getPackage()<<"."<<cleanName(con->getName())<<";"<<endl;
+	for (UMLClassifier *con = imports.first(); con; con = imports.next()) {
+		if (con->getBaseType() == Uml::ot_Datatype)
+			continue;
+		QString pkg = con->getPackage();
+		if (!pkg.isEmpty() && pkg != c->getPackage())
+			java << "import " << pkg << "." << cleanName(con->getName()) << ";"
+			     << m_endl;
+	}
 	writeBlankLine(java);
 
 	// write the opening declaration for the class incl any documentation,
@@ -164,7 +179,7 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	writeClassDecl(c, java);
 
 	// start body of class
-	java<<" {"<<endl;
+	java<<" {"<<m_endl;
 
 	// ATTRIBUTES
 	//
@@ -172,9 +187,9 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	// write comment for section IF needed
 	if (forceDoc() || hasAccessorMethods)
 	{
-		writeComment("", indent, java);
-		writeComment("Fields", indent, java);
-		writeComment("", indent, java);
+		writeComment("", m_indentation, java);
+		writeComment("Fields", m_indentation, java);
+		writeComment("", m_indentation, java);
 		writeBlankLine(java);
 	}
 
@@ -198,9 +213,9 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	{
 
 		java<<startline;
-		writeComment("", indent, java);
-		writeComment("Methods", indent, java);
-		writeComment("", indent, java);
+		writeComment("", m_indentation, java);
+		writeComment("Methods", m_indentation, java);
+		writeComment("", m_indentation, java);
 		writeBlankLine(java);
 		writeBlankLine(java);
 	}
@@ -208,19 +223,19 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	// write comment for sub-section IF needed
 	if (forceDoc() || hasAccessorMethods )
 	{
-		writeComment("", indent, java);
-		writeComment("Accessor methods", indent, java);
-		writeComment("", indent, java);
+		writeComment("", m_indentation, java);
+		writeComment("Accessor methods", m_indentation, java);
+		writeComment("", m_indentation, java);
 		writeBlankLine(java);
 	}
 
 	// Accessors for attributes
-	writeAttributeMethods(final_atpub, Public, java);
-	writeAttributeMethods(final_atprot, Protected, java);
-	writeAttributeMethods(final_atpriv, Private, java);
-	writeAttributeMethods(atpub, Public, java);
-	writeAttributeMethods(atprot, Protected, java);
-	writeAttributeMethods(atpriv, Private, java);
+	writeAttributeMethods(final_atpub, Uml::Public, java);
+	writeAttributeMethods(final_atprot, Uml::Protected, java);
+	writeAttributeMethods(final_atpriv, Uml::Private, java);
+	writeAttributeMethods(atpub, Uml::Public, java);
+	writeAttributeMethods(atprot, Uml::Protected, java);
+	writeAttributeMethods(atpriv, Uml::Private, java);
 
 	// accessor methods for associations
 
@@ -235,15 +250,15 @@ void JavaWriter::writeClass(UMLClassifier *c)
 	// write comment for sub-section IF needed
 	if (forceDoc() || hasOperationMethods)
 	{
-		writeComment("", indent, java);
-		writeComment("Other methods", indent, java);
-		writeComment("", indent, java);
+		writeComment("", m_indentation, java);
+		writeComment("Other methods", m_indentation, java);
+		writeComment("", m_indentation, java);
 		writeBlankLine(java);
 	}
 	writeOperations(c,java);
 
 	writeBlankLine(java);
-	java<<"}"<<endl; // end class
+	java<<"}"<<m_endl; // end class
 
 	file.close();
 	emit codeGenerated(c, true);
@@ -255,7 +270,7 @@ void JavaWriter::writeClassDecl(UMLClassifier *c, QTextStream &java)
 	QString classname = cleanName(c->getName()); // our class name
 
 	// write documentation for class, if any, first
-        if(forceDoc() || !c->getDoc().isEmpty())
+	if(forceDoc() || !c->getDoc().isEmpty())
 	{
 		if(isInterface)
 			writeDocumentation("Interface "+classname,c->getDoc(),"","",java);
@@ -315,7 +330,7 @@ void JavaWriter::writeAttributeDecls(UMLAttributeList &atpub, UMLAttributeList &
 		QString typeName = fixTypeName(at->getTypeName());
 		QString initialValue = fixInitialStringDeclValue(at->getInitialValue(), typeName);
 		if(!documentation.isEmpty())
-			writeComment(documentation, indent, java, true);
+			writeComment(documentation, m_indentation, java, true);
 		java<<startline<<staticValue<<"public "<<typeName<<" "<<cleanName(at->getName())
 		    <<(initialValue.isEmpty()?QString(""):QString(" = ") + initialValue)<<";";
 	}
@@ -327,7 +342,7 @@ void JavaWriter::writeAttributeDecls(UMLAttributeList &atpub, UMLAttributeList &
 		QString staticValue = at->getStatic() ? "static " : "";
 		QString initialValue = fixInitialStringDeclValue(at->getInitialValue(), typeName);
 		if(!documentation.isEmpty())
-			writeComment(documentation, indent, java, true);
+			writeComment(documentation, m_indentation, java, true);
 		java<<startline<<staticValue<<"protected "<<typeName<<" "<<cleanName(at->getName())
 		    <<(initialValue.isEmpty()?QString(""):QString(" = ") + initialValue)<<";";
 	}
@@ -339,14 +354,14 @@ void JavaWriter::writeAttributeDecls(UMLAttributeList &atpub, UMLAttributeList &
 		QString staticValue = at->getStatic() ? "static " : "";
 		QString initialValue = fixInitialStringDeclValue(at->getInitialValue(), typeName);
 		if(!documentation.isEmpty())
-			writeComment(documentation, indent, java, true);
+			writeComment(documentation, m_indentation, java, true);
 		java<<startline<<staticValue<<"private "<<typeName<<" "<<cleanName(at->getName())
 		    <<(initialValue.isEmpty()?QString(""):QString(" = ") + initialValue)<<";";
 	}
 
 }
 
-void JavaWriter::writeAttributeMethods(UMLAttributeList &atpub, Scope visibility, QTextStream &java)
+void JavaWriter::writeAttributeMethods(UMLAttributeList &atpub, Uml::Scope visibility, QTextStream &java)
 {
 
 	UMLAttribute *at;
@@ -363,7 +378,7 @@ void JavaWriter::writeAttributeMethods(UMLAttributeList &atpub, Scope visibility
 						    cleanName(at->getName()),
 						    fieldName,
 						    at->getDoc(),
-						    visibility, chg_Changeable, at->getStatic(), java);
+						    visibility, Uml::chg_Changeable, at->getStatic(), java);
 	}
 
 }
@@ -376,7 +391,7 @@ void JavaWriter::writeComment(QString comment, QString myIndent, QTextStream &ja
 	if (comment.contains(QRegExp("\n"))) {
 
 		if(javaDocStyle)
-			java << myIndent << "/**" << endl;
+			java << myIndent << "/**" << m_endl;
 		QStringList lines = QStringList::split( "\n", comment);
 		for(uint i= 0; i < lines.count(); i++)
 		{
@@ -388,26 +403,26 @@ void JavaWriter::writeComment(QString comment, QString myIndent, QTextStream &ja
 			java << lines[i];
 		}
 		if(javaDocStyle)
-			java << myIndent << " */" << endl;
+			java << myIndent << " */" << m_endl;
 	} else {
 		// this should be more fancy in the future, breaking it up into 80 char
 		// lines so that it doesnt look too bad
 		writeBlankLine(java);
 		if(javaDocStyle)
-			java << myIndent << "/**" << endl << myIndent << " *";
+			java << myIndent << "/**" << m_endl << myIndent << " *";
 		else
 		    java<<myIndent<<"//";
 		if(comment.length() > 0)
 			java << " " << comment;
 		if(javaDocStyle)
-			java << endl << myIndent << " */";
+			java << m_endl << myIndent << " */";
 	}
 }
 
 void JavaWriter::writeDocumentation(QString header, QString body, QString end, QString indent, QTextStream &java)
 {
 	writeBlankLine(java);
-	java<<indent<<"/**"<<endl;
+	java<<indent<<"/**"<<m_endl;
 	if (!header.isEmpty())
 		java<<formatDoc(header, indent+" * ");
 	if (!body.isEmpty())
@@ -421,7 +436,7 @@ void JavaWriter::writeDocumentation(QString header, QString body, QString end, Q
 	java<<indent<<" */";
 }
 
-void JavaWriter::writeAssociationDecls(UMLAssociationList associations, int id, QTextStream &java)
+void JavaWriter::writeAssociationDecls(UMLAssociationList associations, Uml::IDType id, QTextStream &java)
 {
 
 	if( forceSections() || !associations.isEmpty() )
@@ -431,28 +446,28 @@ void JavaWriter::writeAssociationDecls(UMLAssociationList associations, int id,
 		{
 			// it may seem counter intuitive, but you want to insert the role of the
 			// *other* class into *this* class.
-			if (a->getRoleAId() == id)
+			if (a->getRoleId(Uml::A) == id)
 				printRoleB = true;
 
-			if (a->getRoleBId() == id)
+			if (a->getRoleId(Uml::B) == id)
 				printRoleA = true;
 
 			// First: we insert documentaion for association IF it has either role AND some documentation (!)
 			if ((printRoleA || printRoleB) && !(a->getDoc().isEmpty()))
-				writeComment(a->getDoc(), indent, java);
+				writeComment(a->getDoc(), m_indentation, java);
 
 			// print RoleB decl
 			if (printRoleB)
 			{
-				QString fieldClassName = cleanName(getUMLObjectName(a->getObjectB()));
-				writeAssociationRoleDecl(fieldClassName, a->getRoleNameB(), a->getMultiB(), a->getRoleBDoc(), a->getVisibilityB(), java);
+				QString fieldClassName = cleanName(getUMLObjectName(a->getObject(Uml::B)));
+				writeAssociationRoleDecl(fieldClassName, a->getRoleName(Uml::B), a->getMulti(Uml::B), a->getRoleDoc(Uml::B), a->getVisibility(Uml::B), java);
 			}
 
 			// print RoleA decl
 			if (printRoleA)
 			{
-				QString fieldClassName = cleanName(getUMLObjectName(a->getObjectA()));
-				writeAssociationRoleDecl(fieldClassName, a->getRoleNameA(), a->getMultiA(), a->getRoleADoc(), a->getVisibilityA(), java);
+				QString fieldClassName = cleanName(getUMLObjectName(a->getObject(Uml::A)));
+				writeAssociationRoleDecl(fieldClassName, a->getRoleName(Uml::A), a->getMulti(Uml::A), a->getRoleDoc(Uml::A), a->getVisibility(Uml::A), java);
 			}
 		}
 	}
@@ -460,7 +475,7 @@ void JavaWriter::writeAssociationDecls(UMLAssociationList associations, int id,
 
 void JavaWriter::writeAssociationRoleDecl(QString fieldClassName,
 					QString roleName, QString multi,
-					QString doc, Scope visib, QTextStream &java)
+					QString doc, Uml::Scope visib, QTextStream &java)
 {
 	// ONLY write out IF there is a rolename given
 	// otherwise its not meant to be declared in the code
@@ -473,7 +488,7 @@ void JavaWriter::writeAssociationRoleDecl(QString fieldClassName,
 	writeBlankLine(java);
 
 	if (!doc.isEmpty())
-		writeComment(doc, indent, java);
+		writeComment(doc, m_indentation, java);
 
 	// declare the association based on whether it is this a single variable
 	// or a List (Vector). One day this will be done correctly with special
@@ -502,29 +517,29 @@ void JavaWriter::writeAssociationMethods (UMLAssociationList associations, UMLCl
 
 			// insert the methods to access the role of the other
 			// class in the code of this one
-			if (a->getRoleAId() == thisClass->getID())
+			if (a->getRoleId(Uml::A) == thisClass->getID())
 			{
 				// only write out IF there is a rolename given
-				if(!a->getRoleNameB().isEmpty()) {
-					QString fieldClassName = getUMLObjectName(a->getObjectB());
+				if(!a->getRoleName(Uml::B).isEmpty()) {
+					QString fieldClassName = getUMLObjectName(a->getObject(Uml::B));
 					writeAssociationRoleMethod(fieldClassName,
-								   a->getRoleNameB(),
-								   a->getMultiB(), a->getRoleBDoc(),
-								   a->getVisibilityB(),
-								   a->getChangeabilityB(), java);
+								   a->getRoleName(Uml::B),
+								   a->getMulti(Uml::B), a->getRoleDoc(Uml::B),
+								   a->getVisibility(Uml::B),
+								   a->getChangeability(Uml::B), java);
 				}
 			}
 
-			if (a->getRoleBId() == thisClass->getID())
+			if (a->getRoleId(Uml::B) == thisClass->getID())
 			{
 				// only write out IF there is a rolename given
-				if(!a->getRoleNameA().isEmpty()) {
-					QString fieldClassName = getUMLObjectName(a->getObjectA());
-					writeAssociationRoleMethod(fieldClassName, a->getRoleNameA(),
-								   a->getMultiA(),
-								   a->getRoleADoc(),
-								   a->getVisibilityA(),
-								   a->getChangeabilityA(),
+				if(!a->getRoleName(Uml::A).isEmpty()) {
+					QString fieldClassName = getUMLObjectName(a->getObject(Uml::A));
+					writeAssociationRoleMethod(fieldClassName, a->getRoleName(Uml::A),
+								   a->getMulti(Uml::A),
+								   a->getRoleDoc(Uml::A),
+								   a->getVisibility(Uml::A),
+								   a->getChangeability(Uml::A),
 								   java);
 				}
 			}
@@ -534,7 +549,7 @@ void JavaWriter::writeAssociationMethods (UMLAssociationList associations, UMLCl
 }
 
 void JavaWriter::writeAssociationRoleMethod (QString fieldClassName, QString roleName, QString multi,
-					     QString description, Scope visib, Changeability_Type change,
+					     QString description, Uml::Scope visib, Uml::Changeability_Type change,
 					     QTextStream &java)
 {
 	if(multi.isEmpty() || multi.contains(QRegExp("^[01]$")))
@@ -553,7 +568,7 @@ void JavaWriter::writeAssociationRoleMethod (QString fieldClassName, QString rol
 
 void JavaWriter::writeVectorAttributeAccessorMethods (QString fieldClassName, QString fieldVarName,
 						      QString fieldName, QString description,
-						      Scope visibility, Changeability_Type changeType,
+						      Uml::Scope visibility, Uml::Changeability_Type changeType,
 						      QTextStream &java)
 {
 
@@ -562,36 +577,36 @@ void JavaWriter::writeVectorAttributeAccessorMethods (QString fieldClassName, QS
 	QString strVis = scopeToJavaDecl(visibility);
 
 	// ONLY IF changeability is NOT Frozen
-	if (changeType != chg_Frozen)
+	if (changeType != Uml::chg_Frozen)
 	{
-		writeDocumentation("Add a "+fieldName+" object to the "+fieldVarName+" List",description,"",indent,java);
+		writeDocumentation("Add a "+fieldName+" object to the "+fieldVarName+" List",description,"",m_indentation,java);
 		java<<startline<<strVis<<" void add"<<fieldName<<" ( "<<fieldClassName<<" new_object ) {";
-		java<<startline<<indent<<fieldVarName<<".add(new_object);";
-		java<<startline<<"}"<<endl;
+		java<<startline<<m_indentation<<fieldVarName<<".add(new_object);";
+		java<<startline<<"}"<<m_endl;
 	}
 
 	// ONLY IF changeability is Changeable
-	if (changeType == chg_Changeable)
+	if (changeType == Uml::chg_Changeable)
 	{
-		writeDocumentation("Remove a "+fieldName+" object from "+fieldVarName+" List",description,"",indent,java);
+		writeDocumentation("Remove a "+fieldName+" object from "+fieldVarName+" List",description,"",m_indentation,java);
 		java<<startline<<strVis<<" void remove"<<fieldName<<" ( "<<fieldClassName<<" new_object )";
 		java<<startline<<"{";
-		java<<startline<<indent<<fieldVarName<<".remove(new_object);";
-		java<<startline<<"}"<<endl;
+		java<<startline<<m_indentation<<fieldVarName<<".remove(new_object);";
+		java<<startline<<"}"<<m_endl;
 	}
 
 	// always allow getting the list of stuff
-	writeDocumentation("Get the List of "+fieldName+" objects held by "+fieldVarName,description,"@return List of "+fieldName+" objects held by "+fieldVarName,indent,java);
+	writeDocumentation("Get the List of "+fieldName+" objects held by "+fieldVarName,description,"@return List of "+fieldName+" objects held by "+fieldVarName,m_indentation,java);
 	java<<startline<<strVis<<" List get"<<fieldName<<"List ( ) {";
-	java<<startline<<indent<<"return (List) "<<fieldVarName<<";";
-	java<<startline<<"}"<<endl;
+	java<<startline<<m_indentation<<"return (List) "<<fieldVarName<<";";
+	java<<startline<<"}"<<m_endl;
 	writeBlankLine(java);
 }
 
 
 void JavaWriter::writeSingleAttributeAccessorMethods(QString fieldClassName, QString fieldVarName,
 						     QString fieldName, QString description,
-						     Scope visibility, Changeability_Type change,
+						     Uml::Scope visibility, Uml::Changeability_Type change,
 						     bool isFinal, QTextStream &java)
 {
 
@@ -600,17 +615,17 @@ void JavaWriter::writeSingleAttributeAccessorMethods(QString fieldClassName, QSt
 	fieldName = capitaliseFirstLetter(fieldName);
 
 	// set method
-	if (change == chg_Changeable && !isFinal) {
-		writeDocumentation("Set the value of "+fieldVarName,description,"@param newVar the new value of "+fieldVarName,indent,java);
+	if (change == Uml::chg_Changeable && !isFinal) {
+		writeDocumentation("Set the value of "+fieldVarName,description,"@param newVar the new value of "+fieldVarName,m_indentation,java);
 		java<<startline<<strVis<<" void set"<<fieldName<<" ( "<<fieldClassName<<" newVar ) {";
-		java<<startline<<indent<<fieldVarName<<" = newVar;";
-		java<<startline<<"}"<<endl;
+		java<<startline<<m_indentation<<fieldVarName<<" = newVar;";
+		java<<startline<<"}"<<m_endl;
 	}
 
 	// get method
-	writeDocumentation("Get the value of "+fieldVarName,description,"@return the value of "+fieldVarName,indent,java);
+	writeDocumentation("Get the value of "+fieldVarName,description,"@return the value of "+fieldVarName,m_indentation,java);
 	java<<startline<<strVis<<" "<<fieldClassName<<" get"<<fieldName<<" ( ) {";
-	java<<startline<<indent<<"return "<<fieldVarName<<";";
+	java<<startline<<m_indentation<<"return "<<fieldVarName<<";";
 	java<<startline<<"}";
 	writeBlankLine(java);
 }
@@ -621,15 +636,15 @@ void JavaWriter::writeConstructor(UMLClassifier *c, QTextStream &java)
 	if (forceDoc())
 	{
 		java<<startline;
-		writeComment("", indent, java);
-		writeComment("Constructors", indent, java);
-		writeComment("", indent, java);
+		writeComment("", m_indentation, java);
+		writeComment("Constructors", m_indentation, java);
+		writeComment("", m_indentation, java);
 		writeBlankLine(java);
 	}
 
 	// write the first constructor
 	QString className = cleanName(c->getName());
-	java<<indent<<"public "<<className<<" () { };";
+	java<<m_indentation<<"public "<<className<<" () { };";
 
 }
 
@@ -638,22 +653,23 @@ void JavaWriter::writeConstructor(UMLClassifier *c, QTextStream &java)
 // Same thing again for "bool" to "boolean"
 QString JavaWriter::fixTypeName(QString string)
 {
+	if (string.isEmpty())
+		return "void";
 	string.replace(QRegExp("^string$"),"String");
 	string.replace(QRegExp("^bool$"),"boolean");
 	return string;
 }
 
 void JavaWriter::writeOperations(UMLClassifier *c, QTextStream &java) {
-	UMLOperationList *opl;
+	UMLOperationList opl;
 	UMLOperationList oppub,opprot,oppriv;
 	oppub.setAutoDelete(false);
 	opprot.setAutoDelete(false);
 	oppriv.setAutoDelete(false);
 
-	UMLOperation *op;
 	//sort operations by scope first and see if there are abstrat methods
-	opl = c->getFilteredOperationsList();
-	for(op = opl->first(); op ; op=opl->next()) {
+	opl = c->getOpList();
+	for (UMLOperation *op = opl.first(); op; op = opl.next()) {
 		switch(op->getScope()) {
 		case Uml::Public:
 			oppub.append(op);
@@ -671,7 +687,7 @@ void JavaWriter::writeOperations(UMLClassifier *c, QTextStream &java) {
 	/*
 	  if(forceSections() || oppub.count())
 	  {
-	  writeComment("public operations",indent,java);
+	  writeComment("public operations",m_indentation,java);
 		writeBlankLine(java);
 	  }
 	*/
@@ -680,7 +696,7 @@ void JavaWriter::writeOperations(UMLClassifier *c, QTextStream &java) {
 	/*
 	  if(forceSections() || opprot.count())
 	  {
-	  writeComment("protected operations",indent,java);
+	  writeComment("protected operations",m_indentation,java);
 		writeBlankLine(java);
 	  }
 	*/
@@ -689,7 +705,7 @@ void JavaWriter::writeOperations(UMLClassifier *c, QTextStream &java) {
 	/*
 	  if(forceSections() || oppriv.count())
 	  {
-	  writeComment("private operations",indent,java);
+	  writeComment("private operations",m_indentation,java);
 		writeBlankLine(java);
 	  }
 	*/
@@ -711,7 +727,7 @@ void JavaWriter::writeOperations(UMLOperationList &oplist, QTextStream &java) {
 		QString returnStr = "";
 		// write documentation
 
-		QString methodReturnType = fixTypeName(op->getReturnType());
+		QString methodReturnType = fixTypeName(op->getTypeName());
 		if(methodReturnType != "void")
 			returnStr += "@return	"+methodReturnType+"\n";
 
@@ -740,10 +756,10 @@ void JavaWriter::writeOperations(UMLOperationList &oplist, QTextStream &java) {
 		if (op->getAbstract() || isInterface)
 			str+=";\n\n"; // terminate now
 		else
-			str+=startline+"{\n\n"+indent+"}\n\n"; // empty method body
+			str+=startline+"{\n\n"+m_indentation+"}\n\n"; // empty method body
 
 		// write it out
-		writeDocumentation("", op->getDoc(), returnStr, indent, java);
+		writeDocumentation("", op->getDoc(), returnStr, m_indentation, java);
 		java<<startline<<str;
 	}
 }
@@ -796,6 +812,6 @@ QString JavaWriter::capitaliseFirstLetter(QString string)
 
 void JavaWriter::writeBlankLine(QTextStream &java)
 {
-	java<<endl;
+	java<<m_endl;
 }
 
