@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -30,7 +35,7 @@ DatatypeWidget::~DatatypeWidget() {}
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 void DatatypeWidget::init() {
-	UMLWidget::setBaseType(wt_Datatype);
+	UMLWidget::setBaseType(Uml::wt_Datatype);
 	m_pMenu = 0;
 }
 
@@ -48,7 +53,7 @@ void DatatypeWidget::draw(QPainter& p, int offsetX, int offsetY) {
 
 	QFontMetrics &fm = getFontMetrics(FT_NORMAL);
 	int fontHeight  = fm.lineSpacing();
-	QString name = this->getName();
+	QString name = getName();
 
 	p.drawRect(offsetX, offsetY, w, h);
 	p.setPen(QPen(black));
@@ -58,7 +63,7 @@ void DatatypeWidget::draw(QPainter& p, int offsetX, int offsetY) {
 	p.setFont(font);
 	p.drawText(offsetX + DATATYPE_MARGIN, offsetY,
 		   w - DATATYPE_MARGIN* 2,fontHeight,
-		   AlignCenter, "" + m_pObject->getStereotype() + "");
+		   AlignCenter, m_pObject->getStereotype());
 
 	font.setItalic( m_pObject->getAbstract() );
 	p.setFont(font);
@@ -88,7 +93,7 @@ void DatatypeWidget::calculateSize() {
 	//set width to name to start with
 	//set width to name to start with
 	width = getFontMetrics(FT_BOLD_ITALIC).boundingRect(m_pObject->getPackage() + "::" + getName()).width();
-	int w = getFontMetrics(FT_BOLD).boundingRect("" + m_pObject->getStereotype() + "").width();
+	int w = getFontMetrics(FT_BOLD).boundingRect(m_pObject->getStereotype()).width();
 
 	width = w > width?w:width;
 
