@@ -43,7 +43,7 @@ public:
 	 *      @param doc The UMLDoc document
 	 *      @param type The type of listItem this handles
 	 */
-	ClassifierListPage(QWidget* parent, UMLClassifier* classifier, UMLDoc* doc, UMLObject_Type type);
+	ClassifierListPage(QWidget* parent, UMLClassifier* classifier, UMLDoc* doc, Uml::Object_Type type);
 
 	/**
 	 *	Standard deconstructor.
@@ -69,13 +69,42 @@ private:
 	 */
 	void saveCurrentItemDocumentation();
 
+	/**
+	 * Get classifiers
+	 */
+	UMLClassifierListItemList getItemList();
+	
+	/**
+	 * Attempts to add classifier to the appropriate list
+	 *
+	 * @param classifier	Pointer to the classifier to add.
+	 * @param position	Index at which to insert into the list.
+	 * @return true if the classifier could be added
+	 *
+	 */
+	bool addClassifier(UMLClassifierListItem* classifier, int position = -1);
+	
+	/**
+	 * Remove classifier
+	 * @param classifier Classifier to be removed.
+	 * @returns number of remaining classifiers if found, and -1 if not.
+	 */
+	int removeClassifier(UMLClassifierListItem* classifier);
+	
+	/**
+	 * Take classifier, It is the client responsibility to hand over
+	 * ownership of the classifier, or to delete it.
+	 * @param classifier Classifier to be take.
+	 * @returns number of remaining classifiers if found, and -1 if not.
+	 */
+	UMLClassifierListItem* takeClassifier(UMLClassifierListItem* classifier);
+	
 	UMLClassifier* m_pClassifier;
 	QGroupBox* m_pDocGB;
 	QGroupBox* m_pItemListGB;
 	QListBox* m_pItemListLB;
 	QTextEdit* m_pDocTE;
-	UMLClassifierListItemList* m_pItemList;
-	UMLObject_Type itemType;
+	Uml::Object_Type itemType;
 
 	KArrowButton* m_pUpArrowB;
 	KArrowButton* m_pDownArrowB;
@@ -134,16 +163,6 @@ public slots:
 	 * shows properties dialogue for currently selected attribute
 	 */
 	void slotProperties();
-
-	/**
-	 * Give dialogue box to add a new stereotype
-	 */
-	void slotNewStereotype();
-signals:
-	/**
-	 * slots into classpropdlg::slotUpdateChildObject() whatever that does
-	 */
-	void sigUpdateChildObject(int id);
 };
 
 #endif
