@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -16,18 +21,19 @@
 #include "../interfacewidget.h"
 #include "classoptionspage.h"
 
-ClassOptionsPage::ClassOptionsPage(QWidget* pParent, UMLWidget* pWidget, UMLWidget_Type type) : QWidget( pParent ) {
-	if (type == wt_Class) {
+ClassOptionsPage::ClassOptionsPage(QWidget* pParent, UMLWidget* pWidget, Uml::Widget_Type type)
+  : QWidget( pParent ) {
+	if (type == Uml::wt_Class) {
 		m_pClassWidget = static_cast<ClassWidget*>(pWidget);
 		m_pInterfaceWidget = 0;
 		setupClassPage();
-	} else if (type == wt_Interface) {
+	} else if (type == Uml::wt_Interface) {
 		m_pInterfaceWidget = static_cast<InterfaceWidget*>(pWidget);
 		m_pClassWidget = 0;
 		setupInterfacePage();
 	}
 	m_options = 0;
-	
+
 }
 
 ClassOptionsPage::ClassOptionsPage(QWidget* pParent, Settings::OptionState *options) : QWidget( pParent )
