@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,7 +41,7 @@ public:
 	/**
 	*	Constructor - observe and modify a Widget
 	*/
-	ClassOptionsPage(QWidget* pParent, UMLWidget* pWidget, UMLWidget_Type type);
+	ClassOptionsPage(QWidget* pParent, UMLWidget* pWidget, Uml::Widget_Type type);
 	
 	/**
 	*	Constructor - observe and modify an OptionState structure
