@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -77,7 +82,7 @@ private:
 	/**
 	 * list containing the IDs of diagrams to print
 	 */
-	QValueList<int> m_nIdList;
+	QValueList<Uml::IDType> m_nIdList;
 
 	enum FilterType{Current = 0, All, Select, Type};
 public slots:
