@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -30,9 +35,10 @@
 #include "../attribute.h"
 #include "../interface.h"
 #include "../classifier.h"
+#include "../template.h"
 #include "../umldoc.h"
 #include "../uml.h"
-#include "dialog_utils.h"
+#include "../dialog_utils.h"
 
 UMLAttributeDialog::UMLAttributeDialog( QWidget * pParent, UMLAttribute * pAttribute )
   : KDialogBase( Plain, i18n("Attribute Properties"), Help | Ok | Cancel , Ok, pParent, "_UMLATTRIBUTEDLG_", true, true) {
@@ -60,17 +66,17 @@ void UMLAttributeDialog::setupDialog() {
 	valuesLayout -> addWidget(m_pTypeCB, 0, 1);
 	m_pTypeL->setBuddy(m_pTypeCB);
 
-	Dialog_Utils::makeLabeledEditField( m_pValuesGB, valuesLayout, 1,
-					    m_pNameL, i18n("&Name:"),
-					    m_pNameLE, m_pAttribute->getName() );
+	Umbrello::makeLabeledEditField( m_pValuesGB, valuesLayout, 1,
+					m_pNameL, i18n("&Name:"),
+					m_pNameLE, m_pAttribute->getName() );
 
-	Dialog_Utils::makeLabeledEditField( m_pValuesGB, valuesLayout, 2,
-					    m_pInitialL, i18n("&Initial value:"),
-					    m_pInitialLE, m_pAttribute->getInitialValue() );
+	Umbrello::makeLabeledEditField( m_pValuesGB, valuesLayout, 2,
+					m_pInitialL, i18n("&Initial value:"),
+					m_pInitialLE, m_pAttribute->getInitialValue() );
 
-	Dialog_Utils::makeLabeledEditField( m_pValuesGB, valuesLayout, 3,
-					    m_pStereoTypeL, i18n("Stereotype name:"),
-					    m_pStereoTypeLE, m_pAttribute->getStereotype() );
+	Umbrello::makeLabeledEditField( m_pValuesGB, valuesLayout, 3,
+					m_pStereoTypeL, i18n("Stereotype name:"),
+					m_pStereoTypeLE, m_pAttribute->getStereotype(false) );
 
 	m_pStaticCB = new QCheckBox( i18n("Classifier &scope (\"static\")"), m_pValuesGB );
 	m_pStaticCB -> setChecked( m_pAttribute -> getStatic() );
@@ -107,12 +113,7 @@ void UMLAttributeDialog::setupDialog() {
 	UMLClassifierList namesList( pDoc->getConcepts() );
 	UMLClassifier* obj;
 	for (obj=namesList.first(); obj!=0; obj=namesList.next()) {
-		insertType( obj->getName() );
-	}
-	UMLInterfaceList interfaceList( pDoc->getInterfaces() );
-	UMLInterface* pInterface = 0;
-	for(pInterface=interfaceList.first(); pInterface!=0 ;pInterface=interfaceList.next()) {
-		insertType( pInterface->getName() );
+		insertType( obj->getFullyQualifiedName(".") );
 	}
 
 	//work out which one to select
@@ -144,32 +145,14 @@ void UMLAttributeDialog::slotNameChanged( const QString &_text )
 }
 
 bool UMLAttributeDialog::apply() {
-	UMLClassifier * pConcept = dynamic_cast<UMLClassifier *>( m_pAttribute->parent() );
-
-	if ( m_pPublicRB->isChecked() ) {
-		m_pAttribute->setScope(Uml::Public);
-	} else if ( m_pPrivateRB -> isChecked() ) {
-		m_pAttribute->setScope(Uml::Private);
-	} else {
-		m_pAttribute->setScope(Uml::Protected);
-	}
-
-	QString typeName = m_pTypeCB->currentText();
-	// TODO: The type of UMLAttributes is just a "passive" string.
-	// UMLAttribute should be enhanced to give the user the choice
-	// of referencing existing UMLClassifiers instead.
-	m_pAttribute->setTypeName( typeName );
-	m_pAttribute->setInitialValue( m_pInitialLE->text() );
 	QString name = m_pNameLE->text();
-	if( name.length() == 0 ) {
+	if (name.isEmpty()) {
 		KMessageBox::error(this, i18n("You have entered an invalid attribute name."),
 		                   i18n("Attribute Name Invalid"), false);
 		m_pNameLE->setText( m_pAttribute->getName() );
 		return false;
 	}
-
-	m_pAttribute->setStereotype( m_pStereoTypeLE->text() );
-
+	UMLClassifier * pConcept = dynamic_cast<UMLClassifier *>( m_pAttribute->parent() );
 	UMLObjectList list= pConcept->findChildObject(Uml::ot_Attribute, name);
 	if( list.count() != 0 && list.findRef( m_pAttribute ) ) {
 		KMessageBox::error(this, i18n("The attribute name you have chosen is already being used in this operation."),
@@ -178,12 +161,37 @@ bool UMLAttributeDialog::apply() {
 		return false;
 	}
 	m_pAttribute->setName(name);
+	if ( m_pPublicRB->isChecked() ) {
+		m_pAttribute->setScope(Uml::Public);
+	} else if ( m_pPrivateRB -> isChecked() ) {
+		m_pAttribute->setScope(Uml::Private);
+	} else {
+		m_pAttribute->setScope(Uml::Protected);
+	}
+	m_pAttribute->setInitialValue( m_pInitialLE->text() );
+	m_pAttribute->setStereotype( m_pStereoTypeLE->text() );
 	m_pAttribute->setStatic( m_pStaticCB->isChecked() );
 
-	UMLDoc *umldoc = UMLApp::app()->getDocument();
-	if (umldoc->findUMLObject(typeName) == NULL)
-		umldoc->createDatatype( m_pTypeCB->currentText() );
-
+	QString typeName = m_pTypeCB->currentText();
+	UMLTemplate *tmplParam = pConcept->findTemplate(typeName);
+	if (tmplParam) {
+		m_pAttribute->setType(tmplParam);
+		return true;
+	}
+	UMLDoc * pDoc = UMLApp::app()->getDocument();
+	UMLObject *obj = pDoc->findUMLObject(typeName);
+	UMLClassifier *classifier = dynamic_cast<UMLClassifier*>(obj);
+	if (classifier == NULL) {
+		// If it's obviously a pointer type (C++) then create a datatype.
+		// Else we don't know what it is so as a compromise create a class.
+		Uml::Object_Type ot = (typeName.contains('*') ? Uml::ot_Datatype
+							      : Uml::ot_Class);
+		obj = pDoc->createUMLObject(ot, typeName);
+		if (obj == NULL)
+			return false;
+		classifier = static_cast<UMLClassifier*>(obj);
+	}
+	m_pAttribute->setType( classifier );
 	return true;
 }
 
