@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -14,26 +19,21 @@
 #include <kdebug.h>
 #include <klocale.h>
 
-UMLEnum::UMLEnum(const QString& name, int id) : UMLClassifier(name, id) {
+UMLEnum::UMLEnum(const QString& name, Uml::IDType id) : UMLClassifier(name, id) {
 	init();
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 UMLEnum::~UMLEnum() {
-	m_EnumLiteralList.clear();
+	m_List.clear();
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 bool UMLEnum::operator==( UMLEnum & rhs ) {
-	if ( m_OpsList.count() != rhs.m_OpsList.count() ) {
-		return false;
-	}
 	return UMLClassifier::operator==(rhs);
 }
 
 void UMLEnum::copyInto(UMLEnum *rhs) const
 {
 	UMLClassifier::copyInto(rhs);
-
-	m_EnumLiteralList.copyInto(&(rhs->m_EnumLiteralList));
 }
 
 UMLObject* UMLEnum::clone() const
@@ -49,8 +49,8 @@ void UMLEnum::saveToXMI(QDomDocument& qDoc, QDomElement& qElement) {
 	QDomElement enumElement = UMLObject::save("UML:Enumeration", qDoc);
 	//save operations
 	UMLClassifierListItem* pEnumLiteral = 0;
-	for ( pEnumLiteral = m_EnumLiteralList.first(); pEnumLiteral != 0;
-	      pEnumLiteral = m_EnumLiteralList.next() ) {
+	for ( pEnumLiteral = m_List.first(); pEnumLiteral != 0;
+	      pEnumLiteral = m_List.next() ) {
 		pEnumLiteral->saveToXMI(qDoc, enumElement);
 	}
 	qElement.appendChild(enumElement);
@@ -58,16 +58,20 @@ void UMLEnum::saveToXMI(QDomDocument& qDoc, QDomElement& qElement) {
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 bool UMLEnum::load(QDomElement& element) {
 	QDomNode node = element.firstChild();
-	QDomElement tempElement = node.toElement();
-	while( !tempElement.isNull() ) {
+	while( !node.isNull() ) {
+		if (node.isComment()) {
+			node = node.nextSibling();
+			continue;
+		}
+		QDomElement tempElement = node.toElement();
 		QString tag = tempElement.tagName();
-		if (tagEq(tag, "EnumerationLiteral") ||
-		    tagEq(tag, "EnumLiteral")) {   // for backward compatibility
+		if (Uml::tagEq(tag, "EnumerationLiteral") ||
+		    Uml::tagEq(tag, "EnumLiteral")) {   // for backward compatibility
 			UMLEnumLiteral* pEnumLiteral = new UMLEnumLiteral(this);
 			if( !pEnumLiteral->loadFromXMI(tempElement) ) {
 				return false;
 			}
-			m_EnumLiteralList.append(pEnumLiteral);
+			m_List.append(pEnumLiteral);
 		} else if (tag == "stereotype") {
 			kdDebug() << "UMLEnum::load(" << m_Name
 				  << "): losing old-format stereotype." << endl;
@@ -75,19 +79,18 @@ bool UMLEnum::load(QDomElement& element) {
 			kdWarning() << "unknown child type in UMLEnum::load" << endl;
 		}
 		node = node.nextSibling();
-		tempElement = node.toElement();
 	}//end while
 	return true;
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 void UMLEnum::init() {
-	m_BaseType = ot_Enum;
+	m_BaseType = Uml::ot_Enum;
 	setStereotype( i18n("enum") );
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLObject* UMLEnum::addEnumLiteral(QString name, int id) {
+UMLObject* UMLEnum::addEnumLiteral(const QString &name, Uml::IDType id) {
 	UMLEnumLiteral* literal = new UMLEnumLiteral(this, name, id);
-	m_EnumLiteralList.append(literal);
+	m_List.append(literal);
 	emit modified();
 	connect(literal,SIGNAL(modified()),this,SIGNAL(modified()));
 	emit enumLiteralAdded(literal);
@@ -99,7 +102,7 @@ bool UMLEnum::addEnumLiteral(UMLEnumLiteral* literal, IDChangeLog* Log /* = 0*/)
 	if (findChildObject(Uml::ot_EnumLiteral, name).count() == 0) {
 		literal->parent()->removeChild(literal);
 		this->insertChild(literal);
-		m_EnumLiteralList.append(literal);
+		m_List.append(literal);
 		emit modified();
 		connect(literal,SIGNAL(modified()),this,SIGNAL(modified()));
 		emit enumLiteralAdded(literal);
@@ -116,10 +119,10 @@ bool UMLEnum::addEnumLiteral(UMLEnumLiteral* literal, int position) {
 	if (findChildObject( Uml::ot_EnumLiteral, name).count() == 0) {
 		literal->parent()->removeChild(literal);
 		this->insertChild(literal);
-		if ( position >= 0 && position <= (int)m_EnumLiteralList.count() )  {
-			m_EnumLiteralList.insert(position,literal);
+		if ( position >= 0 && position <= (int)m_List.count() )  {
+			m_List.insert(position,literal);
 		} else {
-			m_EnumLiteralList.append(literal);
+			m_List.append(literal);
 		}
 		emit modified();
 		connect(literal,SIGNAL(modified()),this,SIGNAL(modified()));
@@ -130,7 +133,7 @@ bool UMLEnum::addEnumLiteral(UMLEnumLiteral* literal, int position) {
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 int UMLEnum::removeEnumLiteral(UMLObject* literal) {
-	if (!m_EnumLiteralList.remove((UMLEnumLiteral*)literal)) {
+	if (!m_List.remove((UMLEnumLiteral*)literal)) {
 		kdDebug() << "can't find att given in list" << endl;
 		return -1;
 	}
@@ -140,12 +143,12 @@ int UMLEnum::removeEnumLiteral(UMLObject* literal) {
 	// for us by QObject. -b.t.
 	// disconnect(a,SIGNAL(modified()),this,SIGNAL(modified()));
 	delete literal;
-	return m_EnumLiteralList.count();
+	return m_List.count();
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 UMLEnumLiteral* UMLEnum::takeEnumLiteral(UMLEnumLiteral* el) {
-	int index = m_EnumLiteralList.findRef( el );
-	el = (index == -1 ? 0 : dynamic_cast<UMLEnumLiteral*>(m_EnumLiteralList.take( )));
+	int index = m_List.findRef( el );
+	el = (index == -1 ? 0 : dynamic_cast<UMLEnumLiteral*>(m_List.take( )));
 	if (el) {
 		emit enumLiteralRemoved(el);
 		emit modified();
@@ -153,20 +156,16 @@ UMLEnumLiteral* UMLEnum::takeEnumLiteral(UMLEnumLiteral* el) {
 	return el;
 }
 
-UMLObjectList UMLEnum::findChildObject(UMLObject_Type t, QString n,
-				       bool seekStereo /* = false */) {
+UMLObjectList UMLEnum::findChildObject(Uml::Object_Type t, const QString &n) {
 	UMLObjectList list;
-	if (t == ot_Association) {
-		return UMLClassifier::findChildObject(t, n, seekStereo);
-	} else if (t == ot_EnumLiteral) {
+	if (t == Uml::ot_Association) {
+		return UMLClassifier::findChildObject(t, n);
+	} else if (t == Uml::ot_EnumLiteral) {
 		UMLClassifierListItem * obj=0;
-		for(obj=m_EnumLiteralList.first();obj != 0;obj=m_EnumLiteralList.next()) {
+		for (obj = m_List.first(); obj != 0; obj = m_List.next()) {
 			if (obj->getBaseType() != t)
 				continue;
-			if (seekStereo) {
-				if (obj->getStereotype() == n)
-					list.append( obj );
-			} else if (obj->getName() == n)
+			if (obj->getName() == n)
 				list.append( obj );
 		}
 	} else {
@@ -176,32 +175,9 @@ UMLObjectList UMLEnum::findChildObject(UMLObject_Type t, QString n,
 	return list;
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLObject* UMLEnum::findChildObject(int id) {
-	UMLClassifierListItem * a = 0;
-	for(a=m_EnumLiteralList.first();a != 0;a=m_EnumLiteralList.next()) {
-		if(a->getID() == id)
-			return a;
-	}
-	return UMLClassifier::findChildObject(id);
-}
-////////////////////////////////////////////////////////////////////////////////////////////////////
 int UMLEnum::enumLiterals() {
-	return m_EnumLiteralList.count();
-}
-////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLClassifierListItemList* UMLEnum::getEnumLiteralList() {
-	return &m_EnumLiteralList;
-}
-////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLEnumLiteralList UMLEnum::getFilteredEnumLiteralList() {
-	UMLEnumLiteralList enumLiteralList;
-	for(UMLClassifierListItem* listItem = m_EnumLiteralList.first(); listItem;
-	    listItem = m_EnumLiteralList.next())  {
-		if (listItem->getBaseType() == ot_EnumLiteral) {
-			enumLiteralList.append(static_cast<UMLEnumLiteral*>(listItem));
-		}
-	}
-	return enumLiteralList;
+	return m_List.count();
 }
 
+
 #include "enum.moc"
