@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2003-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -14,8 +19,6 @@
 
 class UMLInterface;
 
-#define INTERFACE_MARGIN 5
-
 /**
  * Defines a graphical version of the interface.  Most of the functionality
  * comes from its ancestors, @ref ClassifierWidget and @ref UMLWidget.
