@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -11,6 +16,7 @@
 #define MESSAGEWIDGET_H
 
 #include "umlwidget.h"
+#include "linkwidget.h"
 
 // forward declarations
 class FloatingText;
@@ -32,7 +38,7 @@ class ObjectWidget;
  * @see	FloatingText
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
-class MessageWidget : public UMLWidget {
+class MessageWidget : public UMLWidget, public LinkWidget {
 	Q_OBJECT
 public:
 
@@ -49,7 +55,8 @@ public:
 	 *              The default (-1) will prompt generation of a new ID.
 	 */
 	MessageWidget(UMLView * view, ObjectWidget* a, ObjectWidget* b, FloatingText* ft, 
-		      int y, Sequence_Message_Type sequenceMessageType, int id = -1);
+		      int y, Uml::Sequence_Message_Type sequenceMessageType,
+		      Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Constructs a MessageWidget.
@@ -57,7 +64,7 @@ public:
 	 * @param view		The parent to this class.
 	 * @param id		The ID to assign (-1 will prompt a new ID.)
 	 */
-	MessageWidget(UMLView * view, Sequence_Message_Type sequenceMessageType, int id = -1);
+	MessageWidget(UMLView * view, Uml::Sequence_Message_Type sequenceMessageType, Uml::IDType id = Uml::id_None);
 
 	/**
 	 * Initializes key variables of the class.
@@ -72,7 +79,7 @@ public:
 	/**
 	 * Write property of QString m_SequenceNumber.
 	 */
-	void setSequenceNumber( QString sequenceNumber );
+	void setSequenceNumber( const QString &sequenceNumber );
 
 	/**
 	 * Read property of QString m_SequenceNumber.
@@ -82,7 +89,7 @@ public:
 	/**
 	 * Write property of QString m_Operation.
 	 */
-	void setOperation( QString operation );
+	void setOperation( const QString &operation );
 
 	/**
 	 * Read property of QString m_Operation.
@@ -92,7 +99,7 @@ public:
 	/**
 	 * Returns whether the message is synchronous or asynchronous
 	 */
-	Sequence_Message_Type getSequenceMessageType() const {
+	Uml::Sequence_Message_Type getSequenceMessageType() const {
 		return m_sequenceMessageType;
 	}
 
@@ -109,14 +116,14 @@ public:
 	 *
 	 * @return	The ObjectWidget we are related to.
 	 */
-	ObjectWidget* getWidget(Role_Type role);
+	ObjectWidget* getWidget(Uml::Role_Type role);
 
 	/**
 	 * Sets the related widget on the given side.
 	 *
 	 * @param ow	The ObjectWidget we are related to.
 	 */
-	void setWidget(ObjectWidget * ow, Role_Type role) ;
+	void setWidget(ObjectWidget * ow, Uml::Role_Type role) ;
 
 	/**
 	 * Returns the text widget it is related to.
@@ -136,6 +143,12 @@ public:
 		m_pFText = f;
 	}
 
+	/**
+	 * Implements operation from LinkWidget.
+	 * Required by FloatingText.
+	 */
+	void lwSetFont (QFont font);
+
 	/**
 	 * Overrides operation from LinkWidget.
 	 * Required by FloatingText.
@@ -146,7 +159,7 @@ public:
 	 * Overrides operation from LinkWidget.
 	 * Required by FloatingText.
 	 */
-	void setOperationText(FloatingText *ft, QString opText);
+	void setOperationText(FloatingText *ft, const QString &opText);
 
 	/**
 	 * Overrides operation from LinkWidget.
@@ -163,7 +176,7 @@ public:
 	 * @param ft	The text widget which to update.
 	 * @param newText	The new text to set.
 	 */
-	void setText(FloatingText *ft, QString newText);
+	void setText(FloatingText *ft, const QString &newText);
 
 	/**
 	 * Overrides operation from LinkWidget.
@@ -172,7 +185,7 @@ public:
 	 * @param seqNum	The new sequence number string to set.
 	 * @param op		The new operation string to set.
 	 */
-	void setSeqNumAndOp(QString seqNum, QString op);
+	void setSeqNumAndOp(const QString &seqNum, const QString &op);
 
 	/**
 	 * Overrides operation from LinkWidget.
@@ -185,12 +198,6 @@ public:
 	UMLClassifier * getSeqNumAndOp(FloatingText *ft, QString& seqNum,
 							 QString& op);
 
-	/**
-	 * Overrides operation from LinkWidget.
-	 * Required by FloatingText.
-	 */
-	void setupAfterFTsetLink(FloatingText *ft);
-
 	/**
 	 * Calculate the geometry of the widget.
 	 */
@@ -204,8 +211,9 @@ public:
 
 	/**
 	 * Calculates the size of the widget by calling
-	 * calculateDimenstionsSynchronous() or
-	 * calculateDimenstionsAsynchronous()
+	 * calculateDimenstionsSynchronous(),
+	 * calculateDimenstionsAsynchronous(), or
+	 * calculateDimensionsCreation()
 	 */
 	void calculateDimensions();
 
@@ -219,6 +227,11 @@ public:
 	 */
 	void calculateDimensionsAsynchronous();
 
+	/**
+	 * Calculates and sets the size of the widget for a creation message
+	 */
+	void calculateDimensionsCreation();
+
 	/** 
 	 * Calls drawSynchronous() or drawAsynchronous()
 	 */
@@ -236,17 +249,30 @@ public:
 	 */
 	void drawAsynchronous(QPainter& p, int offsetX, int offsetY);
 
+	/**
+	 * Draws a solid arrow line and a stick arrow head to the
+	 * edge of the target object widget instead of to the
+	 * sequence line.
+	 */
+	void drawCreation(QPainter& p, int offsetX, int offsetY);
+
 	/**
 	 * Sets the text position relative to the sequence message.
 	 */
 	void setTextPosition();
 
 	/**
-	 * Updates the message widget X and Y coordinates based on
-	 * the given height of a floating text.
+	 * Constrains the FloatingText X and Y values supplied.
 	 * Overrides operation from LinkWidget.
+	 *
+	 * @param textX		Candidate X value (may be modified by the constraint.)
+	 * @param textY		Candidate Y value (may be modified by the constraint.)
+	 * @param textWidth	Width of the text.
+	 * @param textHeight	Height of the text.
+	 * @param tr		Uml::Text_Role of the text.
 	 */
-	void updateMessagePos(int textHeight, int& newX, int& newY);
+	void constrainTextPos(int &textX, int &textY, int textWidth, int textHeight,
+			      Uml::Text_Role tr);
 
 	/**
 	 * Used to cleanup any other widget it may need to delete.
@@ -279,6 +305,15 @@ public:
 	 */
 	int getMaxHeight();
 
+	/**
+	 * Overrides operation from UMLWidget.
+	 *
+	 * @param p Point to be checked.
+	 *
+	 * @return True if the point is on a visual part of the MessageWidget
+	 */
+	bool onWidget(const QPoint & p);
+
 	/**
 	 * Overrides the standard operation.
 	 */
@@ -290,23 +325,35 @@ public:
 	virtual void mousePressEvent(QMouseEvent *me);
 
 	/**
-	 * Saves to the <UML:MessageWidget> XMI element.
+	 * Saves to the <messagewidget> XMI element.
 	 */
 	void saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
 
 	/**
-	 * Loads from the <UML:MessageWidget> XMI element.
+	 * Loads from the <messagewidget> XMI element.
 	 */
 	bool loadFromXMI( QDomElement & qElement );
 
 protected:
+	/**
+	 * Shortcut for calling m_pFText->setLink() followed by
+	 * this->setTextPosition().
+	 */
+	void setLinkAndTextPos();
+
+	/**
+	 * Returns the textX arg with constraints applied.
+	 * Auxiliary to setTextPosition() and constrainTextPos().
+	 */
+	int constrainX(int textX, int textWidth, Uml::Text_Role tr);
+
 	// Data loaded/saved
 	QString m_SequenceNumber;
 	QString m_Operation;
 	/**
 	 * Whether the message is synchronous or asynchronous
 	 */
-        Sequence_Message_Type m_sequenceMessageType;
+        Uml::Sequence_Message_Type m_sequenceMessageType;
 
 private:
 	void moveEvent(QMoveEvent */*m*/);
@@ -316,8 +363,9 @@ private:
 	ObjectWidget * m_pOw[2];
 	FloatingText * m_pFText;
 	int m_nY;
+	bool m_inSelection;
 public slots:
-	void slotWidgetMoved(int id);
+	void slotWidgetMoved(Uml::IDType id);
 	void slotMenuSelection(int sel);
 signals:
 	/**
