@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -22,7 +27,6 @@ class UMLClassifier;
  */
 class UMLOperation : public UMLClassifierListItem {
 	Q_OBJECT
-friend UMLOperation* UMLDoc::createOperation( UMLClassifier *, const QString &, UMLAttributeList * );
 public:
 	/**
 	 * Constructs an UMLOperation.
@@ -36,7 +40,8 @@ public:
 	 * @param s		The scope of the operation.
 	 * @param rt		The return type of the operation.
 	 */
-	UMLOperation(const UMLClassifier * parent, QString Name, int id, Scope s = Public, QString rt = "void");
+	UMLOperation(const UMLClassifier * parent, QString Name, Uml::IDType id = Uml::id_None,
+		     Uml::Scope s = Uml::Public, QString rt = "void");
 
 	/**
 	 * Constructs an UMLOperation.
@@ -70,24 +75,6 @@ public:
 	 */
 	virtual UMLObject* clone() const;
 
-	/**
-	 * Sets the return type of the operation.
-	 *
-	 * @param rt	The return type to set the operation to.
-	 */
-	void setReturnType(QString rt) {
-		m_ReturnType = rt;
-	}
-
-	/**
-	 * Get the return type of the operation.
-	 *
-	 * @return	The return type.
-	 */
-	QString getReturnType() const {
-		return m_ReturnType;
-	}
-
 	/**
 	 * Add a parameter to the operation.
 	 *
@@ -99,8 +86,22 @@ public:
 	 * @return	Pointer to the UMLAttribute created.
 	 */
 	UMLAttribute * addParm(QString type, QString name, QString initialValue, QString doc,
-		               Uml::Parameter_Kind kind = Uml::pk_In);
+		               Uml::Parameter_Direction kind = Uml::pd_In);
 
+	/**
+	 * Move a parameter one position to the left.
+	 *
+	 * @param a		The parameter to move.
+	 */
+	void moveParmLeft(UMLAttribute *a);
+	
+	/**
+	 *Move a parameter one position to the right.
+	 *
+	 * @param a		The parameter to move.
+	 */
+	void moveParmRight(UMLAttribute *a);
+	 
 	/**
 	 * Remove a parameter from the operation.
 	 *
@@ -123,7 +124,7 @@ public:
 	 * @param name	The parameter name to search for.
 	 * @return	The found parameter, 0 if not found.
 	 */
-	UMLAttribute * findParm(QString name);
+	UMLAttribute * findParm(const QString &name);
 
 	/**
 	 * Returns a string representation of the operation.
@@ -131,7 +132,7 @@ public:
 	 * @param sig	What type of operation string to show.
 	 * @return	The string representation of the operation.
 	 */
-	QString toString(Signature_Type sig = st_NoSig);
+	QString toString(Uml::Signature_Type sig = Uml::st_NoSig);
 
 	/**
 	 * Add a parameter to the operation.
@@ -143,6 +144,14 @@ public:
 	 */
 	void addParm(UMLAttribute *parameter, int position = -1);
 
+	/**
+	 * Calls resolveRef() on all parameters.
+	 * Needs to be called after all UML objects are loaded from file.
+	 *
+	 * @return	True for success.
+	 */
+	bool resolveRef();
+
 	/**
 	 * Returns an unused parameter name for a new parameter.
 	 */
@@ -163,7 +172,7 @@ public:
 	 *
 	 * @return	True if this operation is a constructor.
 	 */
-	bool isConstructorOperation ( ) const;
+	bool isConstructorOperation();
 
 protected:
 	/**
@@ -172,7 +181,6 @@ protected:
 	bool load( QDomElement & element );
 
 private:
-	QString m_ReturnType;
 	UMLAttributeList m_List;
 };
 
