@@ -1,6 +1,4 @@
 /***************************************************************************
-                          refactoringassistant.cpp
-                             -------------------
     copyright            : (C) 2003 Luis De la Parra
  ***************************************************************************/
 /***************************************************************************
@@ -38,9 +36,6 @@
 using std::type_info;
 
 
-namespace Umbrello {
-
-
 RefactoringAssistant::RefactoringAssistant( UMLDoc *doc, UMLClassifier *obj, QWidget *parent, const char *name ):
 		KListView( parent, name ), m_doc( doc )
 {
@@ -67,7 +62,7 @@ RefactoringAssistant::RefactoringAssistant( UMLDoc *doc, UMLClassifier *obj, QWi
 		this,SLOT(showContextMenu(KListView*,QListViewItem*,const QPoint&)));
 
 	resize(300,400);
-	
+
 	refactor( obj );
 }
 
@@ -110,12 +105,13 @@ UMLObject* RefactoringAssistant::findUMLObject( const QListViewItem *item )
 
 QListViewItem* RefactoringAssistant::findListViewItem( const UMLObject *obj )
 {
-	UMLObjectMap::iterator it;
-	for( it = m_umlObjectMap.begin() ; it != m_umlObjectMap.end() ; ++it )
+	UMLObjectMap::iterator end(m_umlObjectMap.end());
+	for( UMLObjectMap::iterator it(m_umlObjectMap.begin()) ; it != end ; ++it )
 		if( (*it).second == obj )
 			return (*it).first;
-	kdWarning()<<"RefactoringAssistant::findListViewItem( UMLObject *obj )"
-			   <<"object id "<<obj->getID()<<"does not have s ListItem"<<endl;
+	kdWarning() << "RefactoringAssistant::findListViewItem:"
+		    << "object id " << ID2STR(obj->getID())
+		    << "does not have a ListItem" << endl;
 	return 0L;
 }
 
@@ -181,7 +177,7 @@ void RefactoringAssistant::operationAdded( UMLOperation *op )
 			setVisibilityIcon( item, op );
 			break;
 		}
-	}	
+	}
 }
 
 void RefactoringAssistant::operationRemoved( UMLOperation *op )
@@ -337,7 +333,7 @@ void RefactoringAssistant::addBaseClassifier()
 			   <<"called for a non-classifier object"<<endl;
 		return;
 	}
-	
+
 	//classes have classes and interfaces interfaces as super/derived classifiers
 	UMLClassifier *super = static_cast<UMLClassifier*>(m_doc->createUMLObject( typeid(*obj) ) );
 	if(!super)
@@ -358,7 +354,7 @@ void RefactoringAssistant::addBaseClassifier()
 	item->setExpandable( true );
 	m_umlObjectMap[item] = super;
 	addClassifier( super, item, true, false, true);
-/////////////////////////	
+/////////////////////////
 }
 
 void RefactoringAssistant::addDerivedClassifier()
@@ -377,13 +373,13 @@ void RefactoringAssistant::addDerivedClassifier()
 			   <<"called for a non-classifier object"<<endl;
 		return;
 	}
-	
+
 	//classes have classes and interfaces interfaces as super/derived classifiers
 	UMLClassifier *derived = static_cast<UMLClassifier*>(m_doc->createUMLObject( typeid(*obj) ));
 	if(!derived)
 		return;
 	m_doc->createUMLAssociation( derived, obj, Uml::at_Generalization );
-	
+
 //////////////////////   Manually add the classifier to the assitant - would be nicer to do it with
 /////////////////////    a signal, like operations and attributes
 	QListViewItem *derivedFolder = item->firstChild();
@@ -399,7 +395,7 @@ void RefactoringAssistant::addDerivedClassifier()
 	item->setExpandable( true );
 	m_umlObjectMap[item] = derived;
 	addClassifier( derived, item, false, true, true);
-/////////////////////////	
+/////////////////////////
 }
 
 void RefactoringAssistant::addInterfaceImplementation()
@@ -415,7 +411,7 @@ void RefactoringAssistant::addInterfaceImplementation()
 // 	if(!n)
 // 		return;
 // 	m_doc->createUMLAssociation( n, obj, Uml::at_Realization );
-// 	//refresh, add classifier to assistant	
+// 	//refresh, add classifier to assistant
 }
 
 void RefactoringAssistant::createOperation()
@@ -463,36 +459,37 @@ void RefactoringAssistant::addClassifier( UMLClassifier *classifier, QListViewIt
 	}
 
 	connect( classifier, SIGNAL( modified() ), this, SLOT( umlObjectModified() ) );
-	
-	if( dynamic_cast<UMLClass*>(classifier) )
+
+	UMLClass *klass = dynamic_cast<UMLClass*>(classifier);
+	if( klass )
 	{// only Classes have attributes...
 		connect( classifier, SIGNAL(attributeAdded(UMLAttribute*)),
 			this,SLOT( attributeAdded(UMLAttribute*)));
 		connect( classifier, SIGNAL(attributeRemoved(UMLAttribute*)),
 			this,SLOT( attributeRemoved(UMLAttribute*)));
-			
+
 		QListViewItem *attsFolder = new KListViewItem( classifierItem, i18n("Attributes"), "attributes" );
 		attsFolder->setPixmap(0,SmallIcon("folder_green_open"));
 		attsFolder->setExpandable( true );
-		UMLAttributeList* atts = static_cast<UMLClass*>(classifier)->getFilteredAttributeList();
-		for( UMLAttribute *att = atts->first(); att; att = atts->next() )
+		UMLAttributeList atts = klass->getAttributeList();
+		for( UMLAttribute *att = atts.first(); att; att = atts.next() )
 		{
 			attributeAdded( att );
 		}
-		
+
 	}
-	
+
 	// add operations
 	connect( classifier, SIGNAL(operationAdded(UMLOperation*)),
 		this,SLOT( operationAdded(UMLOperation*)));
 	connect( classifier, SIGNAL(operationRemoved(UMLOperation*)),
 		this,SLOT( operationRemoved(UMLOperation*)));
-		
+
 	QListViewItem *opsFolder = new KListViewItem( classifierItem, i18n("Operations"), "operations" );
 	opsFolder->setPixmap(0,SmallIcon("folder_blue_open"));
 	opsFolder->setExpandable( true );
-	UMLOperationList *ops = classifier->getFilteredOperationsList();
-	for( UMLOperation *op = ops->first(); op ; op = ops->next() )
+	UMLOperationList ops(classifier->getOpList());
+	for( UMLOperation *op = ops.first(); op ; op = ops.next() )
 	{
 		operationAdded( op );
 	}
@@ -513,7 +510,7 @@ void RefactoringAssistant::addClassifier( UMLClassifier *classifier, QListViewIt
 			{
 				addClassifier( cl, item, true, false, true);
 			}
-	
+
 		}
 	}
 	if(addSub)
@@ -532,7 +529,7 @@ void RefactoringAssistant::addClassifier( UMLClassifier *classifier, QListViewIt
 			{
 				addClassifier( d, item, false, true, true);
 			}
-	
+
 		}
 	}
 }
@@ -548,10 +545,10 @@ bool RefactoringAssistant::acceptDrag(QDropEvent *event) const
 	}
 
 	RefactoringAssistant *me = const_cast<RefactoringAssistant*>(this);
-	
+
 	//ok, check if the move is valid
 	QListViewItem *movingItem = 0, *afterme = 0, *parentItem = 0;
-	me->findDrop(event->pos(), parentItem, afterme);	
+	me->findDrop(event->pos(), parentItem, afterme);
 	for( movingItem = firstChild(); movingItem != 0; movingItem = movingItem->itemBelow() )
 	{
 		if( movingItem->isSelected() )
@@ -561,7 +558,7 @@ bool RefactoringAssistant::acceptDrag(QDropEvent *event) const
 	{	kdDebug()<<"moving/parent items not found - can't accept drag!"<<endl;
 		return false;
 	}
-	
+
 	UMLObject *movingObject;
 	if( !(movingObject = me->findUMLObject(movingItem)) )
 	{
@@ -573,7 +570,7 @@ bool RefactoringAssistant::acceptDrag(QDropEvent *event) const
 		kdDebug()<<"only operations and attributes are movable! - return false"<<endl;
 		return false;
 	}
-	
+
 	kdDebug()<<"parent item is "<<parentItem->text(0)<<endl;
 	UMLObject *parentObject = me->findUMLObject(parentItem);
 	if( parentObject && dynamic_cast<UMLClassifier*>(parentObject) )
@@ -590,7 +587,7 @@ bool RefactoringAssistant::acceptDrag(QDropEvent *event) const
 		else
 		{
 			kdDebug()<<"moving to item "<<parentItem->text(0)<<" -- "<<parentItem->text(1)<<" not valid"<<endl;
-			return false;	
+			return false;
 		}
 	}
 	if( (typeid(*movingObject) == typeid(UMLAttribute)) && dynamic_cast<UMLClass*>( parentObject ) )
@@ -601,7 +598,7 @@ bool RefactoringAssistant::acceptDrag(QDropEvent *event) const
 	{
 		return true;
 	}
-	
+
 	kdDebug()<<"how did I get here? return false!!"<<endl;
 	return false;
 }
@@ -637,7 +634,7 @@ void RefactoringAssistant::movableDropEvent (QListViewItem* parentItem, QListVie
 			kdWarning()<<"New parent of object is not a Classifier - Drop had already been accepted - check!"<<endl;
 			return;
 		}
-	}	
+	}
 	if( typeid(*movingObject) == typeid(UMLOperation) )
 	{kdDebug()<<"moving operation"<<endl;
 		UMLOperation *op = static_cast<UMLOperation*>(movingObject);
@@ -683,21 +680,18 @@ void RefactoringAssistant::movableDropEvent (QListViewItem* parentItem, QListVie
 void RefactoringAssistant::loadPixmaps()
 {
 	KStandardDirs *dirs = KGlobal::dirs();
-	QString dataDir = dirs -> findResourceDir( "data", "umbrello/pics/public.png" );
+	QString dataDir = dirs -> findResourceDir( "data", "umbrello/pics/object.png" );
 	dataDir += "/umbrello/pics/";
 
-	m_pixmaps.Public.load( dataDir + "public.png" );
-	m_pixmaps.Protected.load( dataDir + "protected.png" );
-	m_pixmaps.Private.load( dataDir + "private.png" );
-	m_pixmaps.Generalization.load( dataDir + "generalization.xpm" );
-	m_pixmaps.Subclass.load( dataDir + "uniassoc.xpm" );
+	m_pixmaps.Public.load( dataDir + "CVpublic_var.png" );
+	m_pixmaps.Protected.load( dataDir + "CVprotected_var.png" );
+	m_pixmaps.Private.load( dataDir + "CVprivate_var.png" );
+	m_pixmaps.Generalization.load( dataDir + "generalisation.png" );
+	m_pixmaps.Subclass.load( dataDir + "uniassociation.png" );
 
 
 }
 
-}  //namespace Umbrello
-
-
 
 
 
