@@ -6,31 +6,37 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+#include "uml.h"
+#include "umldoc.h"
 #include "umlcanvasobject.h"
 #include "classifier.h"
 #include "association.h"
 #include "attribute.h"
 #include "operation.h"
 #include "template.h"
+#include "stereotype.h"
 #include "clipboard/idchangelog.h"
 #include <kdebug.h>
 #include <klocale.h>
 
-UMLCanvasObject::UMLCanvasObject(const QString & name, int id) 
-   : UMLObject(name, id) 
+UMLCanvasObject::UMLCanvasObject(const QString & name, Uml::IDType id)
+   : UMLObject(name, id)
 {
 	init();
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 UMLCanvasObject::~UMLCanvasObject() {
+	removeAllAssociations();
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 UMLAssociationList UMLCanvasObject::getSpecificAssocs(Uml::Association_Type assocType) {
 	UMLAssociationList list;
-	for (UMLAssociation* a = m_AssocsList.first(); a; a = m_AssocsList.next())
+	for (UMLAssociationListIt ait(m_AssocsList); ait.current(); ++ait) {
+		UMLAssociation *a = ait.current();
 		if (a->getAssocType() == assocType)
 			list.append(a);
+	}
 	return list;
 }
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -61,13 +67,46 @@ int UMLCanvasObject::removeAssociation(UMLAssociation * assoc) {
 	emit sigAssociationRemoved(assoc);
 	return m_AssocsList.count();
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
-QString UMLCanvasObject::uniqChildName(const UMLObject_Type type) {
+
+void UMLCanvasObject::removeAllAssociations() {
+	UMLDoc *umldoc = UMLApp::app()->getDocument();
+	for (UMLAssociationListIt ait(m_AssocsList); ait.current(); ++ait) {
+		UMLAssociation *assoc = ait.current();
+		umldoc->slotRemoveUMLObject(assoc);
+		UMLCanvasObject *other = NULL;
+		UMLObject* objA = assoc->getObject(Uml::A);
+		UMLObject* objB = assoc->getObject(Uml::B);
+		if (objA == this)
+			other = dynamic_cast<UMLCanvasObject*>(objA);
+		else
+			other = dynamic_cast<UMLCanvasObject*>(objB);
+		if (other)
+			other->removeAssociation(assoc);
+		delete assoc;
+	}
+	m_AssocsList.clear();
+}
+
+QString UMLCanvasObject::uniqChildName( const Uml::Object_Type type,
+					bool seekStereo /* = false */ ) {
 	QString currentName;
-	if (type == ot_Association) {
+	if (seekStereo) {
+		currentName = i18n("new_stereotype");
+	} else if (type == Uml::ot_Association) {
 		currentName = i18n("new_association");
+	} else if (type == Uml::ot_Attribute) {
+		currentName = i18n("new_attribute");
+	} else if (type == Uml::ot_Template) {
+		currentName = i18n("new_template");
+	} else if (type == Uml::ot_Operation) {
+		currentName = i18n("new_operation");
+	} else if (type == Uml::ot_EnumLiteral) {
+		currentName = i18n("new_literal");
+	} else if (type == Uml::ot_EntityAttribute) {
+		currentName = i18n("new_field");
 	} else {
-		kdWarning() << "uniqChildName() called for unknown child type" << endl;
+		kdWarning() << "uniqChildName() called for unknown child type " << type << endl;
+		return "ERROR_in_UMLCanvasObject_uniqChildName";
 	}
 
 	QString name = currentName;
@@ -76,13 +115,15 @@ QString UMLCanvasObject::uniqChildName(const UMLObject_Type type) {
 	}
 	return name;
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLObjectList UMLCanvasObject::findChildObject(UMLObject_Type t, QString n) {
+
+UMLObjectList UMLCanvasObject::findChildObject(Uml::Object_Type t, const QString &n) {
 	UMLObjectList list;
-	if (t == ot_Association) {
-		UMLAssociation * obj=0;
-		for (obj = m_AssocsList.first(); obj != 0; obj = m_AssocsList.next()) {
-			if (obj->getBaseType() == t && obj -> getName() == n)
+	if (t == Uml::ot_Association) {
+		for (UMLAssociationListIt ait(m_AssocsList); ait.current(); ++ait) {
+			UMLAssociation *obj = ait.current();
+			if (obj->getBaseType() != t)
+				continue;
+			if (obj->getName() == n)
 				list.append( obj );
 		}
 	} else {
@@ -90,10 +131,10 @@ UMLObjectList UMLCanvasObject::findChildObject(UMLObject_Type t, QString n) {
 	}
 	return list;
 }
-////////////////////////////////////////////////////////////////////////////////////////////////////
-UMLObject* UMLCanvasObject::findChildObject(int id) {
-	UMLAssociation * asso = 0;
-	for (asso = m_AssocsList.first(); asso != 0; asso = m_AssocsList.next()) {
+
+UMLObject* UMLCanvasObject::findChildObject(Uml::IDType id) {
+	for (UMLAssociationListIt ait(m_AssocsList); ait.current(); ++ait) {
+		UMLAssociation *asso = ait.current();
 		if (asso->getID() == id)
 			return asso;
 	}
@@ -120,6 +161,16 @@ bool UMLCanvasObject::operator==(UMLCanvasObject& rhs) {
 	return true;
 }
 
+void UMLCanvasObject::copyInto(UMLCanvasObject *rhs) const
+{
+	UMLObject::copyInto(rhs);
+
+	// TODO Associations are not copied at the moment. This because
+	// the duplicate function (on umlwidgets) do not copy the associations.
+	//
+	//rhs->m_AssocsList = m_AssocsList;
+}
+
 int UMLCanvasObject::associations() {
 	return m_AssocsList.count();
 }
@@ -130,27 +181,18 @@ const UMLAssociationList& UMLCanvasObject::getAssociations() {
 
 UMLClassifierList UMLCanvasObject::getSuperClasses() {
 	UMLClassifierList list;
-	// WARNING:
-	// Currently there's quite some chaos regarding role assignments in
-	// generalizations. The diagram code (AssociationWidget et al.) assumes
-	//    B -> A
-	// i.e. the specialized class is role B and the general class is role A,
-	// while the document code (i.e. UMLAssociation et al.) assumes
-	//    A -> B
-	// i.e. the general class is role B and the specialized class is role A,
-	// Right here, we have the A->B case.
-	// I hope to clean this up shortly   --okellogg 2003/12/22
 	for (UMLAssociation* a = m_AssocsList.first(); a; a = m_AssocsList.next()) {
 		if ( a->getAssocType() != Uml::at_Generalization ||
-		     a->getRoleAId() != this->getID() )
+		     a->getObjectId(Uml::A) != getID() )
 			continue;
-		UMLClassifier *c = dynamic_cast<UMLClassifier*>(a->getObjectB());
+		UMLClassifier *c = dynamic_cast<UMLClassifier*>(a->getObject(Uml::B));
 		if (c)
 			list.append(c);
 		else
-			kdDebug() << "UMLCanvasObject::getSuperClasses: generalization's"
-				  << " other end is not a UMLClassifier"
-				  << " (id= " << a->getRoleBId() << ")" << endl;
+			kdDebug() << "UMLCanvasObject::getSuperClasses(" << m_Name
+				  << "): generalization's other end is not a "
+				  << "UMLClassifier (id= " << ID2STR(a->getObjectId(Uml::B)) << ")"
+				  << endl;
 	}
 	return list;
 }
@@ -160,15 +202,15 @@ UMLClassifierList UMLCanvasObject::getSubClasses() {
 	// WARNING: See remark at getSuperClasses()
 	for (UMLAssociation* a = m_AssocsList.first(); a; a = m_AssocsList.next()) {
 		if ( a->getAssocType() != Uml::at_Generalization ||
-		     a->getRoleBId() != this->getID() )
+		     a->getObjectId(Uml::B) != getID() )
 			continue;
-		UMLClassifier *c = dynamic_cast<UMLClassifier*>(a->getObjectA());
+		UMLClassifier *c = dynamic_cast<UMLClassifier*>(a->getObject(Uml::A));
 		if (c)
 			list.append(c);
 		else
 			kdDebug() << "UMLCanvasObject::getSubClasses: specialization's"
 				  << " other end is not a UMLClassifier"
-				  << " (id=" << a->getRoleAId() << ")" << endl;
+				  << " (id=" << ID2STR(a->getObjectId(Uml::A)) << ")" << endl;
 	}
 	return list;
 }
@@ -184,4 +226,8 @@ UMLAssociationList UMLCanvasObject::getAggregations() {
 UMLAssociationList UMLCanvasObject::getCompositions() {
 	return getSpecificAssocs(Uml::at_Composition);
 }
+
+UMLAssociationList UMLCanvasObject::getRelationships() {
+	return getSpecificAssocs(Uml::at_Relationship);
+}
 #include "umlcanvasobject.moc"
