@@ -42,7 +42,7 @@ public:
 	 * @param name		The name of the Concept.
 	 * @param id		The unique id of the Concept.
 	 */
-	UMLCanvasObject(const QString & name = "", int id = -1);
+	UMLCanvasObject(const QString & name = "", Uml::IDType id = Uml::id_None);
 
 
 	/**
@@ -85,6 +85,11 @@ public:
 	 */
 	int removeAssociation(UMLAssociation *assoc);
 
+	/**
+	 * Remove all associations from the CanvasObject.
+	 */
+	void removeAllAssociations();
+
 	/**
 	 * Returns the number of associations for the CanvasObject.
 	 * This is the sum of the aggregations and compositions.
@@ -147,17 +152,21 @@ public:
 	 */
 	UMLAssociationList getCompositions();
 
+	/**
+	 * Shorthand for getSpecificAssocs(Uml::at_Relationship)
+	 *
+	 * @return	The list of relationships for the entity.
+	 */
+	UMLAssociationList getRelationships();
+
 	/**
 	 * Find a list of associations with the given name.
 	 *
 	 * @param t		The type to find.
 	 * @param n		The name of the object to find.
-	 * @param seekStereo	Set this true if we should look at the object's
-	 *			stereotype instead of its name.
 	 * @return	List of objects found (empty if none found.)
 	 */
-	virtual UMLObjectList findChildObject(UMLObject_Type t, QString n,
-					      bool seekStereo = false);
+	virtual UMLObjectList findChildObject(Uml::Object_Type t, const QString &n);
 
 	/**
 	 * Find an association.
@@ -165,7 +174,7 @@ public:
 	 * @param id		The id of the object to find.
 	 * @return	Pointer to the object found (NULL if not found.)
 	 */
-	virtual UMLObject* findChildObject(int id);
+	virtual UMLObject* findChildObject(Uml::IDType id);
 
 	/**
 	 * Returns a name for the new association, operation, template
@@ -175,9 +184,9 @@ public:
 	 * @param type		The object type for which to make a name.
 	 * @param seekStereo	Set this true if we should look at the object's
 	 *			stereotype instead of its name.
-	 * @return	Unique name string for the UMLObject_Type given.
+	 * @return	Unique name string for the Object_Type given.
 	 */
-	virtual QString uniqChildName(const UMLObject_Type type,
+	virtual QString uniqChildName(const Uml::Object_Type type,
 				      bool seekStereo = false);
 
 	// The abstract method UMLObject::saveToXMI() is implemented
