@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -10,9 +15,11 @@
 #ifndef UMLLISTVIEW_H
 #define UMLLISTVIEW_H
 
-#include "umllistviewitemlist.h"
-#include "umllistviewitem.h"
+#include <qdom.h>
+#include <qpixmap.h>
 #include <klistview.h>
+#include "umlnamespace.h"
+#include "umllistviewitemlist.h"
 
 /**
  *	This is one of the main classes used in this program.
@@ -25,21 +32,15 @@
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
  
-namespace Umbrello
-{
-	class Diagram;
-	class DiagramView;
-} 
+class QMouseEvent;
+class QKeyEvent;
 class IDChangeLog;
 class ListPopupMenu;
 class UMLClassifier;
 class UMLDoc;
 class UMLListViewItem;
 class UMLView;
-
-using Umbrello::Diagram;
-using Umbrello::DiagramView;
-
+class UMLObject;
 
 class UMLListView : public KListView {
 	Q_OBJECT
@@ -47,6 +48,8 @@ class UMLListView : public KListView {
 
 	enum Icon_Type	{
 		it_Home = 0,
+		it_Folder_Cyan,
+		it_Folder_Cyan_Open,
 		it_Folder_Green,
 		it_Folder_Green_Open,
 		it_Folder_Grey,
@@ -67,6 +70,7 @@ class UMLListView : public KListView {
 		it_Interface,
 		it_Datatype,
 		it_Enum,
+		it_Entity,
 		it_Actor,
 		it_UseCase,
 		it_Public_Method,
@@ -80,6 +84,7 @@ class UMLListView : public KListView {
 		it_Diagram_Collaboration,
 		it_Diagram_Component,
 		it_Diagram_Deployment,
+		it_Diagram_EntityRelationship,
 		it_Diagram_Sequence,
 		it_Diagram_State,
 		it_Diagram_Usecase
@@ -118,13 +123,15 @@ class UMLListView : public KListView {
 	 */
 	void setView(UMLView* v);
 
-	/**
-	 * Sets loading boolean flag to the value given.
-	 */
-	void setLoading(bool state);
-
 	bool getSelectedItems(UMLListViewItemList &ItemList);
 
+	/**
+	 * Create a listview item for an existing diagram.
+	 *
+	 * @param v		The existing diagram.
+	 */
+	UMLListViewItem* createDiagramItem(UMLView *v);
+
 	/**
 	 * CHECK - This is perhaps redundant since the
 	 *         UMLListViewItemData => UMLListViewItem merge.
@@ -180,39 +187,39 @@ class UMLListView : public KListView {
 	 * Searches through the tree for the item with the given ID.
 	 *
 	 * @param id		The ID to search for.
-	 * @return	The item with the given ID or -1 if not found.
+	 * @return	The item with the given ID or NULL if not found.
 	 */
-	UMLListViewItem * findItem(int id);
+	UMLListViewItem * findItem(Uml::IDType id);
 
 	/**
 	 * Returns true if the listview type also has a widget representation in diagrams.
 	 */
-	static bool typeIsCanvasWidget(ListView_Type type);
+	static bool typeIsCanvasWidget(Uml::ListView_Type type);
 
 	/**
 	 * Returns true if the listview type is a logical, usecase or component folder.
 	 */
-	static bool typeIsFolder(ListView_Type type);
+	static bool typeIsFolder(Uml::ListView_Type type);
 
 	/**
 	 * Returns true if the listview type is a diagram.
 	 */
-	static bool typeIsDiagram(ListView_Type type);
+	static bool typeIsDiagram(Uml::ListView_Type type);
 
 	/**
 	 * Returns true if the listview type is an attribute, operation, or template.
 	 */
-	static bool typeIsClassifierList(ListView_Type type);
+	static bool typeIsClassifierList(Uml::ListView_Type type);
 
 	/**
 	 * Creates a UMLObject out of the given list view item.
 	 */
-	void createUMLObject( UMLListViewItem * item, Uml::UMLObject_Type type );
+	void createUMLObject( UMLListViewItem * item, Uml::Object_Type type );
 
 	/**
 	 * Creates a child UMLObject out of the given list view item.
 	 */
-	void createChildUMLObject( UMLListViewItem * item, Uml::UMLObject_Type type );
+	bool createChildUMLObject( UMLListViewItem * item, Uml::Object_Type type );
 
 	/**
 	 * Creates a diagram out of the given list view item.
@@ -224,20 +231,10 @@ class UMLListView : public KListView {
 	 */
 	QString getUniqueDiagramName( Uml::Diagram_Type type );
 
-	/**
-	 * Returns a unique name for a child umlobject.
-	 */
-	QString getUniqueChildUMLObjectName( UMLClassifier * parent, Uml::UMLObject_Type type );
-
-	/**
-	 * Returns a unique name for an object.
-	 */
-	QString getUniqueUMLObjectName( Uml::UMLObject_Type type );
-
 	/**
 	 * Returns if the given name is unique for the given items type.
 	 */
-	bool isUnique( UMLListViewItem * item, QString name );
+	bool isUnique( UMLListViewItem * item, const QString &name );
 
 	/**
 	 * Cancel rename event has occurred for the given item.
@@ -266,18 +263,21 @@ class UMLListView : public KListView {
 	/**
 	 * Converts an object type enum to the equivalent list view type
 	 */
-	static Uml::ListView_Type convert_OT_LVT(Uml::UMLObject_Type ot);
+	static Uml::ListView_Type convert_OT_LVT(Uml::Object_Type ot);
 
 	/**
-	 * Moves an object given is unique ID and listview type into an
+	 * Moves an object given is unique ID and listview type to an
 	 * other listview parent item.
 	 * Also takes care of the corresponding move in the model.
 	 */
-	UMLListViewItem * moveObject(int srcId, Uml::ListView_Type srcType,
+	UMLListViewItem * moveObject(Uml::IDType srcId, Uml::ListView_Type srcType,
 				     UMLListViewItem *newParent);
 
 	void closeDatatypesFolder();
 
+	UMLListViewItem *theRootView() { return m_rv; }
+	UMLListViewItem *theLogicalView() { return m_lv; }
+
 	void saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
 
 	bool loadFromXMI( QDomElement & element );
@@ -285,17 +285,18 @@ class UMLListView : public KListView {
 	bool loadChildrenFromXMI( UMLListViewItem * parent, QDomElement & element );
 
  protected:
-	UMLListViewItem* rv;//root view
-	UMLListViewItem* ucv;//use case view item
-	UMLListViewItem* lv;//logical view item
+	UMLListViewItem* m_rv;//root view
+	UMLListViewItem* m_ucv;//use case view item
+	UMLListViewItem* m_lv;//logical view item
 	UMLListViewItem* componentView;//component view item
 	UMLListViewItem* deploymentView;//deployment view item
+	UMLListViewItem* entityRelationshipModel;//entity relationship view item
 	UMLListViewItem* datatypeFolder;
 	UMLListViewItem* diagramFolder;
 	ListPopupMenu * m_pMenu;
 	QString oldText, message;
 	UMLDoc *m_doc;
-	bool loading, m_bStartedCut, m_bStartedCopy, m_bIgnoreCancelRename;
+	bool m_bStartedCut, m_bStartedCopy, m_bIgnoreCancelRename;
 
 	/**
 	 * Used when creating an attribute or an operation to stop it adding a second listViewItem
@@ -304,6 +305,8 @@ class UMLListView : public KListView {
 
 	struct Pixmaps {
 		QPixmap Home;
+		QPixmap Folder_Cyan;
+		QPixmap Folder_Cyan_Open;
 		QPixmap Folder_Green;
 		QPixmap Folder_Green_Open;
 		QPixmap Folder_Grey;
@@ -324,6 +327,7 @@ class UMLListView : public KListView {
 		QPixmap Interface;
 		QPixmap Datatype;
 		QPixmap Enum;
+		QPixmap Entity;
 		QPixmap Actor;
 		QPixmap UseCase;
 		QPixmap Public_Method;
@@ -337,6 +341,7 @@ class UMLListView : public KListView {
 		QPixmap Diagram_Collaboration;
 		QPixmap Diagram_Component;
 		QPixmap Diagram_Deployment;
+		QPixmap Diagram_EntityRelationship;
 		QPixmap Diagram_Sequence;
 		QPixmap Diagram_State;
 		QPixmap Diagram_Usecase;
@@ -351,11 +356,12 @@ class UMLListView : public KListView {
 	void startDrag();
 	void tidyChangeItem();
 	bool acceptDrag (QDropEvent* event) const;
+	void keyPressEvent(QKeyEvent *);
 
 	/**
 	 * This methods looks for a object in a folder an its subfolders recursive.
 	 * @param item The folder entry of the list view.
-	 * @param o The object to be find in the folder.
+	 * @param o The object to be found in the folder.
 	 *
 	 * @return The object if found else a NULL pointer.
 	 */
@@ -369,7 +375,7 @@ class UMLListView : public KListView {
 	UMLListViewItem * findView(UMLView *v);
 
 	/**
-	 * Converts a diagram type enum to the equivalent list view type
+	 * Utility: Converts a diagram type enum to the equivalent list view type
 	 */
 	static Uml::ListView_Type convert_DT_LVT(Uml::Diagram_Type dt);
 
@@ -377,10 +383,10 @@ class UMLListView : public KListView {
 	 * Converts a list view type enum to the equivalent object type.
 	 *
 	 * @param lvt		The ListView_Type to convert.
-	 * @return	The converted UMLObject_Type if the listview type
-	 *		has a UMLObject_Type representation, else 0.
+	 * @return	The converted Object_Type if the listview type
+	 *		has a Object_Type representation, else 0.
 	 */
-	static Uml::UMLObject_Type convert_LVT_OT(Uml::ListView_Type lvt);
+	static Uml::Object_Type convert_LVT_OT(Uml::ListView_Type lvt);
 
 	/**
 	 * 	Loads the pixmaps to use in the list items.
@@ -398,19 +404,13 @@ public slots:
 	 * Creates a new item to represent a new diagram
 	 * @param id the id of the new diagram
 	 */
-	void slotDiagramCreated(int id);
-
-	/**
-	 * Creates a new item to represent a new diagram
-	 * @param d pointer to the new diagram
-	 */
-	void diagramCreated(Umbrello::Diagram* d);
+	void slotDiagramCreated(Uml::IDType id);
 
 	/**
 	 * renames a diagram in the list view
 	 * @param the id of the renamed diagram
 	 */
-	void slotDiagramRenamed(int id);
+	void slotDiagramRenamed(Uml::IDType id);
 
 	/**
 	 * Creates a new list view item and connects the appropriate signals/slots
@@ -459,7 +459,7 @@ public slots:
 	 * removes the item representing a diagram
 	 * @param id the id of the diagram
 	 */
-	void slotDiagramRemoved(int id);
+	void slotDiagramRemoved(Uml::IDType id);
 
 	/**
 	 * Called when a right mouse button menu has an item selected
@@ -509,14 +509,15 @@ public slots:
 	/**
 	 * change the current view
 	 */
-	void diagramSelected(int);
+	void diagramSelected(Uml::IDType);
 
  private:
 	/**
 	 * Searches the tree for a diagram (view).
 	 * Used by findView().
 	 */
-	UMLListViewItem* recursiveSearchForView(UMLListViewItem* folder, ListView_Type type, int id);
+	UMLListViewItem* recursiveSearchForView(UMLListViewItem* folder,
+						Uml::ListView_Type type, Uml::IDType id);
 
 };
 
